/*
 * HVBaseDevice.java
 *
 * Created on December 20, 2004, 2:47 PM
 */

package org.cdp1802.hvision;

import java.util.Collection;
import org.cdp1802.xpl.MutableNamedValuesI;
import org.cdp1802.xpl.NamedValueI;
import org.cdp1802.xpl.NamedValuesI;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MutableMessageI;
import org.cdp1802.xpl.xPL_Util;

/**
 * Provides basic/common device services for extension
 *
 * @author gduprey
 */
public abstract class HVBaseDevice {
  protected xPL_MutableMessageI deviceMessage = null;
  protected String ignoreAttrBodyNames[] = null;
  protected MutableNamedValuesI deviceAttrList = xPL_Util.createMutableNamedValues();

  public xPL_DeviceI getDevice() {
    return HVManager.getManager().getXPLDevice();
  }
  
  protected abstract void prepDeviceMessage(String msgItemName, String msgItemValue);
  
  public boolean hasAttr(String theAttrName) {
    return deviceAttrList.hasNamedValue(theAttrName);
  }

  public int getAttrCount() {
    return deviceAttrList.getNamedValuesCount();
  }
  
  public Collection<NamedValueI> getAllAttrs() {
    return deviceAttrList.getAllNamedValues();
  }
  
  public String getAttr(String theAttrName) {
    return deviceAttrList.getNamedValue(theAttrName);
  }
  public int getIntAttr(String theAttrName) {
    return deviceAttrList.getIntNamedValue(theAttrName);
  }
  public boolean getBoolAttr(String theAttrName) {
    return deviceAttrList.getBoolNamedValue(theAttrName);
  }
  
  public void setAttr(String theAttrName, String theAttrValue) {
    synchronized(deviceMessage) {
      prepDeviceMessage("COMMAND", "SET_ATTR");
      deviceMessage.addNamedValue(theAttrName, theAttrValue);
      getDevice().sendMessage(deviceMessage);
    }
  }
  
  public void setAttr(String theAttrName, int theAttrValue) {
    setAttr(theAttrName, String.valueOf(theAttrValue));
  }
  
  public void setAttr(String theAttrName, boolean theAttrValue) {
    setAttr(theAttrName, (theAttrValue ? "1" : "0"));
  }
  
  protected boolean installAttrs(NamedValuesI messageBody) {
    String attrName = null, attrValue = null;
    boolean attrChanged = false;
    boolean ignorableFound = false;
    int ignoreIndex = 0;
    
    for (NamedValueI theAttr: messageBody.getAllNamedValues()) {
      attrName = theAttr.getName();
      
      // Skip over non-attr names
      ignorableFound = false;
      for (ignoreIndex = 0; ignoreIndex < ignoreAttrBodyNames.length; ignoreIndex++) {
        if (attrName.equalsIgnoreCase(ignoreAttrBodyNames[ignoreIndex])) {
          ignorableFound = true;
          break;
        }
      }
      if (ignorableFound) continue;
      
      // Lookup the named value and see if it exists
      if (deviceAttrList.hasNamedValue(attrName)) {
        // If no change, skip this
        if (deviceAttrList.getNamedValue(attrName).equals(theAttr.getValue())) continue;
        
        // Set the value
        deviceAttrList.removeNamedValues(attrName);
        deviceAttrList.addNamedValue(attrName, theAttr.getValue());
        attrChanged = true;
        continue;
      }
      
      // Add as new attr
      deviceAttrList.addNamedValue(attrName, theAttr.getValue());
      attrChanged = true;
    }
    
    return attrChanged;
  }
}
