/*
 * HVManager.java
 *
 * Created on December 20, 2004, 10:18 AM
 */

package org.cdp1802.hvision;

import java.util.ArrayList;
import org.cdp1802.hvision.dinput.*;
import org.cdp1802.hvision.doutput.*;
import org.cdp1802.hvision.flag.*;
import org.cdp1802.hvision.ir.*;
import org.cdp1802.hvision.macro.*;
import org.cdp1802.hvision.phone.*;
import org.cdp1802.hvision.variable.*;
import org.cdp1802.hvision.x10.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_Manager;

/**
 *
 * @author gduprey
 */
public class HVManager {
  private static HVManager theManager = null;
  public static HVManager getManager() {
    if (theManager == null) theManager = new HVManager();
    return theManager; 
  }
  
  class ConfigurationRequestor implements Runnable {
    ConfigurationRequestor() {
      new Thread(this).start();
    }
    
    public void run() {
      for(HVDeviceManagerI theDevice: deviceList) {
        theDevice.requestConfiguration();
        
        // Sleep a bit
        try {
          Thread.currentThread().sleep(1000);
        } catch (Throwable anyError) { }
      }
    }
  }
  
  boolean managerStarted = false;
  xPL_DeviceI hvDevice = null;
  HVX10Manager x10Manager = null;
  HVVariableManager variableManager = null;
  HVFlagManager flagManager = null;
  HVDInputManager dinputManager = null;
  HVDOutputManager doutputManager = null;
  HVMacroManager macroManager = null;
  HVIRManager irManager = null;
  HVPhoneManager phoneManager = null;
  ArrayList<HVDeviceManagerI> deviceList = new ArrayList<HVDeviceManagerI>();
  
  private HVManager() {
    // Setup various managers
    deviceList.add(x10Manager = new HVX10Manager());
    deviceList.add(variableManager = new HVVariableManager());
    deviceList.add(flagManager = new HVFlagManager());
    deviceList.add(dinputManager = new HVDInputManager());
    deviceList.add(doutputManager = new HVDOutputManager());
    deviceList.add(macroManager = new HVMacroManager());
    deviceList.add(irManager = new HVIRManager());
    deviceList.add(phoneManager = new HVPhoneManager());
  }
  
  /**
   * You must start the manager before you can use it.  It's generally best to start
   * the manager after you know all hubs are running.
   *
   * If you have not installed a device to use, the manager will create
   * one when started.  Once started, the device cannot be changed
   */
  public void startManager() {
    if (managerStarted) return;
    
    // Create an xPL device for us
    if (hvDevice == null) {
      hvDevice = xPL_Manager.getManager().getDeviceManager().createDevice("cdp1802", "hvclient", null);
      hvDevice.setEnabled(true);
    }
    
    // And we are done
    managerStarted = true;
    
    // Start each device manager
    for (HVDeviceManagerI theManager: deviceList) {
      theManager.startManager();
    }
    
    // Make a request for each device configuration
    new ConfigurationRequestor();
  }
  
  public boolean isInitialized() { return managerStarted; }
  
  public xPL_DeviceI getXPLDevice() { return hvDevice; }
  public void setXPLDevice(xPL_DeviceI theDevice) {
    if (managerStarted) return;
    hvDevice = theDevice;
  }
  
  public HVX10ManagerI getX10Manager() { return x10Manager; }
  public HVVariableManagerI getVariableManager() { return variableManager; }
  public HVFlagManagerI getFlagManager() { return flagManager; }
  public HVDInputManagerI getDInputManager() { return dinputManager; }
  public HVDOutputManagerI getDOutputManager() { return doutputManager; }
  public HVMacroManagerI getMacroManager() { return macroManager; }
  public HVIRManagerI getIRManager() { return irManager; }
  public HVPhoneManagerI getPhoneManager() { return phoneManager; }
  
  /**
   * This is a convience method to to remove all instances of a listener
   * from all the various HomeVision managers.  It's smart in that it'll
   * only apply to those managers the passed reference is compatible 
   * with.  It's an easy way to cleanup at shutdown
   *
   * @param theListener listener object to remove
   */
  public void removeAllListeners(Object theListener) {
    if (theListener instanceof HVDInputListenerI) dinputManager.removeAllListeners((HVDInputListenerI) theListener);
    if (theListener instanceof HVDOutputListenerI) doutputManager.removeAllListeners((HVDOutputListenerI) theListener);
    if (theListener instanceof HVFlagListenerI) flagManager.removeAllListeners((HVFlagListenerI) theListener);
    if (theListener instanceof HVVariableListenerI) variableManager.removeAllListeners((HVVariableListenerI) theListener);
    if (theListener instanceof HVIRListenerI) irManager.removeAllListeners((HVIRListenerI) theListener);
    if (theListener instanceof HVMacroListenerI) macroManager.removeAllListeners((HVMacroListenerI) theListener);
    if (theListener instanceof HVPhoneListenerI) phoneManager.removeListener((HVPhoneListenerI) theListener);
    if (theListener instanceof HVX10ListenerI) x10Manager.removeAllListeners((HVX10ListenerI) theListener);
  }
}
