/*
 * HVDInput.java
 *
 * Created on December 20, 2004, 11:31 AM
 */

package org.cdp1802.hvision.dinput;

import java.util.Collection;
import org.cdp1802.hvision.HVBaseDevice;
import org.cdp1802.xpl.NamedValueI;
import org.cdp1802.xpl.NamedValuesI;
import org.cdp1802.xpl.xPL_MessageI;


/**
 *
 * @author gduprey
 */
public class HVDInput extends HVBaseDevice implements HVDInputI {
  HVDInputManager dinputManager = null;
  int dinputNum = -1;
  boolean curState = false;
  
  HVDInput(HVDInputManagerI theManager, int dinputNum) {
    this.dinputManager = (HVDInputManager) theManager;
    this.dinputNum = dinputNum;
    this.ignoreAttrBodyNames = new String[] { "TYPE", "DEVICE", "CURRENT" };
  }
  
  protected void prepDeviceMessage(String msgItemName, String msgItemValue) {
    if (deviceMessage == null)     deviceMessage = getDevice().createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    deviceMessage.clearMessageBody();
    deviceMessage.addNamedValue("DEVICE", dinputNum);
    deviceMessage.addNamedValue("TYPE", "INPUT");
    deviceMessage.addNamedValue(msgItemName, msgItemValue);
  }
  
  public int getDInputNum() { return dinputNum; }
  
  public boolean isDInputHigh() { return curState; }

  public void addListener(HVDInputListenerI theListener) {
    dinputManager.addListener(theListener, this);
  }
  
  public void removeListener(HVDInputListenerI theListener) {
    dinputManager.removeListener(theListener, this);
  }

  void handleDInputXPLEvent(xPL_MessageI theMessage) {
    // Get value
    String newValue = theMessage.getNamedValue("CURRENT");
    if ((newValue == null) || (newValue.length() == 0)) return;
    boolean newState = newValue.equalsIgnoreCase("HIGH");
    
    if (newState != curState) {
      curState = newState;
      dinputManager.fireDInputChangedEvent(this, HVDInputEventI.EventReason.STATE_CHANGED);
    }
    
    // Install Attr Values
    if (installAttrs(theMessage.getMessageBody())) {
      dinputManager.fireDInputChangedEvent(this, HVDInputEventI.EventReason.ATTR_CHANGED);
    }
  }
  
  public String toString() {
    return String.valueOf(dinputNum) + "=" + (curState ? "HIGH" : "LOW");
  }
}
