/*
 * HVDInputManager.java
 *
 * Created on December 20, 2004, 10:56 AM
 */

package org.cdp1802.hvision.dinput;

import java.util.ArrayList;
import java.util.Iterator;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;
import org.cdp1802.hvision.*;

/**
 *
 * @author gduprey
 */
public class HVDInputManager implements HVDInputManagerI, xPL_DeviceMessageListenerI {
  class DInputListener {
    HVDInputI listenForDInput = null;
    HVDInputListenerI theListener = null;
    
    DInputListener(HVDInputListenerI theListener) {
      this.theListener = theListener;
    }
    DInputListener(HVDInputI forDInput, HVDInputListenerI theListener) {
      this.listenForDInput = forDInput;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return (listenForDInput == null); }
    boolean isDInputListener() { return listenForDInput != null; }
    
    boolean isInterested(HVDInputI theDInput) {
      if ((listenForDInput != null) && (listenForDInput != theDInput)) return false;
      return true;
    }
    
    HVDInputListenerI getListener() { return theListener; }
    HVDInputI getDInput() { return listenForDInput; }
  }
  
  xPL_DeviceI theXPLDevice = null;
  HVDInputI dinputList[] = new HVDInputI[256];
  ArrayList<DInputListener> listenerList = new ArrayList<DInputListener>();
  
  public HVDInputManager() { }
  
  public void startManager() {
    this.theXPLDevice = HVManager.getManager().getXPLDevice();
    theXPLDevice.addMessageListener(this);
  }

  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = theXPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    configMessage.addNamedValue("TYPE", "INPUT");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    theXPLDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public HVDInputI getDInput(int dinputNum) {
    // Sanity check
    if ((dinputNum < 0) || (dinputNum > 255)) return null;
    
    synchronized(dinputList) {
      if (dinputList[dinputNum] != null) return dinputList[dinputNum];
      
      return (dinputList[dinputNum] = new HVDInput(this, dinputNum));
    }
  }
  
  public void addListener(HVDInputListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new DInputListener(theListener));
    }
  }
  
  public void addListener(HVDInputListenerI theListener, HVDInputI forDInput) {
    synchronized(listenerList) {
      listenerList.add(new DInputListener(forDInput, theListener));
    }
  }
  
  public void addListener(HVDInputListenerI theListener, int forDInputNum) {
    HVDInputI forDInput = getDInput(forDInputNum);
    if (forDInput != null) addListener(theListener, forDInput);
  }

  public void removeListener(HVDInputListenerI theListener) {
    synchronized(listenerList) {
      DInputListener dinputListener = null;
      Iterator<DInputListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        dinputListener = theListeners.next();

        if (dinputListener.theListener != theListener) continue;
        if (!dinputListener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVDInputListenerI theListener, HVDInputI forDInput) {
    synchronized(listenerList) {
      DInputListener dinputListener = null;
      Iterator<DInputListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        dinputListener = theListeners.next();

        if (dinputListener.theListener != theListener) continue;
        if (dinputListener.listenForDInput != forDInput) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVDInputListenerI theListener, int forDInputNum) {
    HVDInputI forDInput = getDInput(forDInputNum);
    if (forDInput != null) removeListener(theListener, forDInput);
  }
  
  public void removeAllListeners(HVDInputListenerI theListener) {
    synchronized(listenerList) {
      DInputListener dinputListener = null;
      Iterator<DInputListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        dinputListener = theListeners.next();

        if (dinputListener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireDInputChangedEvent(HVDInputI theDInput, HVDInputEventI.EventReason theReason) {
    HVDInputEvent theEvent = new HVDInputEvent(theDInput, theReason);
    for (DInputListener theListener: listenerList) {
      if (theListener.isInterested(theDInput)) try {
        theListener.getListener().handleDInputMessage(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    // Make sure it's from this device
    if (this.theXPLDevice != theXPLDevice) return;
   
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("sensor")) return;
    
    // Get the device type and #
    String theType = theMessage.getNamedValue("TYPE");
    int theDevNum = theMessage.getIntNamedValue("DEVICE", -1);

    // Bail out if missing anything important
    if ((theType == null) || !theType.equalsIgnoreCase("INPUT") || (theDevNum < 0) || (theDevNum > 255)) return;

    // Dispatch it
    ((HVDInput) getDInput(theDevNum)).handleDInputXPLEvent(theMessage);
  }
}
