/*
 * HVDOutput.java
 *
 * Created on December 20, 2004, 11:31 AM
 */

package org.cdp1802.hvision.doutput;

import java.util.Collection;
import org.cdp1802.hvision.HVBaseDevice;
import org.cdp1802.xpl.NamedValueI;
import org.cdp1802.xpl.NamedValuesI;
import org.cdp1802.xpl.xPL_MessageI;


/**
 *
 * @author gduprey
 */
public class HVDOutput extends HVBaseDevice implements HVDOutputI {
  HVDOutputManager doutputManager = null;
  int doutputNum = -1;
  boolean curState = false;
  
  HVDOutput(HVDOutputManagerI theManager, int doutputNum) {
    this.doutputManager = (HVDOutputManager) theManager;
    this.doutputNum = doutputNum;
    
    this.ignoreAttrBodyNames = new String[] { "TYPE", "DEVICE", "CURRENT" };
  }

  protected void prepDeviceMessage(String msgItemName, String msgItemValue) {
    if (deviceMessage == null)  deviceMessage = getDevice().createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");

    deviceMessage.clearMessageBody();
    deviceMessage.addNamedValue("DEVICE", doutputNum);
    deviceMessage.addNamedValue("TYPE", "OUTPUT");
    deviceMessage.addNamedValue(msgItemName, msgItemValue);
  }
  
  public int getDOutputNum() { return doutputNum; }
  
  public boolean isDOutputHigh() { return curState; }
  public void setDOutputHigh(boolean setHigh) {
    if (setHigh == curState) return;
            
    // Send value out
    synchronized(this) {
      prepDeviceMessage("CURRENT", (setHigh ? "HIGH" : "LOW"));
      getDevice().sendMessage(deviceMessage);
    }
  }

  public void pulseHigh(int pulseInSeconds) {
    // Send value out
    synchronized(this) {
      prepDeviceMessage("CURRENT", "PULSE_HIGH");
      deviceMessage.addNamedValue("data1", pulseInSeconds);
      getDevice().sendMessage(deviceMessage);
    }
  }
  
  public void pulseLow(int pulseInSeconds) {
    // Send value out
    synchronized(this) {
      prepDeviceMessage("CURRENT", "PULSE_LOW");
      deviceMessage.addNamedValue("data1", pulseInSeconds);
      getDevice().sendMessage(deviceMessage);
    }
  }
  
  
  
  public void addListener(HVDOutputListenerI theListener) {
    doutputManager.addListener(theListener, this);
  }
  
  public void removeListener(HVDOutputListenerI theListener) {
    doutputManager.removeListener(theListener, this);
  }

  void handleDOutputXPLEvent(xPL_MessageI theMessage) {
    // Get value
    String newValue = theMessage.getNamedValue("CURRENT");
    if ((newValue == null) || (newValue.length() == 0)) return;
    boolean newState = newValue.equalsIgnoreCase("HIGH");
    
    if (newState != curState) {
      curState = newState;
      doutputManager.fireDOutputChangedEvent(this, HVDOutputEventI.EventReason.STATE_CHANGED);
    }
    
    // Install Attr Values
    if (installAttrs(theMessage.getMessageBody())) {
      doutputManager.fireDOutputChangedEvent(this, HVDOutputEventI.EventReason.ATTR_CHANGED);
    }
  }
  
  public String toString() {
    return String.valueOf(doutputNum) + "=" + (curState ? "HIGH" : "LOW");
  }
}
