/*
 * HVDOutputManager.java
 *
 * Created on December 20, 2004, 10:56 AM
 */

package org.cdp1802.hvision.doutput;

import java.util.ArrayList;
import java.util.Iterator;

import org.cdp1802.hvision.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gduprey
 */
public class HVDOutputManager implements HVDOutputManagerI, xPL_DeviceMessageListenerI {
  class DOutputListener {
    HVDOutputI listenForDOutput = null;
    HVDOutputListenerI theListener = null;
    
    DOutputListener(HVDOutputListenerI theListener) {
      this.theListener = theListener;
    }
    DOutputListener(HVDOutputI forDOutput, HVDOutputListenerI theListener) {
      this.listenForDOutput = forDOutput;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return (listenForDOutput == null); }
    boolean isDOutputListener() { return listenForDOutput != null; }
    
    boolean isInterested(HVDOutputI theDOutput) {
      if ((listenForDOutput != null) && (listenForDOutput != theDOutput)) return false;
      return true;
    }
    
    HVDOutputListenerI getListener() { return theListener; }
    HVDOutputI getDOutput() { return listenForDOutput; }
  }
  
  xPL_DeviceI theXPLDevice = null;
  HVDOutputI doutputList[] = new HVDOutputI[256];
  ArrayList<DOutputListener> listenerList = new ArrayList<DOutputListener>();
  
  public HVDOutputManager() { }

  public void startManager() {
    this.theXPLDevice = HVManager.getManager().getXPLDevice();
    theXPLDevice.addMessageListener(this);
  }

  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = theXPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    configMessage.addNamedValue("TYPE", "OUTPUT");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    theXPLDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public HVDOutputI getDOutput(int doutputNum) {
    // Sanity check
    if ((doutputNum < 0) || (doutputNum > 255)) return null;
    
    synchronized(doutputList) {
      if (doutputList[doutputNum] != null) return doutputList[doutputNum];
      
      return (doutputList[doutputNum] = new HVDOutput(this, doutputNum));
    }
  }
  
  public void addListener(HVDOutputListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new DOutputListener(theListener));
    }
  }
  
  public void addListener(HVDOutputListenerI theListener, HVDOutputI forDOutput) {
    synchronized(listenerList) {
      listenerList.add(new DOutputListener(forDOutput, theListener));
    }
  }
  
  public void addListener(HVDOutputListenerI theListener, int forDOutputNum) {
    HVDOutputI forDOutput = getDOutput(forDOutputNum);
    if (forDOutput != null) addListener(theListener, forDOutput);
  }

  public void removeListener(HVDOutputListenerI theListener) {
    synchronized(listenerList) {
      DOutputListener doutputListener = null;
      Iterator<DOutputListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        doutputListener = theListeners.next();

        if (doutputListener.theListener != theListener) continue;
        if (!doutputListener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVDOutputListenerI theListener, HVDOutputI forDOutput) {
    synchronized(listenerList) {
      DOutputListener doutputListener = null;
      Iterator<DOutputListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        doutputListener = theListeners.next();

        if (doutputListener.theListener != theListener) continue;
        if (doutputListener.listenForDOutput != forDOutput) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVDOutputListenerI theListener, int forDOutputNum) {
    HVDOutputI forDOutput = getDOutput(forDOutputNum);
    if (forDOutput != null) removeListener(theListener, forDOutput);
  }
  
  public void removeAllListeners(HVDOutputListenerI theListener) {
    synchronized(listenerList) {
      DOutputListener doutputListener = null;
      Iterator<DOutputListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        doutputListener = theListeners.next();

        if (doutputListener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireDOutputChangedEvent(HVDOutputI theDOutput, HVDOutputEventI.EventReason theReason) {
    HVDOutputEvent theEvent = new HVDOutputEvent(theDOutput, theReason);
    for (DOutputListener theListener: listenerList) {
      if (theListener.isInterested(theDOutput)) try {
        theListener.getListener().handleDOutputMessage(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    // Make sure it's from this device
    if (this.theXPLDevice != theXPLDevice) return;
   
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("sensor")) return;
    
    // Get the device type and #
    String theType = theMessage.getNamedValue("TYPE");
    int theDevNum = theMessage.getIntNamedValue("DEVICE", -1);

    // Bail out if missing anything important
    if ((theType == null) || !theType.equalsIgnoreCase("OUTPUT") || (theDevNum < 0) || (theDevNum > 255)) return;

    // Dispatch it
    ((HVDOutput) getDOutput(theDevNum)).handleDOutputXPLEvent(theMessage);
  }
}
