/*
 * HVFlag.java
 *
 * Created on December 20, 2004, 11:31 AM
 */

package org.cdp1802.hvision.flag;

import java.util.Collection;
import org.cdp1802.hvision.HVBaseDevice;
import org.cdp1802.xpl.NamedValueI;
import org.cdp1802.xpl.NamedValuesI;
import org.cdp1802.xpl.xPL_MessageI;


/**
 *
 * @author gduprey
 */
public class HVFlag extends HVBaseDevice implements HVFlagI {
  HVFlagManager flagManager = null;
  int flagNum = -1;
  boolean curState = false;
  
  HVFlag(HVFlagManagerI theManager, int flagNum) {
    this.flagManager = (HVFlagManager) theManager;
    this.flagNum = flagNum;
    this.ignoreAttrBodyNames = new String[] { "TYPE", "DEVICE", "CURRENT" };
  }

  protected void prepDeviceMessage(String msgItemName, String msgItemValue) {
    if (deviceMessage == null) deviceMessage = getDevice().createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    deviceMessage.clearMessageBody();
    deviceMessage.addNamedValue("DEVICE", flagNum);
    deviceMessage.addNamedValue("TYPE", "FLAG");
    deviceMessage.addNamedValue(msgItemName, msgItemValue);
  }
  
  public int getFlagNum() { return flagNum; }
  
  public boolean isFlagSet() { return curState; }
  public void setFlagSet(boolean flagSet) {
    if (flagSet == curState) return;
            
    // Send value out
    synchronized(this) {
      prepDeviceMessage("CURRENT", (flagSet ? "SET" : "CLEAR"));
      getDevice().sendMessage(deviceMessage);
    }
  }

  public void addListener(HVFlagListenerI theListener) {
    flagManager.addListener(theListener, this);
  }
  
  public void removeListener(HVFlagListenerI theListener) {
    flagManager.removeListener(theListener, this);
  }

  void handleFlagXPLEvent(xPL_MessageI theMessage) {
    // Get value
    String newValue = theMessage.getNamedValue("CURRENT");
    if ((newValue == null) || (newValue.length() == 0)) return;
    boolean newState = newValue.equalsIgnoreCase("SET");
    
    if (newState != curState) {
      curState = newState;
      flagManager.fireFlagChangedEvent(this, HVFlagEventI.EventReason.STATE_CHANGED);
    }
    
    // Install Attr Values
    if (installAttrs(theMessage.getMessageBody())) {
      flagManager.fireFlagChangedEvent(this, HVFlagEventI.EventReason.ATTR_CHANGED);
    }
  }
  
  public String toString() {
    return String.valueOf(flagNum) + "=" + (curState ? "SET" : "CLEAR");
  }
}
