/*
 * HVFlagManager.java
 *
 * Created on December 20, 2004, 10:56 AM
 */

package org.cdp1802.hvision.flag;

import java.util.ArrayList;
import java.util.Iterator;

import org.cdp1802.hvision.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gduprey
 */
public class HVFlagManager implements HVFlagManagerI, xPL_DeviceMessageListenerI {
  class FlagListener {
    HVFlagI listenForFlag = null;
    HVFlagListenerI theListener = null;
    
    FlagListener(HVFlagListenerI theListener) {
      this.theListener = theListener;
    }
    FlagListener(HVFlagI forFlag, HVFlagListenerI theListener) {
      this.listenForFlag = forFlag;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return (listenForFlag == null); }
    boolean isFlagListener() { return listenForFlag != null; }
    
    boolean isInterested(HVFlagI theFlag) {
      if ((listenForFlag != null) && (listenForFlag != theFlag)) return false;
      return true;
    }
    
    HVFlagListenerI getListener() { return theListener; }
    HVFlagI getFlag() { return listenForFlag; }
  }
  
  xPL_DeviceI theXPLDevice = null;
  HVFlagI flagList[] = new HVFlagI[256];
  ArrayList<FlagListener> listenerList = new ArrayList<FlagListener>();
  
  public HVFlagManager() { }
  
  public void startManager() {
    this.theXPLDevice = HVManager.getManager().getXPLDevice();
    theXPLDevice.addMessageListener(this);
  }

  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = theXPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    configMessage.addNamedValue("TYPE", "FLAG");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    theXPLDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public HVFlagI getFlag(int flagNum) {
    // Sanity check
    if ((flagNum < 0) || (flagNum > 255)) return null;
    
    synchronized(flagList) {
      if (flagList[flagNum] != null) return flagList[flagNum];
      
      return (flagList[flagNum] = new HVFlag(this, flagNum));
    }
  }
  
  public void addListener(HVFlagListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new FlagListener(theListener));
    }
  }
  
  public void addListener(HVFlagListenerI theListener, HVFlagI forFlag) {
    synchronized(listenerList) {
      listenerList.add(new FlagListener(forFlag, theListener));
    }
  }
  
  public void addListener(HVFlagListenerI theListener, int forFlagNum) {
    HVFlagI forFlag = getFlag(forFlagNum);
    if (forFlag != null) addListener(theListener, forFlag);
  }

  public void removeListener(HVFlagListenerI theListener) {
    synchronized(listenerList) {
      FlagListener flagListener = null;
      Iterator<FlagListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        flagListener = theListeners.next();

        if (flagListener.theListener != theListener) continue;
        if (!flagListener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVFlagListenerI theListener, HVFlagI forFlag) {
    synchronized(listenerList) {
      FlagListener flagListener = null;
      Iterator<FlagListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        flagListener = theListeners.next();

        if (flagListener.theListener != theListener) continue;
        if (flagListener.listenForFlag != forFlag) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVFlagListenerI theListener, int forFlagNum) {
    HVFlagI forFlag = getFlag(forFlagNum);
    if (forFlag != null) removeListener(theListener, forFlag);
  }
  
  public void removeAllListeners(HVFlagListenerI theListener) {
    synchronized(listenerList) {
      FlagListener flagListener = null;
      Iterator<FlagListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        flagListener = theListeners.next();

        if (flagListener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireFlagChangedEvent(HVFlagI theFlag, HVFlagEventI.EventReason theReason) {
    HVFlagEvent theEvent = new HVFlagEvent(theFlag, theReason);
    for (FlagListener theListener: listenerList) {
      if (theListener.isInterested(theFlag)) try {
        theListener.getListener().handleFlagMessage(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    // Make sure it's from this device
    if (this.theXPLDevice != theXPLDevice) return;
   
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("sensor")) return;
    
    // Get the device type and #
    String theType = theMessage.getNamedValue("TYPE");
    int theDevNum = theMessage.getIntNamedValue("DEVICE", -1);

    // Bail out if missing anything important
    if ((theType == null) || !theType.equalsIgnoreCase("FLAG") || (theDevNum < 0) || (theDevNum > 255)) return;

    // Dispatch it
    ((HVFlag) getFlag(theDevNum)).handleFlagXPLEvent(theMessage);
  }
}
