/*
 * HVIR.java
 *
 * Created on December 20, 2004, 11:31 AM
 */

package org.cdp1802.hvision.ir;

import java.util.Collection;
import org.cdp1802.hvision.HVBaseDevice;
import org.cdp1802.xpl.NamedValueI;
import org.cdp1802.xpl.NamedValuesI;
import org.cdp1802.xpl.xPL_MessageI;


/**
 *
 * @author gduprey
 */
public class HVIR extends HVBaseDevice implements HVIRI {
  HVIRManager irManager = null;
  int irNum = -1;
  boolean curState = false;
  
  HVIR(HVIRManagerI theManager, int irNum) {
    this.irManager = (HVIRManager) theManager;
    this.irNum = irNum;
    
    this.ignoreAttrBodyNames = new String[] { "TYPE", "DEVICE", "CURRENT" };
  }

  protected void prepDeviceMessage(String msgItemName, String msgItemValue) {
    if (deviceMessage == null) deviceMessage = getDevice().createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    deviceMessage.clearMessageBody();
    deviceMessage.addNamedValue("DEVICE", irNum);
    deviceMessage.addNamedValue("TYPE", "INFRA RED");
    deviceMessage.addNamedValue(msgItemName, msgItemValue);
  }
  
  public int getIRNum() { return irNum; }
  
  public void selectZones(int theZones[]) {
    int zoneMap = 0;
    
    for (int zoneIndex = 0; zoneIndex < theZones.length; zoneIndex++) {
      if ((theZones[zoneIndex] < 1) || (theZones[zoneIndex] > 8)) continue;
      zoneMap |= (1 << (theZones[zoneIndex] - 1));
    }
    
    synchronized(this) {
      prepDeviceMessage("CURRENT", "ZONE");
      deviceMessage.removeNamedValues("DEVICE");
      deviceMessage.addNamedValue("DEVICE", zoneMap);
      getDevice().sendMessage(deviceMessage);
    }
  }
  
  public void selectZone(int theZone) {
    selectZones(new int[] { theZone });
  }
    
  public void selectAllZones() {
    selectZones(new int[] { 1, 2, 3, 4, 5, 6, 7, 8 });
  }
  
  public void sendIR(int irNum) {
    synchronized(this) {
      prepDeviceMessage("CURRENT", "SEND");
      getDevice().sendMessage(deviceMessage);
    }
  }
  
  public void sendIR(int irNum, int repeatCount) {
    if ((repeatCount < 0) || (repeatCount > 255)) return;
    if (repeatCount == 1) {
      sendIR(irNum);
      return;
    }
    
    synchronized(this) {
      prepDeviceMessage("CURRENT", "SENDX");
      deviceMessage.addNamedValue("data1", repeatCount);
      getDevice().sendMessage(deviceMessage);
    }
  }
  
  public void addListener(HVIRListenerI theListener) {
    irManager.addListener(theListener, this);
  }
  
  public void removeListener(HVIRListenerI theListener) {
    irManager.removeListener(theListener, this);
  }

  void handleIRXPLEvent(xPL_MessageI theMessage) {
    // Install Attr Values
    if (installAttrs(theMessage.getMessageBody())) {
      irManager.fireIRChangedEvent(this, HVIREventI.EventReason.ATTR_CHANGED);
    }
  }
  
  public String toString() {
    return String.valueOf(irNum);
  }
}
