/*
 * HVIRManager.java
 *
 * Created on December 20, 2004, 10:56 AM
 */

package org.cdp1802.hvision.ir;

import java.util.ArrayList;
import java.util.Iterator;

import org.cdp1802.hvision.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gduprey
 */
public class HVIRManager implements HVIRManagerI, xPL_DeviceMessageListenerI {
  class IRListener {
    HVIRI listenForIR = null;
    int listenForDeviceCode = -1, listenForKeyCode = -1;
    HVIRListenerI theListener = null;
    
    IRListener(HVIRListenerI theListener) {
      this.theListener = theListener;
    }
    IRListener(HVIRI forIR, HVIRListenerI theListener) {
      this.listenForIR = forIR;
      this.theListener = theListener;
    }
    IRListener(int forDeviceCode, int forKeyCode, HVIRListenerI theListener) {
      this.listenForDeviceCode = forDeviceCode;
      this.listenForKeyCode = forKeyCode;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return (listenForIR == null); }
    boolean isKnownIRListener() { return listenForIR != null; }
    boolean isStandardIRListener() { return (listenForDeviceCode != -1); }
    
    boolean isInterested(HVIRI theIR) {
      if (isGeneralListener()) return true;
      if (isStandardIRListener()) return false;
      
      if (listenForIR != theIR) return false;
      return true;
    }
    
    boolean isInterested(int theDeviceCode, int theKeyCode) {
      if (isGeneralListener()) return true;
      if (isKnownIRListener()) return false;
      
      if (theDeviceCode != listenForDeviceCode) return false;
      if ((theKeyCode != -1) && (theKeyCode != listenForKeyCode)) return false;
      return true;
    }

    HVIRListenerI getListener() { return theListener; }
    HVIRI getIR() { return listenForIR; }
    int getStandardDeviceCode() { return listenForDeviceCode; }
    int getStandardKeyCode() { return listenForKeyCode; }
  }
  
  xPL_DeviceI theXPLDevice = null;
  HVIRI irList[] = new HVIRI[256];
  ArrayList<IRListener> listenerList = new ArrayList<IRListener>();
  
  public HVIRManager() { }

  public void startManager() {
    this.theXPLDevice = HVManager.getManager().getXPLDevice();
    theXPLDevice.addMessageListener(this);
  }

  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = theXPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    configMessage.addNamedValue("TYPE", "INFRARED");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    theXPLDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public HVIRI getIR(int irNum) {
    // Sanity check
    if ((irNum < 0) || (irNum > 255)) return null;
    
    synchronized(irList) {
      if (irList[irNum] != null) return irList[irNum];
      
      return (irList[irNum] = new HVIR(this, irNum));
    }
  }
  
  public void addListener(HVIRListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new IRListener(theListener));
    }
  }
  
  public void addListener(HVIRListenerI theListener, HVIRI forIR) {
    synchronized(listenerList) {
      listenerList.add(new IRListener(forIR, theListener));
    }
  }
  
  public void addListener(HVIRListenerI theListener, int forIRNum) {
    HVIRI forIR = getIR(forIRNum);
    if (forIR != null) addListener(theListener, forIR);
  }

  public void addListener(HVIRListenerI theListener, int irDeviceCode, int irKeyCode) {
    synchronized(listenerList) {
      listenerList.add(new IRListener(irDeviceCode, irKeyCode, theListener));
    }
  }
  
  public void removeListener(HVIRListenerI theListener) {
    synchronized(listenerList) {
      IRListener irListener = null;
      Iterator<IRListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        irListener = theListeners.next();

        if (irListener.theListener != theListener) continue;
        if (!irListener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVIRListenerI theListener, HVIRI forIR) {
    synchronized(listenerList) {
      IRListener irListener = null;
      Iterator<IRListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        irListener = theListeners.next();

        if (irListener.theListener != theListener) continue;
        if (!irListener.isKnownIRListener() || (irListener.getIR() != forIR)) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVIRListenerI theListener, int forIRNum) {
    HVIRI forIR = getIR(forIRNum);
    if (forIR != null) removeListener(theListener, forIR);
  }
  
  public void removeListener(HVIRListenerI theListener, int irDeviceCode, int irKeyCode) {
    synchronized(listenerList) {
      IRListener irListener = null;
      Iterator<IRListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        irListener = theListeners.next();

        if (irListener.theListener != theListener) continue;
        if (!irListener.isStandardIRListener()) continue;
        if (irListener.getStandardDeviceCode() != irDeviceCode) continue;
        if (irListener.getStandardKeyCode() != irKeyCode) continue;
        theListeners.remove();
      }
    }
  }

  public void removeAllListeners(HVIRListenerI theListener) {
    synchronized(listenerList) {
      IRListener irListener = null;
      Iterator<IRListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        irListener = theListeners.next();

        if (irListener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireIRChangedEvent(HVIRI theIR, HVIREventI.EventReason theReason) {
    HVIREvent theEvent = new HVIREvent(theIR, theReason);
    
    for (IRListener theListener: listenerList) {
      if (theListener.isInterested(theIR)) try {
        theListener.getListener().handleIRMessage(theEvent);
      } catch (Exception dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void fireIRChangedEvent(int deviceCode, int keyCode, HVIREventI.EventReason theReason) {
    HVIREvent theEvent = new HVIREvent(deviceCode, keyCode, theReason);
    
    for (IRListener theListener: listenerList) {
      if (theListener.isInterested(deviceCode, keyCode)) try {
        theListener.getListener().handleIRMessage(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }

  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    // Make sure it's from this device
    if (this.theXPLDevice != theXPLDevice) return;
   
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("sensor")) return;
    
    // Get the device type and #
    boolean signalReceived = false;
    boolean knownSignal = false;
    boolean standardSignal = false;
    String signalType = theMessage.getNamedValue("CURRENT");
    if ((signalType != null) && (signalType.length() != 0)) {
      knownSignal = signalType.equalsIgnoreCase("RECEIVED KNOWN");
      standardSignal = signalType.equalsIgnoreCase("RECEIVED STANDARD");
      signalReceived = knownSignal || standardSignal;
    }
    
    String theType = theMessage.getNamedValue("TYPE");
    int theDevNum = theMessage.getIntNamedValue("DEVICE", -1);
    int theKeyCode = theMessage.getIntNamedValue("KEY", -1);

    // Bail out if missing anything important
    if ((theType == null) || !theType.equalsIgnoreCase("INFRARED")) return;
    if (!standardSignal && ((theDevNum < 0) || (theDevNum > 255))) return;

    // Dispatch it
    if (!standardSignal) ((HVIR) getIR(theDevNum)).handleIRXPLEvent(theMessage);
    
    // Send received alerts, if any
    if (signalReceived) {
      if (knownSignal)
        fireIRChangedEvent(getIR(theDevNum), HVIREventI.EventReason.KNOWN_IR_RECEIVED);
      else
        fireIRChangedEvent(theDevNum, theKeyCode, HVIREventI.EventReason.STANDARD_IR_RECEIVED);
    }
  }
}
