/*
 * HVMacroManager.java
 *
 * Created on December 20, 2004, 10:56 AM
 */

package org.cdp1802.hvision.macro;

import java.util.ArrayList;
import java.util.Iterator;

import org.cdp1802.hvision.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gduprey
 */
public class HVMacroManager implements HVMacroManagerI, xPL_DeviceMessageListenerI {
  class MacroListener {
    HVMacroI listenForMacro = null;
    HVMacroListenerI theListener = null;
    
    MacroListener(HVMacroListenerI theListener) {
      this.theListener = theListener;
    }
    MacroListener(HVMacroI forMacro, HVMacroListenerI theListener) {
      this.listenForMacro = forMacro;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return (listenForMacro == null); }
    boolean isMacroListener() { return listenForMacro != null; }
    
    boolean isInterested(HVMacroI theMacro) {
      if ((listenForMacro != null) && (listenForMacro != theMacro)) return false;
      return true;
    }
    
    HVMacroListenerI getListener() { return theListener; }
    HVMacroI getMacro() { return listenForMacro; }
  }
  
  xPL_DeviceI theXPLDevice = null;
  HVMacroI macroList[] = new HVMacroI[256];
  ArrayList<MacroListener> listenerList = new ArrayList<MacroListener>();
  
  public HVMacroManager() { }

  public void startManager() {
    this.theXPLDevice = HVManager.getManager().getXPLDevice();
    theXPLDevice.addMessageListener(this);
  }

  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = theXPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    configMessage.addNamedValue("TYPE", "MACRO");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    theXPLDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public HVMacroI getMacro(int macroNum) {
    // Sanity check
    if ((macroNum < 0) || (macroNum > 255)) return null;
    
    synchronized(macroList) {
      if (macroList[macroNum] != null) return macroList[macroNum];
      
      return (macroList[macroNum] = new HVMacro(this, macroNum));
    }
  }
  
  public void addListener(HVMacroListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new MacroListener(theListener));
    }
  }
  
  public void addListener(HVMacroListenerI theListener, HVMacroI forMacro) {
    synchronized(listenerList) {
      listenerList.add(new MacroListener(forMacro, theListener));
    }
  }
  
  public void addListener(HVMacroListenerI theListener, int forMacroNum) {
    HVMacroI forMacro = getMacro(forMacroNum);
    if (forMacro != null) addListener(theListener, forMacro);
  }

  public void removeListener(HVMacroListenerI theListener) {
    synchronized(listenerList) {
      MacroListener macroListener = null;
      Iterator<MacroListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        macroListener = theListeners.next();

        if (macroListener.theListener != theListener) continue;
        if (!macroListener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVMacroListenerI theListener, HVMacroI forMacro) {
    synchronized(listenerList) {
      MacroListener macroListener = null;
      Iterator<MacroListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        macroListener = theListeners.next();

        if (macroListener.theListener != theListener) continue;
        if (macroListener.listenForMacro != forMacro) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVMacroListenerI theListener, int forMacroNum) {
    HVMacroI forMacro = getMacro(forMacroNum);
    if (forMacro != null) removeListener(theListener, forMacro);
  }
  
  public void removeAllListeners(HVMacroListenerI theListener) {
    synchronized(listenerList) {
      MacroListener macroListener = null;
      Iterator<MacroListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        macroListener = theListeners.next();

        if (macroListener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireMacroChangedEvent(HVMacroI theMacro, HVMacroEventI.EventReason theReason) {
    HVMacroEvent theEvent = new HVMacroEvent(theMacro, theReason);
    for (MacroListener theListener: listenerList) {
      if (theListener.isInterested(theMacro)) try {
        theListener.getListener().handleMacroMessage(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    // Make sure it's from this device
    if (this.theXPLDevice != theXPLDevice) return;
   
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("sensor")) return;
    
    // Get the device type and #
    String theType = theMessage.getNamedValue("TYPE");
    int theDevNum = theMessage.getIntNamedValue("DEVICE", -1);

    // Bail out if missing anything important
    if ((theType == null) || !theType.equalsIgnoreCase("MACRO") || (theDevNum < 0) || (theDevNum > 255)) return;

    // Dispatch it
    ((HVMacro) getMacro(theDevNum)).handleMacroXPLEvent(theMessage);
  }
}
