/*
 * HVPhoneManager.java
 *
 * Created on January 10, 2005, 7:27 PM
 */

package org.cdp1802.hvision.phone;

import java.util.ArrayList;

import org.cdp1802.hvision.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gerry
 */
public class HVPhoneManager implements HVPhoneManagerI, xPL_DeviceMessageListenerI {
  xPL_DeviceI theXPLDevice = null;
  xPL_MutableMessageI theMessage = null;
  ArrayList<HVPhoneListenerI> listenerList = new ArrayList<HVPhoneListenerI>();
  int lastRingCount = 0;
  String lastCallerIDNum = null, lastCallerIDName = null;
  boolean phoneOffHook = false;
  
  public HVPhoneManager() {  }
  
  public void startManager() {
    this.theXPLDevice = HVManager.getManager().getXPLDevice();
    theMessage = theXPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "phone", "request");
    theXPLDevice.addMessageListener(this);
  }
  
  public void requestConfiguration() {
    // Nothing to do
  }
  
  public boolean isPhoneOffHook() { return phoneOffHook; }

  public void setPhoneOffHook(boolean isOffHook) {
    theMessage.clearMessageBody();
    theMessage.addNamedValue("COMMAND", (isOffHook ? "OFF_HOOK" : "ON_HOOK"));
    theXPLDevice.sendMessage(theMessage);
  }
  
  public int getLastRingCount() { return lastRingCount; }
  public String getLastCallerIDNum() { return lastCallerIDNum; }
  public String getLastCallerIDName() { return lastCallerIDName; }
  
  public void dialPhone(String dialNumber) {
    if ((dialNumber == null) || (dialNumber.length() == 0)) return;
    theMessage.clearMessageBody();
    theMessage.addNamedValue("COMMAND", "DIAL");
    theMessage.addNamedValue("NUMBER", dialNumber);
    theXPLDevice.sendMessage(theMessage);
  }
  
  public void addListener(HVPhoneListenerI theListener) {
    synchronized(listenerList) {
      if (!listenerList.contains(theListener)) listenerList.add(theListener);
    }
  }
  
  public void removeListener(HVPhoneListenerI theListener) {
    synchronized(listenerList) {
      listenerList.remove(theListener);
    }
  }
  
  private void dispatchPhoneEvent(HVPhoneEvent theEvent) {
    for (HVPhoneListenerI theListener: listenerList) {
      try {
        theListener.handlePhoneMessage(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void firePhoneEvent(HVPhoneEventI.EventReason theReason) {
    dispatchPhoneEvent(new HVPhoneEvent(theReason));
  }

  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    // Make sure it's from this device
    if (this.theXPLDevice != theXPLDevice) return;
   
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("phone")) return;
    
    String theEvent = theMessage.getNamedValue("EVENT");
    if (theEvent == null) return;
    
    if (theEvent.equalsIgnoreCase("ON_HOOK")) {
      if (phoneOffHook) {
        phoneOffHook = false;
        firePhoneEvent(HVPhoneEventI.EventReason.PHONE_ON_HOOK);
      }
      return;
    }
    
    if (theEvent.equalsIgnoreCase("OFF_HOOK")) {
      if (!phoneOffHook) {
        phoneOffHook = true;
        firePhoneEvent(HVPhoneEventI.EventReason.PHONE_OFF_HOOK);
      }
      return;
    }
    
    if (theEvent.equalsIgnoreCase("RING")) {
      String textRingCount = theMessage.getNamedValue("VALUE");
      if ((textRingCount == null) || (textRingCount.length() == 0)) return;
      
      if (textRingCount.equalsIgnoreCase("STOPPED")) {
        dispatchPhoneEvent(new HVPhoneEvent(HVPhoneEventI.EventReason.PHONE_STOPPED_RINGING, lastRingCount));
        return;
      }
      
      int phoneRings = theMessage.getIntNamedValue("VALUE", -1);
      if (phoneRings == -1) return;
      dispatchPhoneEvent(new HVPhoneEvent(HVPhoneEventI.EventReason.PHONE_RINGING, (lastRingCount = phoneRings)));
      return;
    }
    
    if (theEvent.equalsIgnoreCase("BUTTON")) {
      String theButton = theMessage.getNamedValue("VALUE");
      if ((theButton == null) || (theButton.length() != 1)) return;
      dispatchPhoneEvent(new HVPhoneEvent(HVPhoneEventI.EventReason.BUTTON_PRESSED, theButton.charAt(0)));
      return;
    }
    
    if (theEvent.equalsIgnoreCase("CALLER_ID")) {
      String callerNum = theMessage.getNamedValue("NUMBER");
      if ((callerNum == null) || (callerNum.length() == 0)) return;
      lastCallerIDNum = callerNum;
      
      String callerName = theMessage.getNamedValue("NAME");
      if ((callerName == null) || (callerName.length() == 0)) {
        lastCallerIDName = null;
      } else {
        lastCallerIDName = callerName;
      }

      dispatchPhoneEvent(new HVPhoneEvent(HVPhoneEventI.EventReason.CALLER_ID, lastCallerIDNum, lastCallerIDName));
    }
  }
}
