/*
 * HVVariable.java
 *
 * Created on December 20, 2004, 11:31 AM
 */

package org.cdp1802.hvision.variable;

import java.util.Collection;
import org.cdp1802.hvision.HVBaseDevice;
import org.cdp1802.xpl.NamedValueI;
import org.cdp1802.xpl.NamedValuesI;
import org.cdp1802.xpl.xPL_MessageI;


/**
 *
 * @author gduprey
 */
public class HVVariable extends HVBaseDevice implements HVVariableI {
  HVVariableManager varManager = null;
  int variableNum = -1;
  int curValue = 0;
  
  HVVariable(HVVariableManagerI theManager,int variableNum) {
    this.varManager = (HVVariableManager) theManager;
    this.variableNum = variableNum;
    this.ignoreAttrBodyNames = new String[] { "TYPE", "DEVICE", "CURRENT" };
  }

  protected void prepDeviceMessage(String msgItemName, String msgItemValue) {
    if (deviceMessage == null) deviceMessage = getDevice().createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    deviceMessage.clearMessageBody();
    deviceMessage.addNamedValue("DEVICE", variableNum);
    deviceMessage.addNamedValue("TYPE", "VARIABLE");
    deviceMessage.addNamedValue(msgItemName, msgItemValue);
  }
  
  public int getVariableNum() { return variableNum; }
  
  public int getValue() { return curValue; }
  public void setValue(int newValue) {
    if ((newValue < 0) || (newValue > 255) || (newValue == curValue)) return;
  
    // Send value out
    synchronized(this) {
      prepDeviceMessage("CURRENT", String.valueOf(newValue));
      getDevice().sendMessage(deviceMessage);
    }
  }

  public int getSignedValue() {
    if (curValue < 128) return curValue;
    return curValue - 256;
  }

  public void setSignedValue(int newValue) {
    if ((newValue < -128) || (newValue > 127)) return;
    if (newValue < 0) newValue = 256 + newValue;
    setValue(newValue);
  }
  
  
  public void addListener(HVVariableListenerI theListener) {
    varManager.addListener(theListener, this);
  }
  
  public void removeListener(HVVariableListenerI theListener) {
    varManager.removeListener(theListener, this);
  }

  void handleVariableXPLEvent(xPL_MessageI theMessage) {
    // Get value
    int newValue = theMessage.getIntNamedValue("CURRENT", -1);
    if (newValue == -1) return;

    if (newValue != curValue) {
      curValue = newValue;
      varManager.fireVariableChangedEvent(this, HVVariableEventI.EventReason.VALUE_CHANGED);
    }
    
    // Install Attr Values
    if (installAttrs(theMessage.getMessageBody())) {
      varManager.fireVariableChangedEvent(this, HVVariableEventI.EventReason.ATTR_CHANGED);
    }
  }
  
  public String toString() {
    return String.valueOf(variableNum) + "=" + curValue;
  }
}
