/*
 * HVVariableManager.java
 *
 * Created on December 20, 2004, 10:56 AM
 */

package org.cdp1802.hvision.variable;

import java.util.ArrayList;
import java.util.Iterator;

import org.cdp1802.hvision.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gduprey
 */
public class HVVariableManager implements HVVariableManagerI, xPL_DeviceMessageListenerI {
  class VarListener {
    HVVariableI listenForVariable = null;
    HVVariableListenerI theListener = null;
    
    VarListener(HVVariableListenerI theListener) {
      this.theListener = theListener;
    }
    VarListener(HVVariableI forVariable, HVVariableListenerI theListener) {
      this.listenForVariable = forVariable;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return (listenForVariable == null); }
    boolean isVariableListener() { return listenForVariable != null; }
    
    boolean isInterested(HVVariableI theVariable) {
      if (isGeneralListener()) return true;
      if (isVariableListener() && (listenForVariable != theVariable)) return false;
      return true;
    }
    
    HVVariableListenerI getListener() { return theListener; }
    HVVariableI getVariable() { return listenForVariable; }
  }
  
  xPL_DeviceI theXPLDevice = null;
  HVVariableI varList[] = new HVVariableI[256];
  ArrayList<VarListener> listenerList = new ArrayList<VarListener>();
  
  public HVVariableManager() { }
  
  public void startManager() {
    this.theXPLDevice = HVManager.getManager().getXPLDevice();
    theXPLDevice.addMessageListener(this);
  }
  
  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = theXPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "control", "advanced");
    configMessage.addNamedValue("TYPE", "VARIABLE");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    theXPLDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public HVVariableI getVariable(int varNum) {
    // Sanity check
    if ((varNum < 0) || (varNum > 255)) return null;
    
    synchronized(varList) {
      if (varList[varNum] != null) return varList[varNum];
      
      return (varList[varNum] = new HVVariable(this, varNum));
    }
  }
  
  public void addListener(HVVariableListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new VarListener(theListener));
    }
  }
  
  public void addListener(HVVariableListenerI theListener, HVVariableI forVariable) {
    synchronized(listenerList) {
      listenerList.add(new VarListener(forVariable, theListener));
    }
  }
  
  public void addListener(HVVariableListenerI theListener, int forVariableNum) {
    HVVariableI forVariable = getVariable(forVariableNum);
    if (forVariable != null) addListener(theListener, forVariable);
  }

  public void removeListener(HVVariableListenerI theListener) {
    synchronized(listenerList) {
      VarListener varListener = null;
      Iterator<VarListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        varListener = theListeners.next();

        if (varListener.theListener != theListener) continue;
        if (!varListener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVVariableListenerI theListener, HVVariableI forVariable) {
    synchronized(listenerList) {
      VarListener varListener = null;
      Iterator<VarListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        varListener = theListeners.next();

        if (varListener.theListener != theListener) continue;
        if (varListener.listenForVariable != forVariable) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVVariableListenerI theListener, int forVariableNum) {
    HVVariableI forVariable = getVariable(forVariableNum);
    if (forVariable != null) removeListener(theListener, forVariable);
  }
  
  public void removeAllListeners(HVVariableListenerI theListener) {
    synchronized(listenerList) {
      VarListener varListener = null;
      Iterator<VarListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        varListener = theListeners.next();

        if (varListener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireVariableChangedEvent(HVVariableI theVariable, HVVariableEventI.EventReason theReason) {
    HVVariableEvent theEvent = new HVVariableEvent(theVariable, theReason);
    for (VarListener theListener: listenerList) {
      if (theListener.isInterested(theVariable)) try {
        theListener.getListener().handleVariableMessage(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    if (this.theXPLDevice != theXPLDevice) return;
    
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("sensor")) return;
    
    // Get the device type and #
    String theType = theMessage.getNamedValue("TYPE");
    int theDevNum = theMessage.getIntNamedValue("DEVICE", -1);

    // Bail out if missing anything important
    if ((theType == null) || !theType.equalsIgnoreCase("VARIABLE") || (theDevNum < 0) || (theDevNum > 255)) return;
    
    // Dispatch it
    ((HVVariable) getVariable(theDevNum)).handleVariableXPLEvent(theMessage);
  }
}
