/*
 * HVX10Device.java
 *
 * Created on December 8, 2004, 6:07 PM
 */

package org.cdp1802.hvision.x10;

import org.cdp1802.hvision.HVBaseDevice;
import org.cdp1802.xpl.xPL_MessageI;

/**
 *
 * @author  gerry
 */
public class HVX10Device extends HVBaseDevice implements HVX10DeviceI {
  HVX10Manager x10Manager = null;
  HVX10HouseI theHouse = null;
  int unitNum = -1;
  String unitName = null;
  boolean unitOn = false;
  int dimLevel = 0;
  boolean dimmable = false;
  
  HVX10Device(HVX10Manager theManager, HVX10HouseI theHouse, int unitNum) {
    this.x10Manager = theManager;
    this.theHouse = theHouse;
    this.unitNum = unitNum;
    
    unitName = String.valueOf(theHouse.getHouseCode()) + String.valueOf(unitNum);
    ignoreAttrBodyNames = new String[] { "COMMAND", "LEVEL", "DEVICE", "HOUSE" };
  }

  public String getDeviceName() { return unitName; }
  public HVX10HouseI getHouse() { return theHouse; }
  public int getUnitNum() { return unitNum; }
  
  protected void prepDeviceMessage(String theItemName, String theItemValue) {
    if (deviceMessage == null) deviceMessage = getDevice().createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "x10", "advanced");
    deviceMessage.clearMessageBody();
    deviceMessage.addNamedValue(theItemName, theItemValue);
    deviceMessage.addNamedValue("DEVICE", unitName);
  }
  
  public void setUnitOn(boolean unitOn) {
    if (this.unitOn == unitOn) return;

    synchronized(this) {
      if (unitOn)
        prepDeviceMessage("COMMAND", "ON");
      else
        prepDeviceMessage("COMMAND", "OFF");
      getDevice().sendMessage(deviceMessage);
    }
  }
  public boolean isUnitOn() { return unitOn; }

  public boolean isDimmable() { return dimmable; }
  public void setDimmable(boolean isDimmable) {
    if (dimmable == isDimmable) return;
    
    synchronized(this) {
      dimmable = isDimmable;
      prepDeviceMessage("COMMAND", "SET_ATTR");
      deviceMessage.addNamedValue("DIMMABLE", (isDimmable ? "TRUE" : "FALSE"));
      getDevice().sendMessage(deviceMessage);
    }
  }

  public void dimUnit(int dimCount) {
    if (!dimmable || (dimCount < 1)) return;
    
    synchronized(this) {
      prepDeviceMessage("COMMAND", "DIM");
      if (dimCount > 1) deviceMessage.addNamedValue("COUNT", dimCount);
      getDevice().sendMessage(deviceMessage);
    }
  }
  public void dimUnit() {
    dimUnit(1);
  }

  public void brightenUnit(int brightenCount) {
    if (!dimmable || (brightenCount < 1)) return;
    
    synchronized(this) {
      prepDeviceMessage("COMMAND", "BRIGHT");
      if (brightenCount > 1) deviceMessage.addNamedValue("COUNT", brightenCount);
      getDevice().sendMessage(deviceMessage);
    }
  }
  public void brightenUnit() {
    brightenUnit(1);
  }

  public void setDimLevel(int dimLevel) {
    if (!dimmable || (dimLevel < 0) || (dimLevel > 100)) return;
    
    synchronized(this) {
      prepDeviceMessage("COMMAND", "DIM_TO");
      deviceMessage.addNamedValue("LEVEL", dimLevel);
      getDevice().sendMessage(deviceMessage);
    }
  }
  public int getDimLevel() { return dimLevel; }
  
  public void selectDevice() {
    synchronized(this) {
      prepDeviceMessage("COMMAND", "SELECT");
      getDevice().sendMessage(deviceMessage);
    }
  }
  
  public void setEnabled(boolean deviceEnabled) {
    synchronized(this) {
      if (deviceEnabled)
        prepDeviceMessage("COMMAND", "ENABLE");
      else
        prepDeviceMessage("COMMAND", "DISABLE");
        
      getDevice().sendMessage(deviceMessage);
    }
  }

  public void addListener(HVX10ListenerI theListener) {
    x10Manager.addListener(theListener, this);
  }
  
  public void removeListener(HVX10ListenerI theListener) {
    x10Manager.removeListener(theListener, this);
  }

  void handleX10XPLEvent(xPL_MessageI theMessage) {
    String theCommand = theMessage.getNamedValue("COMMAND");
    if ((theCommand == null) || (theCommand.length() == 0)) return;
    
    if (theCommand.equalsIgnoreCase("ON")) {
      int newLevel = theMessage.getIntNamedValue("LEVEL", -1);
      
      if (!unitOn) {
        unitOn = true;
        if (newLevel != -1) dimLevel = newLevel;
        x10Manager.fireDeviceChangedEvent(this, HVX10EventI.EventReason.DEVICE_ON);
      } else if ((newLevel != -1) && (newLevel != dimLevel)) {
        dimLevel = newLevel;
        x10Manager.fireDeviceChangedEvent(this, HVX10EventI.EventReason.DEVICE_LEVEL_CHANGED);
      }
    }
    
    if (theCommand.equalsIgnoreCase("OFF")) {
      if (unitOn) {
        unitOn = false;
        x10Manager.fireDeviceChangedEvent(this, HVX10EventI.EventReason.DEVICE_OFF);
      }
    }

    // Install Attr Values
    if (installAttrs(theMessage.getMessageBody())) {
      dimmable = deviceAttrList.getBoolNamedValue("DIMMABLE");
      x10Manager.fireDeviceChangedEvent(this, HVX10EventI.EventReason.ATTR_CHANGED);
    }
  }
  
  public String toString() {
    return unitName;
  }
}
