/*
 * HVX10House.java
 *
 * Created on December 9, 2004, 10:22 AM
 */

package org.cdp1802.hvision.x10;

import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author  gerry
 */
public class HVX10House implements HVX10HouseI {
  HVX10Manager x10Manager = null;
  char houseCode = '?';
  xPL_MutableMessageI x10Message = null;
  HVX10DeviceI unitList[] = new HVX10DeviceI[16];
  
  HVX10House(HVX10Manager theManager, char houseCode) {
    this.x10Manager = theManager;
    this.houseCode = houseCode;
  }
  
  public char getHouseCode() { return houseCode; }
  
  public HVX10DeviceI getUnit(int unitNum) {
    if ((unitNum < 0) || (unitNum > 15)) return null;
    if (unitList[unitNum] == null) {
      unitList[unitNum] = new HVX10Device(x10Manager, this, unitNum);
    }
    return unitList[unitNum];
  }
  
  
  void prepMessage(String theCommand) {
    if (x10Message == null) x10Message = x10Manager.x10XPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "x10", "advanced");
    x10Message.clearMessageBody();
    x10Message.addNamedValue("COMMAND", theCommand);
    x10Message.addNamedValue("HOUSE", houseCode);
  }
  
  public void sendHouseOn() {
    synchronized(x10Message) {
      prepMessage("HOUSE_ON");
      x10Manager.x10XPLDevice.sendMessage(x10Message);
    }
  }
  
  public void sendHouseOff() {
    synchronized(x10Message) {
      prepMessage("HOUSE_OFF");
      x10Manager.x10XPLDevice.sendMessage(x10Message);
    }
  }
  
  public void sendHouseDim() {
    synchronized(x10Message) {
      prepMessage("HOUSE_DIM");
      x10Manager.x10XPLDevice.sendMessage(x10Message);
    }
  }
  
  public void sendHouseBrighten() {
    synchronized(x10Message) {
      prepMessage("HOUSE_BRIGHT");
      x10Manager.x10XPLDevice.sendMessage(x10Message);
    }
  }
  
  public void sendAllLightsOn() {
    synchronized(x10Message) {
      prepMessage("ALL_LIGHTS_ON");
      x10Manager.x10XPLDevice.sendMessage(x10Message);
    }
  }
  
  public void sendAllLightsOff() {
    synchronized(x10Message) {
      prepMessage("ALL_LIGHTS_OFF");
      x10Manager.x10XPLDevice.sendMessage(x10Message);
    }
  }
  
  public void sendAllUnitsOff() {
    synchronized(x10Message) {
      prepMessage("ALL_UNITS_OFF");
      x10Manager.x10XPLDevice.sendMessage(x10Message);
    }
  }
  
  public void addListener(HVX10ListenerI theListener) {
    x10Manager.addListener(theListener, this);
  }
  
  public void removeListener(HVX10ListenerI theListener) {
    x10Manager.removeListener(theListener, this);
  }

  void dispatchX10XPLEvent(xPL_MessageI theMessage) {
    HVX10DeviceI theDevice = null;
    for (int unitIndex = 0; unitIndex < 16; unitIndex++) {
      theDevice = getUnit(unitIndex);
      if (!(theDevice instanceof HVX10Device)) continue;
      ((HVX10Device) theDevice).handleX10XPLEvent(theMessage);
    }
  }

  public String toString() { return String.valueOf(houseCode); }
}
