/*
 * HVX10.java
 *
 * Created on December 9, 2004, 11:18 AM
 */

package org.cdp1802.hvision.x10;

import java.util.ArrayList;
import java.util.Iterator;

import org.cdp1802.hvision.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author  gerry
 */
public class HVX10Manager implements HVX10ManagerI, xPL_DeviceMessageListenerI {
  class X10Listener {
    HVX10HouseI listenForHouse = null;
    HVX10DeviceI listenForDevice = null;
    HVX10ListenerI theListener = null;
    
    X10Listener(HVX10ListenerI theListener) {
      this.theListener = theListener;
    }
    X10Listener(HVX10HouseI forHouse, HVX10ListenerI theListener) {
      this.listenForHouse = forHouse;
      this.theListener = theListener;
    }
    X10Listener(HVX10DeviceI forDevice, HVX10ListenerI theListener) {
      this.listenForDevice = forDevice;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return ((listenForHouse == null) && (listenForDevice == null)); }
    boolean isHouseListener() { return listenForHouse != null; }
    boolean isDeviceListener() { return listenForDevice != null; }
    
    boolean isInterested(HVX10DeviceI theDevice) {
      if ((listenForDevice != null) && (listenForDevice != theDevice)) return false;
      if ((listenForHouse != null) && (listenForHouse != theDevice.getHouse())) return false;
      return true;
    }
    
    HVX10ListenerI getListener() { return theListener; }
    HVX10HouseI getHouse() { return listenForHouse; }
    HVX10DeviceI getDevice() { return listenForDevice; }
  }
  
  xPL_DeviceI x10XPLDevice = null;
  HVX10HouseI houseList[] = new HVX10HouseI[16];
  ArrayList<X10Listener> listenerList = new ArrayList<X10Listener>();
  
  public HVX10Manager() {
    for (int houseIndex = 0; houseIndex < 16; houseIndex++)
      houseList[houseIndex] = new HVX10House(this, (char) ('A' + houseIndex));
  }

  public void startManager() {
    this.x10XPLDevice = HVManager.getManager().getXPLDevice();
    x10XPLDevice.addMessageListener(this);
  }
  
  
  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = x10XPLDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "x10", "advanced");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    x10XPLDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public HVX10HouseI getHouse(char houseCode) {
    if ((houseCode >= 'A') && (houseCode <= 'P')) {
      return houseList[(int) (houseCode - 'A')];
    } else if ((houseCode >= 'a') && (houseCode <= 'p')) {
      return houseList[(int) (houseCode - 'a')];
    } else 
      return null;
  }

  public HVX10DeviceI getDevice(char houseCode, int unitNum) {
    return getHouse(houseCode).getUnit(unitNum);
  }
  
  public HVX10DeviceI getDevice(HVX10HouseI theHouse, int unitNum) {
    if (theHouse == null) return null;
    return theHouse.getUnit(unitNum);
  }
  
  public HVX10DeviceI getDevice(String houseUnitSpec) {
    if ((houseUnitSpec == null) || (houseUnitSpec.length() < 2)) return null;
    
    // Get house code
    HVX10HouseI theHouse = getHouse(houseUnitSpec.toUpperCase().charAt(0));
    if (theHouse == null) return null;
    
    // Extract unit number
    int unitNum = -1;
    try {
      unitNum = Integer.parseInt(houseUnitSpec.substring(1));
    } catch (Exception badNumber) { return null; }
    
    return theHouse.getUnit(unitNum);
  }
  
  public void addListener(HVX10ListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new X10Listener(theListener));
    }
  }
  
  public void addListener(HVX10ListenerI theListener, HVX10HouseI forHouse) {
    synchronized(listenerList) {
      listenerList.add(new X10Listener(forHouse, theListener));
    }
  }
  
  public void addListener(HVX10ListenerI theListener, char forHouseCode) {
    HVX10HouseI theHouse = getHouse(forHouseCode);
    if (theHouse == null) return;
    addListener(theListener, theHouse);
  }
  
  public void addListener(HVX10ListenerI theListener, HVX10DeviceI forDevice) {
    synchronized(listenerList) {
      listenerList.add(new X10Listener(forDevice, theListener));
    }
  }
  
  public void addListener(HVX10ListenerI theListener, char forHouseCode, int forUnitNum) {
    HVX10DeviceI theDevice = getDevice(forHouseCode, forUnitNum);
    if (theDevice == null) return;
    addListener(theListener, theDevice);
  }
  
  public void addListener(HVX10ListenerI theListener, String forHouseUnitSpec) {
    HVX10DeviceI theDevice = getDevice(forHouseUnitSpec);
    if (theDevice == null) return;
    addListener(theListener, theDevice);
  }
  
  public void removeListener(HVX10ListenerI theListener) {
    synchronized(listenerList) {
      X10Listener x10Listener = null;
      Iterator<X10Listener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        x10Listener = theListeners.next();

        if (x10Listener.theListener != theListener) continue;
        if (!x10Listener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVX10ListenerI theListener, HVX10HouseI forHouse) {
    synchronized(listenerList) {
      X10Listener x10Listener = null;
      Iterator<X10Listener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        x10Listener = theListeners.next();

        if (x10Listener.theListener != theListener) continue;
        if (x10Listener.listenForHouse != forHouse) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVX10ListenerI theListener, char forHouseCode) {
    HVX10HouseI forHouse = getHouse(forHouseCode);
    if (forHouse == null) return;
    
    synchronized(listenerList) {
      X10Listener x10Listener = null;
      Iterator<X10Listener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        x10Listener = theListeners.next();

        if (x10Listener.theListener != theListener) continue;
        if (x10Listener.listenForHouse != forHouse) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVX10ListenerI theListener, HVX10DeviceI forDevice) {
    synchronized(listenerList) {
      X10Listener x10Listener = null;
      Iterator<X10Listener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        x10Listener = theListeners.next();

        if (x10Listener.theListener != theListener) continue;
        if (x10Listener.listenForDevice != forDevice) continue;
        theListeners.remove();
      }
    }
  }
  
  public void removeListener(HVX10ListenerI theListener, char forHouseCode, int forUnitNum) {
    HVX10DeviceI forDevice = getDevice(forHouseCode, forUnitNum);
    if (forDevice == null) return;
    
    synchronized(listenerList) {
      X10Listener x10Listener = null;
      Iterator<X10Listener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        x10Listener = theListeners.next();

        if (x10Listener.theListener != theListener) continue;
        if (x10Listener.listenForDevice != forDevice) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(HVX10ListenerI theListener, String forHouseUnitSpec) {
    HVX10DeviceI forDevice = getDevice(forHouseUnitSpec);
    if (forDevice == null) return;
    
    synchronized(listenerList) {
      X10Listener x10Listener = null;
      Iterator<X10Listener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        x10Listener = theListeners.next();

        if (x10Listener.theListener != theListener) continue;
        if (x10Listener.listenForDevice != forDevice) continue;
        theListeners.remove();
      }
    }
  }
  
  public void removeAllListeners(HVX10ListenerI theListener) {
    synchronized(listenerList) {
      X10Listener x10Listener = null;
      Iterator<X10Listener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        x10Listener = theListeners.next();

        if (x10Listener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireDeviceChangedEvent(HVX10DeviceI theDevice, HVX10EventI.EventReason theReason) {
    HVX10Event theEvent = new HVX10Event(theDevice, theReason);
    for (X10Listener theListener: listenerList) {
      if (theListener.isInterested(theDevice)) try {
        theListener.getListener().handleX10Message(theEvent);
      } catch (Throwable dispatchException) { }
    }
    theEvent.releaseResources();
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI theXPLDevice, xPL_MessageI theMessage) {
    // Make sure it's from this device
    if (this.x10XPLDevice != theXPLDevice) return;
   
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("x10")) return;
    
    // Get the house or device code and command
    String theCommand = theMessage.getNamedValue("COMMAND");
    String theDeviceSpec = theMessage.getNamedValue("DEVICE");
    String theHouseSpec = theMessage.getNamedValue("HOUSE");
    
    // Bail out if missing anything important
    if ((theCommand == null) || ((theDeviceSpec == null) && (theHouseSpec == null))) return;
    
    // Handle dispatch
    if (theDeviceSpec != null) {
      HVX10DeviceI theDevice = getDevice(theDeviceSpec);
      if ((theDevice == null) || !(theDevice instanceof HVX10Device)) return;
      
      ((HVX10Device) theDevice).handleX10XPLEvent(theMessage);
    } else if ((theHouseSpec != null) && (theHouseSpec.length() == 1)) {
      HVX10HouseI theHouse = getHouse(theHouseSpec.charAt(0));
      if ((theHouse == null) || !(theHouse instanceof HVX10House)) return;
      
      ((HVX10House) theHouse).dispatchX10XPLEvent(theMessage);
    }
  }
}
