/*
 * TStat.javaa
 *
 * Created on December 20, 2004, 11:31 AM
 */

package org.cdp1802.tstat;

import java.util.ArrayList;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;


/**
 *
 * @author gduprey
 */
public class TStat implements TStatI {
  static TStatScheme autoScheme = new TStatScheme(null, "AUTO");
  
  TStatManager tstatManager = null;
  xPL_DeviceI tstatDevice = null;
  xPL_MutableMessageI tstatMessage = null;
  xPL_MutableMessageI schemeMessage = null;
  
  int tstatNum = -1;
  TStatMode curMode;
  TStatState curState;
  FanMode curFanMode;
  int curInsideTemp = 0, curRemoteTemp = 0;
  int curHeatSetpointTemp = 0, curCoolSetpointTemp = 0;
  boolean relayLights = false;
  boolean ledLit[] = new boolean[] { false, false, false, false };
  boolean tstatHeating = false, tstatCooling = false, tstatFanRunning = false;
  int filterRuntime = 0;
  TStatRuntime fanRuntime = new TStatRuntime();
  TStatRuntime heatRuntime = new TStatRuntime();
  TStatRuntime coolRuntime = new TStatRuntime();
  
  TStatSchemeI installedScheme = autoScheme;
  TStatSchemeI curScheme = null;
  TStatSetpointI curSetpoint = null;

  ArrayList<TStatSchemeI> schemeList = new ArrayList<TStatSchemeI>();
  
  TStat(TStatManagerI theManager, xPL_DeviceI theDevice, int tstatNum) {
    this.tstatManager = (TStatManager) theManager;
    this.tstatDevice = theDevice;
    this.tstatNum = tstatNum;
    
    tstatMessage = tstatDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "tstat", "command");
    schemeMessage = tstatDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "tstat", "scheme");
  }

  xPL_MutableMessageI getSchemeMessage() { return schemeMessage; }
  xPL_DeviceI getDevice() { return tstatDevice; }
  
  protected void prepTStatMessage(String theCommand) {
    tstatMessage.clearMessageBody();
    tstatMessage.addNamedValue("TSTAT", tstatNum);
    tstatMessage.addNamedValue("COMMAND", theCommand);
  }
  
  public void addListener(TStatListenerI theListener) {
    tstatManager.addListener(theListener, this);
  }
  
  public void removeListener(TStatListenerI theListener) {
    tstatManager.removeListener(theListener, this);
  }

  public int getTStatNum() { return tstatNum; }

  public int getInsideTemp() { return curInsideTemp; }
  public int getRemoteTemp() { return curRemoteTemp; }

  public int getSetpointTemp() {
    if (curMode == null) return 0;
    switch(curMode) {
      case HEAT: return curHeatSetpointTemp;
      case COOL: return curCoolSetpointTemp;
      default: return 0;
    }
  }
  public int getHeatSetpointTemp() { return curHeatSetpointTemp; }
  public int getCoolSetpointTemp() { return curCoolSetpointTemp; }
  
  public TStatMode getMode() { return curMode; }
  public void setMode(TStatMode newMode) {
    if (newMode == curMode) return;
    
    prepTStatMessage("MODE");
    tstatMessage.addNamedValue("TSTAT_MODE", newMode.toString());
    tstatDevice.sendMessage(tstatMessage);
  }
  
  public TStatState getState() { return curState; }
  public void setState(TStatState newState) {
    if (newState == curState) return;
    
    prepTStatMessage("STATE");
    tstatMessage.addNamedValue("STATE", newState.toString());
    tstatDevice.sendMessage(tstatMessage);
  }
  
  public FanMode getFanMode() { return curFanMode; }
  public void setFanMode(FanMode newFanMode) {
    if (newFanMode == curFanMode) return;
    
    prepTStatMessage("FAN_MODE");
    tstatMessage.addNamedValue("STATE", newFanMode.toString());
    tstatDevice.sendMessage(tstatMessage);
  }
  
  public boolean isRelayLightsMode() { return relayLights; }
  public void setRelayLightsMode(boolean doRelayLights) {
    if (doRelayLights == relayLights) return;
    
    prepTStatMessage("RELAY_LIGHTS");
    tstatMessage.addNamedValue("STATE", doRelayLights);
    tstatDevice.sendMessage(tstatMessage);
  }
  
  public boolean isLEDLit(int ledNum) { 
    if ((ledNum < 0) || (ledNum >= ledLit.length)) return false;
    return ledLit[ledNum];
  }
  public void setLEDLit(int ledNum, boolean isLit) {
    if (ledNum != -1) {
      if ((ledNum < 0) || (ledNum >= ledLit.length)) return;
      if (ledLit[ledNum] == isLit) return;
    }

    prepTStatMessage("LED");
    tstatMessage.addNamedValue("LED", (ledNum == -1 ? "ALL" : String.valueOf(ledNum + 1)));
    tstatMessage.addNamedValue("STATE", isLit);
    tstatDevice.sendMessage(tstatMessage);
  }
  
  public boolean isHeating() { return tstatHeating; }
  public boolean isCooling() { return tstatCooling; }
  public boolean isFanRunning() { return tstatFanRunning; }
  
  public TStatRuntimeI getFanRuntime() { return fanRuntime; }
  public TStatRuntimeI getHeatingRuntime() { return heatRuntime; }
  public TStatRuntimeI getCoolingRuntime() { return coolRuntime; }
  
  public int getFilterRuntime() { return filterRuntime; }

  
  void sendSchemeMessage(String theCommand, String schemeName) {
    synchronized(schemeMessage) {
      schemeMessage.clearMessageBody();
      schemeMessage.addNamedValue("COMMAND", theCommand);
      schemeMessage.addNamedValue("TSTAT", tstatNum);
      schemeMessage.addNamedValue("SCHEME_NAME", schemeName);
      tstatDevice.sendMessage(schemeMessage);
    }
  }
  
  public TStatSchemeI getCurrentScheme() { return curScheme; }
  public TStatSetpointI getCurrentSetpoint() { return curSetpoint; }

  public TStatSchemeI getInstalledScheme() { return installedScheme; }
  public void setInstalledScheme(TStatSchemeI theScheme) {
    if (theScheme == installedScheme) return;
    installedScheme = theScheme;
    sendSchemeMessage("SET_SCHEME", installedScheme.getName());
  }
  public void setInstalledScheme(String schemeName) {
    if ((schemeName == null) || (schemeName.length() == 0)) return;
    if (schemeName.equalsIgnoreCase("AUTO")) {
      setInstalledScheme(autoScheme);
    } else {
      TStatSchemeI theScheme = getSchemeNamed(schemeName);
      if (theScheme != null) setInstalledScheme(theScheme);
    }
  }
  
  public int getSchemeCount() { return schemeList.size(); }
  public TStatSchemeI getSchemeAt(int schemeIndex) {
    synchronized(schemeList) {
      if ((schemeIndex < 0) || (schemeIndex >= schemeList.size())) return null;
      return schemeList.get(schemeIndex);
    }
  }
  
  public TStatSchemeI getSchemeNamed(String schemeName) {
    if ((schemeName == null) || (schemeName.length() == 0)) return null;
    
    synchronized(schemeList) {
      for (TStatSchemeI theScheme: schemeList) {
        if (theScheme.getName().equalsIgnoreCase(schemeName)) return theScheme;
      }
    }
    return null;
  }
  
  public TStatSchemeI createNewScheme(String schemeName) {
    synchronized(schemeList) {
      // Handle a duplicate
      if (getSchemeNamed(schemeName) != null) return null;
      
      // Create a new scheme
      TStatScheme newScheme = new TStatScheme(this, schemeName);
      schemeList.add(newScheme);
      
      // Send scheme message 
      sendSchemeMessage("ADD_SCHEME", schemeName);
      return newScheme;
    }
  }

  public void deleteScheme(String schemeName) {
    synchronized(schemeList) {
      deleteScheme(getSchemeNamed(schemeName));
    }
  }

  public void deleteScheme(TStatSchemeI theScheme) {
    if (theScheme == null) return;
    
    synchronized(schemeList) {
      sendSchemeMessage("DELETE_SCHEME", theScheme.getName());
      schemeList.remove(theScheme);
    }
  }
  

  void handleTStatStatusMessage(xPL_MessageI theMessage) {
    boolean changeInStatus = false;
    boolean theBoolValue = false;
    int theIntValue = 0;

    // Xlate mode
    TStatMode newMode = curMode;
    String theValue = theMessage.getNamedValue("TSTAT_MODE").toUpperCase();
    if (theValue.equals("OFF"))
      newMode = TStatI.TStatMode.OFF;
    else if (theValue.equals("HEAT"))
      newMode = TStatI.TStatMode.HEAT;
    else if (theValue.equals("COOL"))
      newMode = TStatI.TStatMode.COOL;
    else if (theValue.equals("AUTO"))
      newMode = TStatI.TStatMode.AUTO;
    
    if (newMode != curMode) {
      curMode = newMode;
      changeInStatus = true;
    }

    // Xlate tstat state
    theValue = theMessage.getNamedValue("TSTAT_STATE").toUpperCase();
    TStatState newState = curState;
    if (theValue.equals("HOLD"))
      newState = TStatI.TStatState.HOLD;
    else if (theValue.equals("RUN"))
      newState = TStatI.TStatState.RUN;
    
    if (newState != curState) {
      curState = newState;
      changeInStatus = true;
    }

    // Xlate fan mode
    theValue = theMessage.getNamedValue("FAN_MODE").toUpperCase();
    FanMode newFanMode = curFanMode;
    if (theValue.equals("ON"))
      newFanMode = TStatI.FanMode.ON;
    else if (theValue.equals("AUTO"))
      newFanMode = TStatI.FanMode.AUTO;
    
    if (newFanMode != curFanMode) {
      curFanMode = newFanMode;
      changeInStatus = true;
    }
    
    if ((theBoolValue = theMessage.getBoolNamedValue("FAN_RUNNING")) != tstatFanRunning) {
      tstatFanRunning = theBoolValue;
      changeInStatus = true;
    }
    
    if ((theBoolValue = theMessage.getBoolNamedValue("HEAT_RUNNING")) != tstatHeating) {
      tstatHeating = theBoolValue;
      changeInStatus = true;
    }

    if ((theBoolValue = theMessage.getBoolNamedValue("COOL_RUNNING")) != tstatCooling) {
      tstatCooling = theBoolValue;
      changeInStatus = true;
    }
    
    if ((theIntValue = theMessage.getIntNamedValue("HEAT_SET_POINT")) != curHeatSetpointTemp) {
      curHeatSetpointTemp = theIntValue;
      changeInStatus = true;
    }
    
    if ((theIntValue = theMessage.getIntNamedValue("COOL_SET_POINT")) != curCoolSetpointTemp) {
      curCoolSetpointTemp = theIntValue;
      changeInStatus = true;
    }
    
    if ((theIntValue = theMessage.getIntNamedValue("CUR_INSIDE_TEMP")) != curInsideTemp) {
      curInsideTemp = theIntValue;
      changeInStatus = true;
    }

    if ((theIntValue = theMessage.getIntNamedValue("CUR_REMOTE_TEMP")) != curRemoteTemp) {
      curRemoteTemp = theIntValue;
      changeInStatus = true;
    }
    
    if ((theBoolValue = theMessage.getBoolNamedValue("LED_1")) != ledLit[0]) {
      ledLit[0] = theBoolValue;
      changeInStatus = true;
    }
    
    if ((theBoolValue = theMessage.getBoolNamedValue("LED_2")) != ledLit[1]) {
      ledLit[1] = theBoolValue;
      changeInStatus = true;
    }

    if ((theBoolValue = theMessage.getBoolNamedValue("LED_3")) != ledLit[2]) {
      ledLit[2] = theBoolValue;
      changeInStatus = true;
    }

    if ((theBoolValue = theMessage.getBoolNamedValue("LED_4")) != ledLit[3]) {
      ledLit[3] = theBoolValue;
      changeInStatus = true;
    }

    if ((theBoolValue = theMessage.getBoolNamedValue("RELAY_LIGHTS")) != relayLights) {
      relayLights = theBoolValue;
      changeInStatus = true;
    }
    
    if (changeInStatus) {
      tstatManager.fireTStatChangedEvent(this, TStatEventI.EventReason.STATE_CHANGED);
    }
  }
  
  void handleCurSetpointMessage(xPL_MessageI theMessage) {
    boolean setpointChanged = false;
    String theInstallScheme = theMessage.getNamedValue("SET_SCHEME");
    String theCurScheme = theMessage.getNamedValue("CUR_SCHEME");

    // Handle an auto-scheme
    if (theInstallScheme.equalsIgnoreCase("AUTO")) {
      if (installedScheme != autoScheme) {
        installedScheme = autoScheme;
        setpointChanged = true;
      }
    } else {
      TStatSchemeI theScheme = getSchemeNamed(theInstallScheme);
      if ((theScheme != null) && (theScheme != installedScheme)) {
        installedScheme = theScheme;
        setpointChanged = true;
      }
    }
    
    // Handle current scheme
    TStatSchemeI theScheme = getSchemeNamed(theCurScheme);
    if ((theScheme != null) && (theScheme != curScheme)) {
      curScheme = theScheme;
      curSetpoint = null;
      setpointChanged = true;
    }
    
    // Handle setpoint
    if (curScheme != null) {
      String theSetpointName = theMessage.getNamedValue("CUR_SET_POINT_NAME");
      if (theSetpointName != null) {
        TStatSetpointI theSetpoint = curScheme.getSetpointNamed(theSetpointName);
        if ((theSetpoint != null) && (theSetpoint != curSetpoint)) {
          curSetpoint = theSetpoint;
          setpointChanged = true;
        }
      }
    }
    
    if (setpointChanged) {
      tstatManager.fireTStatChangedEvent(this, curScheme, curSetpoint, TStatEventI.EventReason.SETPOINT_CHANGED);
    }
  }

  void handleSchemeStatusMessage(xPL_MessageI theMessage) {
    String whatChanged = theMessage.getNamedValue("CHANGE").toUpperCase();
    String schemeName = theMessage.getNamedValue("SCHEME_NAME");
    
    if (whatChanged.equals("NEW_SCHEME") || whatChanged.equals("UPDATE_SCHEME")) {
      synchronized(schemeList) {
        // Handle a duplicate
        if (getSchemeNamed(schemeName) != null) return;

        // Create a new scheme
        TStatScheme newScheme = new TStatScheme(this, schemeName);
        schemeList.add(newScheme);
        
        // Send scheme message 
        tstatManager.fireTStatChangedEvent(this, newScheme, TStatEventI.EventReason.SCHEME_ADDED);
        return;
      }
    }
    
    if (whatChanged.equals("DELETE_SCHEME")) {
      synchronized(schemeList) {
        // Fetch the scheme
        TStatSchemeI theScheme = getSchemeNamed(schemeName);
        if (theScheme == null) return;

        // Delete it
        tstatManager.fireTStatChangedEvent(this, theScheme, TStatEventI.EventReason.SCHEME_DELETED);
        schemeList.remove(theScheme);
        ((TStatScheme) theScheme).releaseResources();
        return;
      }
    }
    
    // Anything else involves set points -- punt it
    TStatSchemeI theScheme = getSchemeNamed(schemeName);
    if (theScheme == null) return;
    ((TStatScheme) theScheme).handleSetpointMessage(theMessage, whatChanged);
  }
  
  void handleRuntimeMessage(xPL_MessageI theMessage) {
    boolean runtimeChanged = false;
    
    int newFilterTime = theMessage.getIntNamedValue("FILTER");
    if ((newFilterTime != -1) && (newFilterTime != filterRuntime)) {
      filterRuntime = newFilterTime;
      tstatManager.fireTStatChangedEvent(this, (TStatRuntimeI) null);
    }
    
    if (fanRuntime.parseRuntime(theMessage, "FAN")) tstatManager.fireTStatChangedEvent(this, fanRuntime);
    if (heatRuntime.parseRuntime(theMessage, "HEAT")) tstatManager.fireTStatChangedEvent(this, heatRuntime);
    if (coolRuntime.parseRuntime(theMessage, "COOL")) tstatManager.fireTStatChangedEvent(this, coolRuntime);
  }
  
  void handleTStatXPLEvent(xPL_MessageI theMessage) {
    // Handle different message types
    String schemaType = theMessage.getSchemaType().toLowerCase();
    if (schemaType.equals("status"))
      handleTStatStatusMessage(theMessage);
    else if (schemaType.equals("cur_setpoint"))
      handleCurSetpointMessage(theMessage);
    else if (schemaType.equals("scheme_status"))
      handleSchemeStatusMessage(theMessage);
    else if (schemaType.equals("runtime"))
      handleRuntimeMessage(theMessage);
    else 
      System.out.println("TSTAT:: Got an unhandled message type of [" + schemaType + "] -- ignored");
  }
  
  public String toString() {
    return String.valueOf(tstatNum);
  }
}
