/*
 * StatNetManager.java
 *
 * Created on December 22, 2004, 10:55 AM
 */

package org.cdp1802.tstat;

import java.util.ArrayList;
import java.util.Iterator;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.xPL_Manager;
import org.cdp1802.xpl.xPL_MessageI;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gduprey
 */
public class TStatManager implements TStatManagerI, xPL_DeviceMessageListenerI {
  private static TStatManager theManager = null;
  public static TStatManagerI getManager() {
    if (theManager == null) theManager = new TStatManager();
    return theManager; 
  }
  
  class TStatListener {
    TStatI listenForTStat = null;
    TStatListenerI theListener = null;
    
    TStatListener(TStatListenerI theListener) {
      this.theListener = theListener;
    }
    TStatListener(TStatI forTStat, TStatListenerI theListener) {
      this.listenForTStat = forTStat;
      this.theListener = theListener;
    }
    
    boolean isGeneralListener() { return (listenForTStat == null); }
    boolean isTStatListener() { return listenForTStat != null; }
    
    boolean isInterested(TStatI theTStat) {
      if (isGeneralListener()) return true;
      if (isTStatListener() && (listenForTStat != theTStat)) return false;
      return true;
    }
    
    TStatListenerI getListener() { return theListener; }
    TStatI getTStat() { return listenForTStat; }
  }

  boolean managerInitialized = false;
  xPL_DeviceI tstatDevice = null;
  TStatI tstatList[] = new TStatI[10];
  ArrayList<TStatListener> listenerList = new ArrayList<TStatListener>();

  private TStatManager() { }
  
  public void startManager() {
    if (managerInitialized) return;
    
    // Create an xPL Device for us
    if (tstatDevice == null) {
      tstatDevice = xPL_Manager.getManager().getDeviceManager().createDevice("cdp1802", "tstatclient", null);
      tstatDevice.setEnabled(true);
    } 

    // Start listening
    tstatDevice.addMessageListener(this);
    
    // And we are done
    managerInitialized = true;
    
    // Request configuration
    requestConfiguration();
  }
  
  public boolean isInitialized() { return managerInitialized; }
  public xPL_DeviceI getDevice() { return tstatDevice; }
  public void setDevice(xPL_DeviceI theDevice) {
    if (managerInitialized) return;
    tstatDevice = theDevice;
  }

  public void requestConfiguration() {
    xPL_MutableMessageI configMessage = tstatDevice.createBroadcastMessage(xPL_MessageI.MessageType.COMMAND, "tstat", "command");
    configMessage.addNamedValue("COMMAND", "GET_CONFIG");
    tstatDevice.sendMessage(configMessage);
    configMessage.releaseResources();
  }
  
  public TStatI getTStat(int tstatNum) {
    // Sanity check
    if ((tstatNum < 0) || (tstatNum > 9)) return null;
    
    synchronized(tstatList) {
      if (tstatList[tstatNum] != null) return tstatList[tstatNum];
      
      return (tstatList[tstatNum] = new TStat(this, tstatDevice, tstatNum));
    }
  }
  
  public void addListener(TStatListenerI theListener) {
    synchronized(listenerList) {
      listenerList.add(new TStatListener(theListener));
    }
  }
  
  public void addListener(TStatListenerI theListener, TStatI forTStat) {
    synchronized(listenerList) {
      listenerList.add(new TStatListener(forTStat, theListener));
    }
  }
  
  public void addListener(TStatListenerI theListener, int forTStatNum) {
    TStatI forTStat = getTStat(forTStatNum);
    if (forTStat != null) addListener(theListener, forTStat);
  }

  public void removeListener(TStatListenerI theListener) {
    synchronized(listenerList) {
      TStatListener varListener = null;
      Iterator<TStatListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        varListener = theListeners.next();

        if (varListener.theListener != theListener) continue;
        if (!varListener.isGeneralListener()) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(TStatListenerI theListener, TStatI forTStat) {
    synchronized(listenerList) {
      TStatListener varListener = null;
      Iterator<TStatListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        varListener = theListeners.next();

        if (varListener.theListener != theListener) continue;
        if (varListener.listenForTStat != forTStat) continue;
        theListeners.remove();
      }
    }
  }

  public void removeListener(TStatListenerI theListener, int forTStatNum) {
    TStatI forTStat = getTStat(forTStatNum);
    if (forTStat != null) removeListener(theListener, forTStat);
  }
  
  public void removeAllListeners(TStatListenerI theListener) {
    synchronized(listenerList) {
      TStatListener varListener = null;
      Iterator<TStatListener> theListeners = listenerList.iterator();
      while(theListeners.hasNext()) {
        varListener = theListeners.next();

        if (varListener.theListener != theListener) continue;
        theListeners.remove();
      }
    }
  }
  
  public void fireTStatChangedEvent(TStatI theTStat, TStatSchemeI theScheme, TStatSetpointI theSetpoint, TStatRuntimeI theRuntime, TStatEventI.EventReason theReason) {
    TStatEvent theEvent = null;
    
    // Create the appropriate event
    if (theScheme == null) {
      if (theRuntime == null) 
        theEvent = new TStatEvent(theTStat, theReason);       
      else
        theEvent = new TStatEvent(theTStat, theRuntime, theReason);
    } else {
      if (theSetpoint == null) 
        theEvent = new TStatEvent(theTStat, theScheme, theReason);
      else
        theEvent = new TStatEvent(theTStat, theScheme, theSetpoint, theReason);
    }
    
    for (TStatListener theListener: listenerList) {
      if (theListener.isInterested(theTStat)) try {
        theListener.getListener().handleTStatMessage(theEvent);
      } catch (Throwable dispatchException) { 
        System.err.println("Exception dispatching tstat message -- " + dispatchException.getMessage());
        dispatchException.printStackTrace();
      }
    }
    theEvent.releaseResources();
  }
  public void fireTStatChangedEvent(TStatI theTStat, TStatSchemeI theScheme, TStatSetpointI theSetpoint, TStatEventI.EventReason theReason) {
    fireTStatChangedEvent(theTStat, theScheme, theSetpoint, null, theReason);
  }
  public void fireTStatChangedEvent(TStatI theTStat, TStatSchemeI theScheme, TStatEventI.EventReason theReason) {
    fireTStatChangedEvent(theTStat, theScheme, null, null, theReason);
  }
  public void fireTStatChangedEvent(TStatI theTStat, TStatRuntimeI theRuntime) {
    fireTStatChangedEvent(theTStat, null, null, theRuntime, TStatEventI.EventReason.RUNTIME_UPDATED);
  }
  public void fireTStatChangedEvent(TStatI theTStat, TStatEventI.EventReason theReason) {
    fireTStatChangedEvent(theTStat, null, null, null, theReason);
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI tstatDevice, xPL_MessageI theMessage) {
    if (this.tstatDevice != tstatDevice) return;
    
    // Make sure it's something we might be interested in
    if (!theMessage.getSchemaClass().equalsIgnoreCase("tstat")) return;
    
    // Get the tstat #
    int theTstatNum = theMessage.getIntNamedValue("TSTAT", -1);
    
    // Bail out if missing anything important
    if ((theTstatNum < 0) || (theTstatNum > 9)) return;
    
    // Dispatch it
    ((TStat) getTStat(theTstatNum)).handleTStatXPLEvent(theMessage);
  }
}
