/*
 * TStatScheme.java
 *
 * Created on December 22, 2004, 12:27 PM
 */

package org.cdp1802.tstat;

import java.util.Vector;
import org.cdp1802.xpl.xPL_MessageI;

/**
 *
 * @author gduprey
 */
public class TStatScheme implements TStatSchemeI {
  TStat theTStat = null;
  String schemeName = null;
  Vector<TStatSetpointI> setpointList = new Vector<TStatSetpointI>();
  
  TStatScheme(TStat theTStat, String schemeName) { 
    this.theTStat = theTStat;
    this.schemeName = schemeName;
  }
  
  public TStatI getTStat() { return theTStat; }
  public String getName() { return schemeName; }

  public int getSetpointCount() { return setpointList.size(); }
  public TStatSetpointI getSetpointAt(int setpointIndex) {
    synchronized(setpointList) {
      if ((setpointIndex < 0) || (setpointIndex >= setpointList.size())) return null;
      return setpointList.get(setpointIndex);
    }
  }

  public int getIndexOfSetpoint(TStatSetpointI theSetpoint) {
    synchronized(setpointList) {
      return setpointList.indexOf(theSetpoint);
    }
  }
  
  public TStatSetpointI getSetpointNamed(String setpointName) {
    if ((setpointName == null) || (setpointName.length() == 0)) return null;
    
    synchronized(setpointList) {
      for (TStatSetpointI theSetpoint: setpointList) {
        if (theSetpoint.getName().equalsIgnoreCase(setpointName)) return theSetpoint;
      }
    }
    return null;
  }
  
  TStatSetpointI findByStartTime(int startTime) {
    synchronized(setpointList) {
      // Prevent dup times
      for (TStatSetpointI theSetpoint: setpointList) {
        if (theSetpoint.getStartTime() == startTime) return theSetpoint; 
      }
    }
    
    return null;
  }

  TStatSetpoint addNewSetpoint(String setpointName, int setpointTemp, int setpointTime, boolean fanOn) {
    // Simple sanity checkss
    if ((setpointName == null) || (setpointName.length() == 0) || (setpointTemp < 40) || (setpointTemp > 120) || (setpointTime < 0) || (setpointTime >= 1440)) return null;
    
    synchronized(setpointList) {
      // Prevent dup names
      if (getSetpointNamed(setpointName) != null) return null;
      
      // Prevent dup times
      if (findByStartTime(setpointTime) != null) return null;
      
      // Create a new setpoint
      TStatSetpoint newSetpoint = new TStatSetpoint(this, setpointName, setpointTemp, setpointTime, fanOn);
      
      // Figure out where to insert it
      int setpointCount = setpointList.size();
      boolean setpointInserted = false;
      for (int theIndex = 0; theIndex < setpointCount; theIndex++) {
        if (setpointList.get(theIndex).getStartTime() > setpointTime) {
          setpointList.insertElementAt(newSetpoint, theIndex);
          setpointInserted = true;
          break;
        }
      }
      if (!setpointInserted) setpointList.add(newSetpoint);
      
      return newSetpoint;
    }
  }
  
  public TStatSetpointI createNewSetpoint(String setpointName, int setpointTemp, int setpointTime, boolean fanOn) {
    TStatSetpoint newSetpoint = addNewSetpoint(setpointName, setpointTemp, setpointTime, fanOn);
    if (newSetpoint == null) return null;
    newSetpoint.sendSetpointMessage("ADD_SET_POINT", setpointName, setpointTemp, setpointTime, fanOn);
    return newSetpoint;
  }
  
  public void deleteSetpoint(TStatSetpointI theSetpoint) {
    if (theSetpoint == null) return;

    synchronized(setpointList) {
      ((TStatSetpoint) theSetpoint).sendSetpointMessage("DELETE_SET_POINT");
      setpointList.remove(theSetpoint);
    }
  }

  public void deleteSetpoint(String setpointName) {
    synchronized(setpointList) {
      deleteSetpoint(getSetpointNamed(setpointName));
    }
  }
  
  void handleSetpointMessage(xPL_MessageI theMessage, String whatChanged) {
    if (whatChanged.equals("NEW_SET_POINT") || whatChanged.equals("UPDATE_SET_POINT")) {
      // Parse out the important stuff
      String theName = theMessage.getNamedValue("SET_POINT_NAME");
      int theTemp = theMessage.getIntNamedValue("SET_POINT_TEMP");
      int theTime = TStatSetpoint.parseTime(theMessage.getNamedValue("SET_POINT_TIME"));
      boolean fanOn = theMessage.getNamedValue("SET_POINT_FAN").equalsIgnoreCase("ON");
      TStatSetpoint theSetpoint = null;

      synchronized(setpointList) {
        // Handle an update
        if (whatChanged.equals("UPDATE_SET_POINT")) {
          theSetpoint = (TStatSetpoint) getSetpointNamed(theName);
          if (theSetpoint != null) {
            boolean setpointChanged = false;
            if (theSetpoint.getSetpointTemp() != theTemp) {
              theSetpoint.setpointTemp = theTime;
              setpointChanged = true;
            }

            if (theSetpoint.getStartTime() != theTime) {
              theSetpoint.startTime = theTime;
              setpointChanged = true;
            }

            if (theSetpoint.isFanOn() != fanOn) {
              theSetpoint.fanOn = fanOn;
              setpointChanged = true;
            }

            if (setpointChanged) {
              theTStat.tstatManager.fireTStatChangedEvent(theTStat, this, theSetpoint, TStatEventI.EventReason.SCHEME_SETPOINT_UPDATED);
            }

            return;
          }
        }

        // Handle a new add (or failed update, so do an implicit add)
        theSetpoint = addNewSetpoint(theName, theTemp, theTime, fanOn);
        if (theSetpoint == null) return;
        theTStat.tstatManager.fireTStatChangedEvent(theTStat, this, theSetpoint, TStatEventI.EventReason.SCHEME_SETPOINT_ADDED);
      }
      return;
    }
    
    if (whatChanged.equals("DELETE_SET_POINT")) {
      synchronized(setpointList) {
        String theName = theMessage.getNamedValue("SET_POINT_NAME");
        TStatSetpoint theSetpoint = (TStatSetpoint) getSetpointNamed(theName);
        if (theSetpoint == null) return;
        theTStat.tstatManager.fireTStatChangedEvent(theTStat, this, theSetpoint, TStatEventI.EventReason.SCHEME_SETPOINT_DELETED);
        setpointList.remove(theSetpoint);
      }
      return;
    }
  }
  
  
  void releaseResources() {
    for (TStatSetpointI theSetpoint: setpointList) {
      ((TStatSetpoint) theSetpoint).releaseResources();
    }
    
    theTStat = null;
    schemeName = null;
    setpointList.clear();
    setpointList.trimToSize();
    setpointList = null;
  }
}
