/*
 * TStatSetpoint.java
 *
 * Created on December 22, 2004, 12:32 PM
 */

package org.cdp1802.tstat;

import java.text.SimpleDateFormat;
import org.cdp1802.xpl.xPL_MutableMessageI;

/**
 *
 * @author gduprey
 */
public class TStatSetpoint implements TStatSetpointI {
  public static int parseTime(String theTime) {
    int delimPtr = theTime.indexOf(":");
    if (delimPtr == -1) return -1;
    int theHour = 0, theMinute = 0;
    
    try { 
      theHour = Integer.parseInt(theTime.substring(0, delimPtr));
    } catch (Exception anyError) { return -1; }
    
    try { 
      theMinute = Integer.parseInt(theTime.substring(delimPtr + 1));
    } catch (Exception anyError) { return -1; }
    
    return (theHour * 60) + theMinute;
  }
  
  public static String formatTime(int theIntTime) {
    StringBuffer theTime = new StringBuffer(6);
    int theHour = theIntTime / 60;
    int theMinute = theIntTime % 60;
    
    if (theHour < 10) theTime.append('0');
    theTime.append(theHour);
    theTime.append(':');
    if (theMinute < 10) theTime.append('0');
    theTime.append(theMinute);
    
    return theTime.toString();
  }
  
  TStatScheme theScheme = null;
  xPL_MutableMessageI schemeMessage = null;
  
  String setpointName = "";
  int setpointTemp = 0;
  int startTime = 0;
  boolean fanOn = false;
  
  TStatSetpoint(TStatScheme theScheme, String theName, int theTemp, int theTime, boolean theFanOn) {
    this.theScheme = theScheme;
    this.setpointName = theName;
    this.setpointTemp = theTemp;
    this.startTime = theTime;
    this.fanOn = theFanOn;
  }
  
  public TStatSchemeI getScheme() { return theScheme; }
  public String getName() { return setpointName; }

  void sendSetpointMessage(String theCommand, String theName, int theTemp, int theTime, boolean theFanOn) {
    if (schemeMessage == null) schemeMessage = theScheme.theTStat.getSchemeMessage();
    
    synchronized(schemeMessage) {
      schemeMessage.clearMessageBody();
      schemeMessage.addNamedValue("COMMAND", theCommand);
      schemeMessage.addNamedValue("TSTAT", theScheme.getTStat().getTStatNum());
      schemeMessage.addNamedValue("SCHEME_NAME", theScheme.getName());
      schemeMessage.addNamedValue("SET_POINT_NAME", theName);
      schemeMessage.addNamedValue("SET_POINT_TEMP", theTemp);
      schemeMessage.addNamedValue("SET_POINT_TIME", theTime);
      schemeMessage.addNamedValue("SET_POINT_FAN", (theFanOn ? "ON" : "AUTO"));
      theScheme.theTStat.getDevice().sendMessage(schemeMessage);
    }
  }
  void sendSetpointMessage(String theCommand) {
    sendSetpointMessage(theCommand, setpointName, setpointTemp, startTime, fanOn);
  }
  
  public int getSetpointTemp() { return setpointTemp; }
  public void setSetpointTemp(int newTemp) {
    if (newTemp == setpointTemp) return;
    sendSetpointMessage("UPDATE_SET_POINT", setpointName, newTemp, startTime, fanOn);
  }
  
  public int getStartTime() { return startTime; }
  public String getTextStartTime() { return formatTime(startTime); }
  
  public void setStartTime(int newStartTime) {
    if ((newStartTime == startTime) || (newStartTime < 0) || (newStartTime >= 1440)) return;
    sendSetpointMessage("UPDATE_SET_POINT", setpointName, setpointTemp, newStartTime, fanOn);
  }
  public void setStartTime(String newStartTime) {
    int newTime = parseTime(newStartTime);
    if (newTime != -1) setStartTime(newTime);
  }
  
  public boolean isFanOn() { return fanOn; }
  public void setFanOn(boolean newFanOn) {
    if (newFanOn == fanOn) return;
    sendSetpointMessage("UPDATE_SET_POINT", setpointName, setpointTemp, startTime, newFanOn);
  }
  
  void releaseResources() {
    theScheme = null;
    schemeMessage = null;
    setpointName = null;
  }
}
