/*
 * MutableNamedValues.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */
package org.cdp1802.xpl;

/**
 *
 * @author  gerry
 */
public class MutableNamedValues extends NamedValues implements MutableNamedValuesI {
  MutableNamedValues() {  }

  public void clear() { 
    synchronized(namedValues) {
      namedValues.clear(); 
    }
  }

  public int removeNamedValues(String theName) {
    int valueCount = 0;
    NamedValueI namedValue = null;
    
    synchronized(namedValues) {
      int nvCount = namedValues.size();
      
      for (int nvIndex = nvCount - 1; nvIndex >= 0; nvIndex--) {
        if ((namedValue = namedValues.get(nvIndex)).getName().equalsIgnoreCase(theName)) {
          valueCount++;
          namedValues.remove(nvIndex);
        }
      }
    }
    
    return valueCount;
  }
  
  public MutableNamedValueI setNamedValue(String theName, String theValue) {
    synchronized(namedValues) {
      int nvCount = namedValues.size();
      MutableNamedValue namedValue = null, usedNamedValue = null;
      
      for (int nvIndex = 0; nvIndex < nvCount; nvIndex++) {
        if ((namedValue = (MutableNamedValue) namedValues.get(nvIndex)).getName().equalsIgnoreCase(theName)) {
          if (usedNamedValue == null) {
            namedValue.setValue(theValue);  
            usedNamedValue = namedValue;
            continue;
          }
          
          // These are extra/dups -- dump them
          namedValues.remove(nvIndex);
          nvCount--;
          nvIndex--;
        }
      }
      
      // If we did set the value, we are done
      if (usedNamedValue != null) return usedNamedValue;
      
      // Add a new one
      return addNamedValue(theName, theValue);
    }
  }
  
  public MutableNamedValueI setNamedValue(String theName, int theValue) {
    return setNamedValue(theName, String.valueOf(theValue));
  }
  
  public MutableNamedValueI setNamedValue(String theName, boolean theValue) {
    return setNamedValue(theName, (theValue ? "1" : "0"));
  }
  
  public MutableNamedValueI addNamedValue(String theName, String theValue) {
    MutableNamedValueI theNamedValue = new MutableNamedValue(theName, theValue);
    namedValues.add(theNamedValue);
    return theNamedValue;
  }
  
  public MutableNamedValueI addNamedValue(String theName, int theValue) {
    return addNamedValue(theName, String.valueOf(theValue));
  }
  
  public MutableNamedValueI addNamedValue(String theName, boolean theValue) {
    return addNamedValue(theName, (theValue ? "1" : "0"));
  }

  public NamedValuesI getImmutableNamedValues() {
    synchronized(namedValues) {
      NamedValues immutableNamedValues = new NamedValues();
      immutableNamedValues.namedValues.addAll(namedValues);
      return immutableNamedValues;
    }
  }
}
