/*
 * MutableNameValueListI.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */

package org.cdp1802.xpl;

/**
 *
 * @author  gerry
 */
public interface MutableNamedValuesI extends NamedValuesI {
  /**
   * Clear/delete all named values
   */
  public void clear();

  /**
   * Add the passed named/value.  If you add a value using a name which already
   * exists, the original named value is left alone and this new one is added to
   * the end.
   *
   * If you attempt to set a value of null, the value is converted into an empty
   * string ("").  An attempt to add a null or empty name is ignored/rejected.
   *
   * @param theName name of value to add
   * @param theValue value to add with this name
   */
  public MutableNamedValueI addNamedValue(String theName, String theValue);

  /**
   * Add the passed named/value.  If you add a value using a name which already
   * exists, the original named value is left alone and this new one is added to
   * the end.
   *
   * An attempt to add a null or empty name is ignored/rejected.
   *
   * @param theName name of value to add
   * @param theValue value to add with this name
   */
  public MutableNamedValueI addNamedValue(String theName, int theValue);

  /**
   * Add the passed named/value.  If you add a value using a name which already
   * exists, the original named value is left alone and this new one is added to
   * the end.
   *
   * An attempt to add a null or empty name is ignored/rejected.
   *
   * @param theName name of value to add
   * @param theValue value to add with this name
   */
  public MutableNamedValueI addNamedValue(String theName, boolean theValue);
  
  /**
   * Remove all occurances of the values with the passed name.  The number of
   * values actually removed is returned.
   *
   * @param theName name of the values to remove
   * @return the number of values actually removed
   */
  public int removeNamedValues(String theName);

  /**
   * Set a named value.  If there is an existing value OR values for the
   * name, they are purged and the single name/value installed.  THis is
   * a convience method and does the equivilent of a removeNamedValues()
   * followed by an addNamedValue() call.
   *
   * An attempt to add a null or empty name is ignored/rejected.
   *
   * @param theName name of value to add
   * @param theValue value to add with this name
   */
  public MutableNamedValueI setNamedValue(String theName, String theValue);
  
  /**
   * Set a named value.  If there is an existing value OR values for the
   * name, they are purged and the single name/value installed.  THis is
   * a convience method and does the equivilent of a removeNamedValues()
   * followed by an addNamedValue() call.
   *
   * An attempt to add a null or empty name is ignored/rejected.
   *
   * @param theName name of value to add
   * @param theValue value to add with this name
   */
  public MutableNamedValueI setNamedValue(String theName, int theValue);

  /**
   * Set a named value.  If there is an existing value OR values for the
   * name, they are purged and the single name/value installed.  THis is
   * a convience method and does the equivilent of a removeNamedValues()
   * followed by an addNamedValue() call.
   *
   * An attempt to add a null or empty name is ignored/rejected.
   *
   * @param theName name of value to add
   * @param theValue value to add with this name
   */
  public MutableNamedValueI setNamedValue(String theName, boolean theValue);

  /**
   * Return an immutable set of namevalues based on this current population. 
   * The list is a copy, not a wrapper, so additional changes to these name/values
   * after this call will NOT be reflected in the object previously returned.
   *
   * @return Unchangable copy of this NameValues
   */
  public NamedValuesI getImmutableNamedValues();
}
