/*
 * NamedValueI.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */

package org.cdp1802.xpl;

/**
 *
 * @author  gerry
 */
public interface NamedValueI {
  /**
   * Return the name of this named/value pair
   *
   * @return name of this named value
   */
  public String getName();

  /**
   * Return the value of this named value.  The
   * returned value is never null
   *
   * @return value of this named value
   */
  public String getValue();
  
  /** 
   * Attempt to return the value of this named value as 
   * an integer.  If the current value is not compatible
   * with an integer, then -1 is returned. 
   *
   * This is a convience method only -- internally values are
   * always stored as strings.  An empty value is returned as 0.
   *
   * @return integer version of the current value or -1.
   */
  public int getIntValue();
  
  /** 
   * Attempt to return the value of this named value as a boolean.
   * For the purposes of a true state, the following values are 
   * allows 'true', 'yes', 't', 'y' or '1' (number one).  Case is
   * not important.
   *
   * This is a convience method only -- internally values are
   * always stored as strings.  An empty value is returned as false.
   *
   * @return boolean version of current value
   */
  public boolean getBoolValue();
  
  /**
   * Release resources for this named value.  After this call, the 
   * instance of this object is invalid and unusable
   */
   public void releaseResources();
}
