/*
 * xPL_DeviceChangeEventI.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */

package org.cdp1802.xpl.device;

import org.cdp1802.xpl.xPL_IdentifierI;
import org.cdp1802.xpl.xPL_FilterI;

/**
 * Device Changed Event
 * @author gerry
 */
public interface xPL_DeviceChangeEventI {
  public enum ChangeReason {
    /**
     * Device has been successfully enabled
     */
    DEVICE_ENABLED, 
    /**
     * Device has been successfully disabled
     */
    DEVICE_DISABLED,

    /**
     * All Filters for the device have been cleared
     */
    FILTERS_CLEARED, 
    /**
     * A new filter has been installed
     */
    FILTER_ADDED,
            
    /**
     * All groups for this device have been cleared
     */
    GROUPS_CLEARED, 
    /**
     * A group has been assigned to this device
     */
    GROUP_ADDED,
            
    /**
     * All configuration atems for this device have been cleared
     */
    CONFIGURATION_ITEMS_CLEARED,

    /**
     * A new configurable item has been added to this device
     */
    CONFIGURATION_ITEM_ADDED,
            
    /**
     * All configurable elements for this device have been preserved
     */
    CONFIGURATION_PRESERVED,
            
    /**
     * All configurable elements for this device have been restored
     */
    CONFIGURATION_RESTORED,
            
    /**
     * A configuration message has been received and it's elements
     * installed into the device
     */
    CONFIGURATION_CHANGED,
    
    /**
     * A configurable device has been configured.  This can happen because the
     * configuration was loaded locally or a configuration was sent from a 
     * configuration manager.  However, it only occurs once (subsequent
     * reconfiguration will not fire this event) and may not happen at all (non
     * configurable devices and configurable =devices where a Configurmation
     * Manager never replies).  This will always be proceeded by a 
     * CONFIGURATION_CHANGED event.
     */
    DEVICE_CONFIGURED }
  
  /**
   * Get the reason code this change describes
   * @return reason code describing this change
   */
  public ChangeReason getChangeReason();
  
  /**
   * Get the device this change applies to
   *
   * @return Device related to this change
   */
  public xPL_DeviceI getDevice();
  
  /**
   * Get group message describes (for GROUP_ADDED)
   *
   * @return added group or null for non-group added messages
   */
  public xPL_IdentifierI getGroup();
  
  /**
   * Get filter message describes (for FILTER_ADDED)
   *
   * @return filter added to device or null if non-filter message
   */
  public xPL_FilterI getFilter();
  
  /**
   * Get the configurable item message describes (for CONFIGURATION_ITEM_ADDED)
   *
   * @return configurable item or null in non-config-added message
   */
  public xPL_DeviceConfigItemI getConfigItem();
}
