/*
 * LightingDeviceEvent.java
 *
 * Copyright (c) 2005, Gerald R Duprey Jr
 * More info at http://www.xPL4Java.org
 */

package org.cdp1802.xpl.lighting;

/**
 * An event descring something changing to a device
 *
 * @author gerry
 */
public class LightingDeviceEvent {
  public enum EventCode { 
    /** Device state/dim level has changed */
    DEVICE_STATE_CHANGED, 
  };
  
  LightingDevice eventDevice = null;
  EventCode eventCode = null;
  int eventChannel = LightingDevice.ALL_CHANNELS;
  
  LightingDeviceEvent(LightingDevice theDevice, EventCode theEventCode, int theChannel) {
    this.eventDevice = theDevice;
    this.eventCode = theEventCode;
    this.eventChannel = theChannel;
  }

  /**
   * Get the device associated with this event
   *
   * @return device associated with this event
   */
  public LightingDevice getDevice() { return eventDevice; }

  /**
   * Get the event code describing this event
   *
   * @return event code describing this event
   */
  public EventCode getEventCode() { return eventCode; }
  
  /**
   * Get channel this event applies to.
   *
   * @return channel this event applies to
   */
  public int getChannel() { return eventChannel; }
  
  
  void releaseResources() {
    eventDevice = null;
    eventCode = null;
  }
}
