/*
 * LightingGatewayEvent.java
 *
 * Copyright (c) 2005, Gerald R Duprey Jr
 * More info at http://www.xPL4Java.org
 */

package org.cdp1802.xpl.lighting;

/**
 * Used to alert of changes and activity of interest to the 
 * this gateway
 *
 * @author gerry
 */
public class LightingGatewayEvent {
  public enum EventCode { 
    /**
     * Sent when discovery is started
     */
    DISCOVERY_STARTED,
    
    /** Sent when discovery ended in failure */
    DISCOVERY_FAILED,
    
    /** 
     * Sent when discovery succeeds
     */
    DISCOVERY_COMPLETE,
    
    /** 
     * Sent when the LightingGateway first hears from a remote gateway and binds to it.  
     */
    REMOTE_GATEWAY_BOUND,
    
    /**
     * Sent when the gateway says it changed (you should start/restart discovery at that point)
     */
    REMOTE_GATEWAY_CHANGED,
    
    /**
     * Sent when the remote gateway goes away (announces it's shutdown) 
     */
    REMOTE_GATEWAY_UNBOUND
  };
  
  EventCode eventCode = null;
  
  LightingGatewayEvent(EventCode theEventCode) {
    this.eventCode = theEventCode;
  }
  
  /** 
   * Get the event code describing this event
   *
   * @return the event code for this event
   */
  public EventCode getEventCode() { return eventCode; }
  
  void releaseResources() {
    eventCode = null;
  }
}
