/*
 * LightingMessage.java
 *
 * Copyright (c) 2005, Gerald R Duprey Jr
 * More info at http://www.xPL4Java.org
 */

package org.cdp1802.xpl.lighting;


/**
 * An abstracted representation of a message. 
 * 
 * @author gerry
 */
class LightingMessage {
  public enum MsgType { 
    ACTIVATE,  /** Got or send a scene activate */
    DEACTIVATE, /** Got or send a scene deactiave */
    GOTO, /** Got or send a GOTO (scene or device) */
    QUERY_STATE, /** Send a query state request (device) */
    STATE_REPORT /** Got a state report (device) */
  }
  
  private MsgType messageType = null;
  private LightingDevice theDevice = null;
  private LightingScene theScene = null;
  private LightingDevice sourceDevice = null;
  private int theLevel = LightingDevice.UNASSIGNED_DEVICE_STATE;
  private float theFadeRate = LightingDevice.DEFAULT_FADE_RATE;
  private int theChannel = LightingDevice.ALL_CHANNELS;
  
  public LightingMessage(LightingDevice theDevice, MsgType theMessageType, int theLevel, float theFadeRate, int forChannel) {
    this.theDevice = theDevice;
    this.messageType = theMessageType;
    this.theLevel = theLevel;
    this.theFadeRate = theFadeRate;
    this.theChannel = forChannel;
  }

  public LightingMessage(LightingDevice theDevice, MsgType theMessageType) {
    this.theDevice = theDevice;
    this.messageType = theMessageType;
  }

  public LightingMessage(LightingScene theScene, MsgType theMessageType, int theLevel, float theFadeRate, LightingDevice sourceDevice) {
    this.theScene = theScene;
    this.messageType = theMessageType;
    this.theLevel = theLevel;
    this.theFadeRate = theFadeRate;
    this.sourceDevice = sourceDevice;
  }

  public LightingMessage(LightingScene theScene, MsgType theMessageType, int theLevel, float theFadeRate) {
    this(theScene, theMessageType, theLevel, theFadeRate, null);
  }

  public LightingMessage(LightingScene theScene, MsgType theMessageType, int theLevel) {
    this(theScene, theMessageType, theLevel, LightingDevice.DEFAULT_FADE_RATE);
  }

  public LightingMessage(LightingScene theScene, MsgType theMessageType) {
    this(theScene, theMessageType, 0);
  }

  public boolean isSceneMessage() { return theScene != null; }
  public boolean isDeviceMessage() { return theDevice != null; }
  
  public LightingDevice getDevice() { return theDevice; }
  public LightingScene getScene() { return theScene; }
  
  public MsgType getMessageType() { return messageType; }
  
  public int getLevel() { return theLevel; }
  public float getFadeRate() { return theFadeRate; }
  public int getChannel() { return theChannel; }
  
  public LightingDevice getSourceDevice() { return sourceDevice; }
  
  
  public void releaseResources() {
    messageType = null;
    theDevice = null;
    theScene = null;
    sourceDevice = null;
    theLevel = -1;
    theFadeRate = -1;
    theChannel = -1;
  }
}
