/*
 * LightingSceneDevice.java
 *
 * Copyright (c) 2005, Gerald R Duprey Jr
 * More info at http://www.xPL4Java.org
 */

package org.cdp1802.xpl.lighting;

/**
 * This describes the association of a scene and a device.  It not only is
 * the record that "connects" the two, but it also describes what level the
 * device should go to when the scene is activated and what fade rate to 
 * use to do it.
 *
 * @author gerry
 */
public class LightingSceneDevice {
  LightingDevice theDevice = null;
  LightingScene theScene = null;
  int theLevel = 100;
  float theFadeRate = LightingDevice.DEFAULT_FADE_RATE;
  int theChannel = LightingDevice.ALL_CHANNELS;
  
  LightingSceneDevice(LightingScene theScene, LightingDevice theDevice, int theLevel, float theFadeRate, int theChannel) {
    this.theScene = theScene;
    this.theDevice = theDevice;
    this.theLevel = theLevel;
    this.theFadeRate = theFadeRate;
    this.theChannel = theChannel;
  }
  
  /**
   * Get the device described by this association
   *
   * @return device associated with the scene
   */
  public LightingDevice getDevice() { return theDevice; }
  
  /**
   * Get the scene described by this association
   *
   * @return scene associated with this device
   */
  public LightingScene getScene() { return theScene; }

  /**
   * Get the level the device should go to when the scene is
   * activated.
   *
   * @return level to set device when scene is activated
   */
  public int getLevel() { return theLevel; }
  
  /**
   * Get the rate the device should change to it's new level 
   * when the scene is activated.
   *
   * @return fade rate for device when scene is activated
   */
  public float getFadeRate() { return theFadeRate; }

  /**
   * Get the channel this scene applies to on the device.
   *
   * @return channel for this scene
   */
  public int getChannel() { return theChannel; }
  
  /**
   * Get a printable description of this association
   *
   * @return printable associated including device name/id and scene name/id
   */
  public String toString() {
      return "Device " + theDevice + " tied to scene " + theScene;
  }
  
  void releaseResources() {
    theDevice = null;
    theScene = null;
  }
}
