/*
 * ThreadPoolRunnable.java
 *
 * Copyright (c) 2005, Gerald R Duprey Jr
 * More info at http://www.xPL4Java.org
 */

package org.cdp1802.xpl.lighting;

import java.util.*;

// This class is the thread that actually runs the request
class ThreadPoolRunner extends Thread {
  
  static int threadIDTracker = 0;
  ThreadPool threadPool = null;
  ThreadPool.TaskInfo curTask = null;
  boolean timeToDie = false;
  boolean threadBusy = true;
  int threadID = threadIDTracker++;

  ThreadPoolRunner(ThreadPool thePool, ThreadGroup theGroup, String threadName) {
    super(theGroup, threadName);
    threadPool = thePool;
  }

  public String toString() {
    if (curTask == null) 
      return "Thread #" + threadID + " (idle)";
    else
      return "Thread #" + threadID + " - " + curTask.taskDescription;
  }
  
  void setThreadDescription(String theDescription) {
    curTask.setDescription(theDescription);
  }
  String getThreadDescription() { return curTask.getDescription(); }
  
  public void run() {
    for(;;) {
      setName("Idle");
      
      // See if it's time to stop running
      if (timeToDie) return;

      // Get the next request from the request queue
      synchronized(threadPool.requestQueue) {
        // See if the queue is empty.  If so, wait for something
        // to be put into it.  Otherwise, fetch a request from the queue
        if (threadPool.isPoolOnHold() || (threadPool.requestQueue.size() == 0)) {
          try {
            threadPool.idleThreadCount++;
            if (LightingGateway.DEBUG_MODE) System.out.println("THREAD_" + threadID + ":: Idled");
            curTask = null;
            synchronized(this) { threadBusy = false; }
            threadPool.requestQueue.wait();
          } catch (InterruptedException intError) { }
          finally {
            threadPool.idleThreadCount--;
            synchronized(this) { threadBusy = true; }
          }

          // After waking up, try again to get a request
          continue;
        } else {
          // Fetch the first request, in order, from the queue
          try {
            curTask = (ThreadPool.TaskInfo) threadPool.requestQueue.removeLast();
            setName(curTask.getDescription());
          } catch (NoSuchElementException emptyQueueError) { 
            continue;
          } catch (Exception otherError) { 
            if (LightingGateway.DEBUG_MODE) {
              System.err.println("THREAD_POOL:: Exception setting up new threaded task -- " + otherError.getMessage() + " -- ignored");
              otherError.printStackTrace();
            }
            continue;
          }
        }
      }

      // Execute this request
      try {
        if (LightingGateway.DEBUG_MODE) System.out.println("THREAD_" + threadID + ":: " + curTask.taskDescription + " -- engaged");
        curTask.theTask.run();
      } catch (Exception runtimeException) {
        System.err.println("THREAD_" + threadID + ":: EXCEPTION: " + runtimeException.getMessage());
        if (LightingGateway.DEBUG_MODE) runtimeException.printStackTrace();
      } catch (Error runtimeError) {
        System.err.println("THREAD_" + threadID + ":: ERROR: " + runtimeError.getMessage());
        if (LightingGateway.DEBUG_MODE) runtimeError.printStackTrace();
      }

      // Clear the reference to the request
      if (LightingGateway.DEBUG_MODE) System.out.println("THREAD_" + threadID + ":: " + curTask.taskDescription + " -- Completed/Released");
      curTask = null;

      // And we are done - go back for another
      continue;
    }
  }
}


