/*
 * BasePluginItem.java
 *
 * Created on October 6, 2005, 11:14 AM
 */

package org.cdp1802.xpl.plugins;

import java.util.*;
import org.jdom.*;

/**
 *
 * @author gerry
 */
public abstract class BasePluginItem {
  protected BasePluginItem parent = null;
  protected BasePluginItem copiedFrom = null;
  protected Element nodeElement = null;
  protected ArrayList<PluginEventListenerI> listenerList = null;
  protected boolean suppressEvents = false;
  
  public BasePluginItem getParent() { return parent; }
  
  public VendorPlugin getPlugin() {
    if (this instanceof VendorPlugin) return (VendorPlugin) this;
    if (parent == null) return null;
    return parent.getPlugin();
  }
  public Element getNodeElement() { return nodeElement; }
  
  protected void markModified() {
    if (parent != null) parent.markModified();
  }

  public void addListener(PluginEventListenerI theListener) {
    if (parent != null) {
      parent.addListener(theListener);
      return;
    }
    
    synchronized(this) {
      if (listenerList == null) listenerList = new ArrayList<PluginEventListenerI>();
      if (listenerList.contains(theListener)) return;
      listenerList.add(theListener);
    }
  }

  public void removeListener(PluginEventListenerI  theListener) {
    if (parent != null) {
      parent.removeListener(theListener);
      return;
    }
    
    synchronized(this) {
      if (listenerList == null) return;
      
      listenerList.remove(theListener);
    }
  }
  
  protected void fireEvent(BasePluginItem theItem, PluginEvent.EventCode theEventCode, int eventIndex) {
    // Pass it up a level if we're not at the top of the food chain
    if (parent != null) {
      parent.fireEvent(theItem, theEventCode, eventIndex);
      return;
    }
    
    // Make sure there is something to fire
    if ((listenerList == null) || suppressEvents) return;
    
    // Create an event
    PluginEvent theEvent = new PluginEvent(theItem, theEventCode, eventIndex);
    if ((theEvent.getEventIndex() == -1) && !(theItem instanceof VendorPlugin)) return;
    
    int listenerCount = listenerList.size();
    for (int listenerIndex = listenerCount - 1; listenerIndex >= 0; listenerIndex--) {
      try {
        listenerList.get(listenerIndex).handlePluginEvent(theEvent);
      } catch (Throwable ignoredException) { }
    }
    
    // Free event resources
    theEvent.releaseResources();
  }
  
  protected void fireEvent(BasePluginItem theItem, PluginEvent.EventCode theEventCode) {
    fireEvent(theItem, theEventCode, -1);
  }
  protected void fireEvent(PluginEvent.EventCode theEventCode, int eventIndex) {
    fireEvent(this, theEventCode, eventIndex);
  }
  protected void fireEvent(PluginEvent.EventCode theEventCode) {
    fireEvent(this, theEventCode, -1);
  }

  public void clearCopiedFrom() { copiedFrom = null; }
  public BasePluginItem getCopiedFrom() { return copiedFrom; }

  
  public void releaseResources() {
    parent = null;
    copiedFrom = null;
    nodeElement = null;
    
    if (listenerList != null) listenerList.clear();
    listenerList = null;
  }
  
  
}
