/*
 * DeviceAbility
 *
 * Created on September 28, 2005, 9:33 PM
 *
 */

package org.cdp1802.xpl.plugins;

import java.util.*;
import org.jdom.*;

/**
 *
 * @author gerry
 */
public abstract class DeviceAbility extends BasePluginItem {
  List elementList = null;
  ArrayList<DeviceAbilityElement> elementItems = new ArrayList<DeviceAbilityElement>();
  
  DeviceAbility() { }
  
  DeviceAbility(PluginDevice theDevice, Element theElement) {
    this.parent = theDevice;
    this.nodeElement = theElement;
    this.elementList = theElement.getChildren("element");
    
    for (Object anObject: elementList) {
      addElement(new DeviceAbilityElement(this, (Element) anObject));
    }
  }
  
  DeviceAbility(PluginDevice theDevice, String nodeName, String msgType, String theName) {
    this.parent = theDevice;
    this.nodeElement = new Element(nodeName);
    this.nodeElement.setAttribute("msg_type", msgType);
    this.elementList = nodeElement.getChildren("element");
    setName(theName);
  }
  
  abstract void fireAbilityRenamed();
  abstract void fireAbilityChanged();
  
  public String getName() {
    String theText = nodeElement.getAttributeValue("name");
    return (theText == null ? "" : theText);
  }
  
  public void setName(String newName) {
    String curValue = getName();
    
    if ((newName == null) || (newName.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("name", "");
        markModified();
        fireAbilityRenamed();
      }
      return;
    }
    
    if (curValue.equals(newName)) return;
    nodeElement.setAttribute("name", newName);
    markModified();
    fireAbilityRenamed();
  }
  
  public String getDescription() {
    String theText = nodeElement.getAttributeValue("description");
    return (theText == null ? "" : theText);
  }
  
  public void setDescription(String newDescription) {
    String curValue = getDescription();
    
    if ((newDescription == null) || (newDescription.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeAttribute("description");
        markModified();
        fireAbilityChanged();
      }
      return;
    }
    
    if (curValue.equals(newDescription)) return;
    nodeElement.setAttribute("description", newDescription);
    markModified();
    fireAbilityChanged();
  }
  
  public String getMessageType() {
    String theText = nodeElement.getAttributeValue("msg_type");
    return (theText == null ? "" : theText);
  }
  
  public void setMessageType(String newType) {
    newType = newType.toLowerCase();
    String curValue = getMessageType();
    
    if ((newType == null) || (newType.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeAttribute("msg_type");
        markModified();
        fireAbilityChanged();
      }
      return;
    }
    
    if (curValue.equals(newType)) return;
    if (!newType.equals("cmnd") && !newType.equals("stat") && !newType.equals("trig")) return;
  
    nodeElement.setAttribute("msg_type", newType);
    markModified();
    fireAbilityChanged();
  }
  
  public String getMessageSchema() {
    String theText = nodeElement.getAttributeValue("msg_schema");
    return (theText == null ? "" : theText);
  }
  
  public void setMessageSchema(String newSchema) {
    newSchema = newSchema.toLowerCase();
    String curValue = getMessageSchema();
    
    if ((newSchema == null) || (newSchema.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeAttribute("msg_schema");
        markModified();
        fireAbilityChanged();
      } 
      return;
    }
    
    if (curValue.equals(newSchema)) return;
    if (newSchema.indexOf(".") == -1) return;
  
    nodeElement.setAttribute("msg_schema", newSchema);
    markModified();
    fireAbilityChanged();
  }

  public int getElementCount() { return elementItems.size(); }
  public DeviceAbilityElement getElementAt(int elementIndex) {
    if ((elementIndex < 0) || (elementIndex >= elementList.size())) return null;
    return elementItems.get(elementIndex);
  }

  public DeviceAbilityElement getElementNamed(String theName) {
    if ((theName == null) || (theName.length() == 0)) return null;
    
    theName = theName.toLowerCase();
    for (DeviceAbilityElement anElement: elementItems) {
      if (anElement.getName().equals(theName)) return anElement;
    }
    return null;
  }

  public int getIndexOfElement(DeviceAbilityElement theElement) {
    if (theElement == null) return -1;
    return elementItems.indexOf(theElement);
  }
  
  public DeviceAbilityElement createElement(String theName) {
    if ((theName != null) && ((theName.length() == 0) || (getElementNamed(theName) != null))) return null;
    return new DeviceAbilityElement(this, theName);
  }
  
  public boolean addElement(DeviceAbilityElement theElement) {
    if ((theElement == null) || (getElementNamed(theElement.getName()) != null)) return false;

    elementItems.add(theElement);
    
    if (!elementList.contains(theElement.getNodeElement())) elementList.add(theElement.getNodeElement());
    
    theElement.parent = this;
    
    markModified();
    fireEvent(theElement, PluginEvent.EventCode.ELEMENT_ADDED);
    return true;
  }
  
  public boolean removeElementAt(int theIndex) {
    if ((theIndex < 0) || (theIndex >= elementList.size())) return false;
    
    DeviceAbilityElement theElement = elementItems.get(theIndex);
    elementList.remove(theElement.getNodeElement());
    elementItems.remove(theIndex);
    markModified();
    fireEvent(theElement, PluginEvent.EventCode.ELEMENT_REMOVED, theIndex);
    theElement.releaseResources();
    return true;
  }
  
  public void copyInto(DeviceAbility theTarget) {
    boolean setCopiedFrom = (this.copiedFrom != theTarget);
    
    if (setCopiedFrom) theTarget.copiedFrom = this;
    if (theTarget.nodeElement == null) theTarget.nodeElement = new Element(nodeElement.getName());
    if (theTarget.elementList == null) theTarget.elementList = theTarget.nodeElement.getChildren("element");
    
    theTarget.setDescription(getDescription());
    theTarget.setMessageSchema(getMessageSchema());
    theTarget.setMessageType(getMessageType());
    theTarget.setName(getName());
    
    DeviceAbilityElement origElement = null, targetElement = null;

    // Add or update each element
    for (int theIndex = 0; theIndex < getElementCount(); theIndex++) {
      origElement = getElementAt(theIndex);
      
      if ((targetElement = theTarget.getElementNamed(origElement.getName())) == null) {
        targetElement = origElement.copyOf(theTarget);
        if (!theTarget.addElement(targetElement)) 
          System.err.println("  FAILED to add new element " + targetElement.getName() + " to ability");
      } else {
        origElement.copyInto(targetElement);
      }
    }
    
    // Purge obsolete elements
    for (int theIndex = theTarget.getElementCount() - 1; theIndex >= 0; theIndex--) {
      targetElement = theTarget.getElementAt(theIndex);
      if (getElementNamed(targetElement.getName()) != null) continue;
      theTarget.removeElementAt(theIndex);
    }
  }
    
  public DeviceAbility copyOf(PluginDevice copiesParent) {
    try {
      DeviceAbility newCopy = this.getClass().newInstance();
      newCopy.parent = copiesParent;
      copyInto(newCopy);
      return newCopy;
    } catch (Exception anyError) {
      System.err.println("Unable to create new instance/copyOf() DeviceAbility -- " + anyError.getMessage());
      anyError.printStackTrace();
      return null; 
    }
  }
  public DeviceAbility copyOf() { return copyOf(null); }
  
  
  public void releaseResources() {
    super.releaseResources();
    elementList = null;
  }
}
