/*
 * DeviceAbilityElement.java
 *
 * Created on September 28, 2005, 9:42 PM
 *
 */

package org.cdp1802.xpl.plugins;

import java.util.*;
import org.jdom.*;

/**
 *
 * @author gerry
 */
public class DeviceAbilityElement extends BasePluginItem {
  List optionItems = null;
  ArrayList<DeviceAbilityElementOption> elementOptions = new ArrayList<DeviceAbilityElementOption>();
  
  DeviceAbilityElement() { }
  
  DeviceAbilityElement(DeviceAbility theAbility, Element theElement) {
    this.parent = theAbility;
    this.nodeElement = theElement;
    this.optionItems = theElement.getChildren("option");
    
    for (Object anObject: optionItems) {
      addOption(new DeviceAbilityElementOption(this, (Element) anObject));
    }
  }
  
  DeviceAbilityElement(DeviceAbility theAbility, String theName) {
    this.parent = theAbility;
    this.nodeElement = new Element("element");
    this.optionItems = nodeElement.getChildren("option");
    setName(theName);
  }
  
  public String getName() {
    String theText = nodeElement.getAttributeValue("name");
    return (theText == null ? "" : theText.toLowerCase());
  }
  
  public void setName(String newName) {
    String curValue = getName();
    
    if ((newName == null) || (newName.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("name", "");
        markModified();
        fireEvent(PluginEvent.EventCode.ELEMENT_RENAMED);
      }
      return;
    }
    
    newName = newName.toLowerCase();
    
    if (curValue.equals(newName)) return;
    nodeElement.setAttribute("name", newName);
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_RENAMED);
  }
  
  public String getLabel() {
    String theText = nodeElement.getAttributeValue("label");
    return (theText == null ? "" : theText);
  }
  
  public void setLabel(String newLabel) {
    String curValue = getLabel();
    
    if ((newLabel == null) || (newLabel.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("label", "");
        markModified();
        fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newLabel)) return;
    nodeElement.setAttribute("label", newLabel);
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
  }
  
  public String getControlType() {
    String theText = nodeElement.getAttributeValue("control_type");
    return (theText == null ? "textbox" : ((theText.length() == 0) ? "textbox" : theText));
  }
  
  public void setControlType(String newType) {
    String curValue = getControlType();
    
    if ((newType == null) || (newType.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("control_type", "");
        markModified();
        fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newType)) return;
    if (!newType.equals("textbox") && !newType.equals("dropdownlist") && !newType.equals("numeric")) return;
  
    nodeElement.setAttribute("control_type", newType);
    
    if (newType.equals("textbox")) {
      nodeElement.removeAttribute("minval");
      nodeElement.removeAttribute("maxval");
      while (getOptionCount() != 0) removeOptionAt(0);
   } else if (newType.equals("dropdownlist")) {
      nodeElement.removeChild("regexp");
      nodeElement.removeAttribute("minval");
      nodeElement.removeAttribute("maxval");
    } else if (newType.equals("numeric")) {
      nodeElement.removeChild("regexp");
      while (getOptionCount() != 0) removeOptionAt(0);
    }
    
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
  }
  
  public String getConditionalVisibility() {
    String theText = nodeElement.getAttributeValue("conditional-visibility");
    return (theText == null ? "" : theText);
  }
  
  public void setConditionalVisibility(String newVisibility) {
    String curValue = getConditionalVisibility();
    
    if ((newVisibility == null) || (newVisibility.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeAttribute("conditional-visibility");
        markModified();
        fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newVisibility)) return;
  
    nodeElement.setAttribute("conditional-visibility", newVisibility);
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
  }
  
  public String getFormat() {
    Element theChild = nodeElement.getChild("regexp");
    if (theChild == null) return "";
    
    String theText = theChild.getTextTrim();
    return (theText == null ? "" : theText);
  }
  
  public void setFormat(String newRegExp) {
    String curValue = getFormat();
    
    if ((newRegExp == null) || (newRegExp.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeChild("regexp");
        markModified();
        fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newRegExp)) return;
  
    Element theChild = nodeElement.getChild("regexp");
    if (theChild == null) nodeElement.addContent(theChild = new Element("regexp"));
    theChild.setText(newRegExp);
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
  }  
  
  public String getDefaultValue() {
    String theText = nodeElement.getAttributeValue("default");
    return (theText == null ? "" : theText);
  }
  
  public void setDefautlValue(String newValue) {
    String curValue = getDefaultValue();
    
    if ((newValue == null) || (newValue.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("default", "");
        markModified();
        fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newValue)) return;
    nodeElement.setAttribute("default", newValue);
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
  }
  
  public int getMinValue() {
    Attribute theAttr = nodeElement.getAttribute("minval");
    if (theAttr == null) return 0;

    try {
      return theAttr.getIntValue();
    } catch (Exception anything) { return 0; }
  }
  
  public void setMinValue(int newValue) {
    int curValue = getMinValue();
    
    if (curValue == newValue) return;

    nodeElement.setAttribute("minval", String.valueOf(newValue));
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
  }
  
  public int getMaxValue() {
    Attribute theAttr = nodeElement.getAttribute("maxval");
    if (theAttr == null) return 0;

    try {
      return theAttr.getIntValue();
    } catch (Exception anything) { return 0; }
  }
  
  public void setMaxValue(int newValue) {
    int curValue = getMaxValue();
    
    if (curValue == newValue) return;

    nodeElement.setAttribute("maxval", String.valueOf(newValue));
    markModified();
    fireEvent(PluginEvent.EventCode.ELEMENT_UPDATED);
  }
  
  
  public int getOptionCount() { return elementOptions.size(); }

  public DeviceAbilityElementOption getOptionAt(int optionIndex) {
    if ((optionIndex < 0) || (optionIndex >= optionItems.size())) return null;
    return elementOptions.get(optionIndex);
  }

  public DeviceAbilityElementOption getOptionByValue(String theValue) {
    if ((theValue == null) || (theValue.length() == 0)) return null;
    for (DeviceAbilityElementOption theOption: elementOptions) {
      if (theOption.getValue().equals(theValue)) return theOption;
    }
    return null;
  }

  public DeviceAbilityElementOption getOptionByLabel(String theLabel) {
    if ((theLabel == null) || (theLabel.length() == 0)) return null;
    for (DeviceAbilityElementOption theOption: elementOptions) {
      if (theOption.getLabel().equals(theLabel)) return theOption;
    }
    return null;
  }

  public int getIndexOfOption(DeviceAbilityElementOption theOption) {
    if (theOption == null) return -1;
    return elementOptions.indexOf(theOption);
  }

  public DeviceAbilityElementOption createOption(String newLabel, String newValue) {
    if ((newValue != null) && ((newValue.length() == 0) || (getOptionByValue(newValue) != null))) return null;
    if ((newLabel != null) && ((newLabel.length() == 0) || (getOptionByValue(newLabel) != null))) return null;
    
    return new DeviceAbilityElementOption(this, newLabel, newValue);
  }
  
  public boolean addOption(DeviceAbilityElementOption newOption) {
    if ((newOption == null) || (getOptionByValue(newOption.getValue()) != null) || (getOptionByLabel(newOption.getLabel()) != null)) return false;

    
    // Add it in
    elementOptions.add(newOption);
    if (!optionItems.contains(newOption.getNodeElement())) optionItems.add(newOption.getNodeElement());

    // Establish parentage
    newOption.parent = this;

    // And update the world
    markModified();
    fireEvent(newOption, PluginEvent.EventCode.OPTION_ADDED);
    return true;
  }

  public boolean removeOptionAt(int optionIndex) {
    if ((optionIndex < 0) || (optionIndex >= optionItems.size())) return false;
    
    DeviceAbilityElementOption theOption = elementOptions.get(optionIndex);
    
    optionItems.remove(theOption.getNodeElement());
    elementOptions.remove(optionIndex);
    markModified();
    fireEvent(theOption, PluginEvent.EventCode.OPTION_REMOVED, optionIndex);
    theOption.releaseResources();

    return true;
  }
  
  public void copyInto(DeviceAbilityElement theTarget) {
    boolean setCopiedFrom = (this.copiedFrom != theTarget);
    
    if (setCopiedFrom) theTarget.copiedFrom = this;
    if (theTarget.nodeElement == null) theTarget.nodeElement = new Element(nodeElement.getName());
    if (theTarget.optionItems == null) theTarget.optionItems = theTarget.nodeElement.getChildren("option");
   
    theTarget.setConditionalVisibility(getConditionalVisibility());
    theTarget.setControlType(getControlType());
    theTarget.setDefautlValue(getDefaultValue());
    theTarget.setFormat(getFormat());
    theTarget.setLabel(getLabel());
    theTarget.setMaxValue(getMaxValue());
    theTarget.setMinValue(getMinValue());
    theTarget.setName(getName());
    
    DeviceAbilityElementOption origOption = null, targetOption = null;
    for (int theIndex = 0; theIndex < getOptionCount(); theIndex++) {
      origOption = getOptionAt(theIndex);
      
      if ((targetOption = theTarget.getOptionAt(theIndex)) != null) {
        origOption.copyInto(targetOption);
      } else {
        targetOption = origOption.copyOf(theTarget);
        theTarget.addOption(targetOption);
      }
    }
    
    // Trim the target list, if needed
    while(theTarget.getOptionCount() > getOptionCount()) {
      theTarget.removeOptionAt(theTarget.getOptionCount() - 1);
    }
  }
    
  public DeviceAbilityElement copyOf(DeviceAbility copiesParent) {
    DeviceAbilityElement newCopy = new DeviceAbilityElement();
    newCopy.parent = copiesParent;
    copyInto(newCopy);
    return newCopy;
  }
  public DeviceAbilityElement copyOf() { return copyOf(null); }
  
  public void releaseResources() {
    super.releaseResources();
  
    optionItems = null;
    if (elementOptions != null) {
      for (DeviceAbilityElementOption theOption: elementOptions){
        theOption.releaseResources();
      }
      elementOptions.clear();
      elementOptions = null;
    }
  }
}
