/*
 * DeviceConfigItem.java
 *
 * Created on September 28, 2005, 7:36 PM
 *
 */

package org.cdp1802.xpl.plugins;

import org.jdom.*;

/**
 *
 * @author gerry
 */
public class DeviceConfigItem extends BasePluginItem {
  DeviceConfigItem() { }
  
  DeviceConfigItem(PluginDevice theDevice, Element theElement) {
    this.parent = theDevice;
    this.nodeElement = theElement;
  }
  
  DeviceConfigItem(PluginDevice theDevice, String theName) {
    this.parent = theDevice;
    this.nodeElement = new Element("configItem");
    setName(theName);
  }

  public String getName() {
    String theText = nodeElement.getAttributeValue("name");
    return (theText == null ? "" : theText);
  }
  
  public void setName(String newName) {
    String curValue = getName();
    
    if ((newName == null) || (newName.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("name", "");
        markModified();
        fireEvent(PluginEvent.EventCode.CONFIG_RENAMED);
      }
      return;
    }
    
    newName = newName.toLowerCase();
    
    if (curValue.equals(newName)) return;
    nodeElement.setAttribute("name", newName);
    markModified();
    fireEvent(PluginEvent.EventCode.CONFIG_RENAMED);
  }
  
  public String getDescription() {
    String theText = nodeElement.getAttributeValue("description");
    return (theText == null ? "" : theText);
  }
  
  public void setDescription(String newDescription) {
    String curValue = getDescription();
    
    if ((newDescription == null) || (newDescription.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("description", "");
        markModified();
        fireEvent(PluginEvent.EventCode.CONFIG_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newDescription)) return;
    nodeElement.setAttribute("description", newDescription);
    markModified();
    fireEvent(PluginEvent.EventCode.CONFIG_UPDATED);
  }
  
  public String getFormat() {
    String theText = nodeElement.getAttributeValue("format");
    return (theText == null ? "" : theText);
  }
  
  public void setFormat(String newFormat) {
    String curValue = getFormat();
    
    if ((newFormat == null) || (newFormat.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.setAttribute("format", "");
        markModified();
        fireEvent(PluginEvent.EventCode.CONFIG_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newFormat)) return;
    nodeElement.setAttribute("format", newFormat);
    markModified();
    fireEvent(PluginEvent.EventCode.CONFIG_UPDATED);
  }
  
  public void copyInto(DeviceConfigItem theTarget) {
    boolean setCopiedFrom = (this.copiedFrom != theTarget);
    
    if (setCopiedFrom) theTarget.copiedFrom = this;
    if (theTarget.nodeElement == null) theTarget.nodeElement = new Element(nodeElement.getName());
    
    theTarget.setName(getName());
    theTarget.setDescription(getDescription());
    theTarget.setFormat(getFormat());
  }
    
  public DeviceConfigItem copyOf(PluginDevice copiesParent) {
    DeviceConfigItem newCopy = new DeviceConfigItem();
    newCopy.parent = copiesParent;
    copyInto(newCopy);
    return newCopy;
  }
  public DeviceConfigItem copyOf() { return copyOf(null); }
  
}
