/*
 * PluginDevice.java
 *
 * Created on September 28, 2005, 5:57 PM
 *
 */

package org.cdp1802.xpl.plugins;

import java.util.*;
import org.jdom.*;

/**
 *
 * @author gerry
 */
public class PluginDevice extends BasePluginItem {
  Attribute deviceID = null;
  String deviceIdent = null;
  
  List configItems = null;
  ArrayList<DeviceConfigItem> deviceConfigItems = new ArrayList<DeviceConfigItem>();
  
  List commandItems = null;
  ArrayList<DeviceCommand> deviceCommandItems = new ArrayList<DeviceCommand>();
  
  List triggerItems = null;
  ArrayList<DeviceTrigger> deviceTriggerItems = new ArrayList<DeviceTrigger>();
  ArrayList<DeviceStatus> deviceStatusItems = new ArrayList<DeviceStatus>();

  List menuItems = null;
  
  boolean deviceModified = false;
  boolean deviceNew = false;
  boolean deviceDiscovered = false;
  
  PluginDevice() { }
  
  PluginDevice(VendorPlugin parent, Element relatedElement) {
    this.parent = parent;
    nodeElement = relatedElement;
    if ((deviceID = nodeElement.getAttribute("id")) != null) {
      deviceIdent = deviceID.getValue().toLowerCase();
      setVendorName(parent.getVendorID());
    } else {
      nodeElement.setAttribute("id", deviceIdent = parent.getVendorID() + "-default");
    }
    
    // Populate configuration items
    configItems = nodeElement.getChildren("configItem");
    for (Object anObject: configItems) {
      addConfigItem(new DeviceConfigItem(this, (Element) anObject));
    }

    // Populate command items
    commandItems = nodeElement.getChildren("command");
    for (Object anObject: commandItems) {
      addCommand(new DeviceCommand(this, (Element) anObject));
    }
    
    // Populate trigger items
    triggerItems = nodeElement.getChildren("trigger");
    String messageType = null;
    for (Object anObject: triggerItems) {
      if ((messageType = ((Element) anObject).getAttributeValue("msg_type")) == null) continue;
      
      if (messageType.equalsIgnoreCase("trig")) {
        addTrigger(new DeviceTrigger(this, (Element) anObject));
      } else if (messageType.equalsIgnoreCase("stat")) {
        addStatus(new DeviceStatus(this, (Element) anObject));
      }
    }
  }
  
  PluginDevice(VendorPlugin parent, String deviceName) {
    this.parent = parent;
    this.deviceIdent = parent.getVendorID().toLowerCase() + "-" + deviceName.toLowerCase();
    nodeElement = new Element("device");
    nodeElement.setAttribute("id", deviceIdent);

    configItems = nodeElement.getChildren("configItem");
    commandItems = nodeElement.getChildren("command");
    triggerItems = nodeElement.getChildren("trigger");
    
    deviceNew = true;
  }
  
  protected void markModified() {
    deviceModified = true;
    super.markModified();
  }
  
  void clearModified() {
    deviceModified = false; 
  }
  
  void clearNewDevice() {
    if (!deviceNew) return;
    deviceNew = false;
    fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
  }
  
  public void clearDiscoveredDevice() {
    if (!deviceDiscovered) return;
    deviceDiscovered = false;
    fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
  }
  
  public void markDiscoveredDevice() {
    deviceDiscovered = true;
  }

  public void markNewDevice() {
    deviceNew = true;
  }
  
  public boolean isModified() { return deviceModified; }
  public boolean isNewDevice() { return deviceNew; }
  public boolean isDiscoveredDevice() { return deviceDiscovered; }
  
  public String getID() { return deviceIdent; }
  
  public String getVendorName() {
    int delimPtr = deviceIdent.indexOf("-");
    if (delimPtr == -1) return "";
    return deviceIdent.substring(0, delimPtr);
  }
  
  public void setVendorName(String newName) {
    // Skip if no real change
    if ((newName == null) || (newName.length() == 0) || getVendorName().equalsIgnoreCase(newName)) return;
    
    // Create new ident
    deviceIdent = newName.toLowerCase() + "-" + getDeviceName();
    nodeElement.setAttribute("id", deviceIdent);
    markModified();
    
    if (getVendorName().length() != 0) fireEvent(PluginEvent.EventCode.DEVICE_RENAMED);
  }
  
  public String getDeviceName() {
    int delimPtr = deviceIdent.indexOf("-");
    if (delimPtr == -1) return "";
    return deviceIdent.substring(delimPtr + 1);
  }
  
  public void setDeviceName(String newName) {
    // Skip if no real change
    if ((newName == null) || (newName.length() == 0) || getDeviceName().equalsIgnoreCase(newName)) return;
    
    // Create new ident
    deviceIdent = getVendorName() + "-" + newName.toLowerCase();
    nodeElement.setAttribute("id", deviceIdent);
    markModified();
    if (getDeviceName().length() != 0) fireEvent(PluginEvent.EventCode.DEVICE_RENAMED);
  }
  
  public String getDescription() {
    String theText = nodeElement.getAttributeValue("description");
    return (theText == null ? "" : theText);
  }
  
  public void setDescription(String newDescription) {
    String curValue = getDescription();
    
    if ((newDescription == null) || (newDescription.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeAttribute("description");
        markModified();
        fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newDescription)) return;
    nodeElement.setAttribute("description", newDescription);
    markModified();
    fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
  }
  
  public String getVersion() {
    String theText = nodeElement.getAttributeValue("version");
    return (theText == null ? "" : theText);
  }
  
  public void setVersion(String newVersion) {
    String curValue = getVersion();
    
    if ((newVersion == null) || (newVersion.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeAttribute("version");
        markModified();
        fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newVersion)) return;
    nodeElement.setAttribute("version", newVersion);
    markModified();
    fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
  }
  
  public String getInfoURL() {
    String theText = nodeElement.getAttributeValue("info_url");
    return (theText == null ? "" : theText);
  }
  
  public void setInfoURL(String newURL) {
    String curValue = getInfoURL();
    
    if ((newURL == null) || (newURL.length() == 0)) {
      if (curValue.length() != 0) {
        nodeElement.removeAttribute("info_url");
        markModified();
        fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
      }
      return;
    }
    
    if (curValue.equals(newURL)) return;
    nodeElement.setAttribute("info_url", newURL);
    markModified();
    fireEvent(PluginEvent.EventCode.DEVICE_UPDATED);
  }

  
  /** Configuration support */
  public int getConfigItemCount() { return deviceConfigItems.size(); }

  public DeviceConfigItem getConfigItemAt(int itemIndex) {
    if ((itemIndex < 0) || (itemIndex >= configItems.size())) return null;
    return deviceConfigItems.get(itemIndex);
  }
  
  public int getIndexOfConfigItem(DeviceConfigItem theItem) {
    if (theItem == null) return -1;
    return deviceConfigItems.indexOf(theItem);
  }

  public DeviceConfigItem getConfigItemNamed(String itemName) {
    if ((itemName == null) || (itemName.length() == 0)) return null;
    
    itemName = itemName.toLowerCase();
    for (DeviceConfigItem theItem: deviceConfigItems) {
      if (theItem.getName().equals(itemName)) return theItem;
    }
    
    return null;
  }

  public DeviceConfigItem createConfigItem(String newItemName) {
    if ((newItemName == null) || (newItemName.length() == 0) || (getConfigItemNamed(newItemName) != null)) return null;
    return new DeviceConfigItem(this, newItemName);
  }
  
  
  public boolean addConfigItem(DeviceConfigItem newItem) {
    if ((newItem == null) || (getConfigItemNamed(newItem.getName()) != null)) return false;
    
    int itemCount = getConfigItemCount();
    int insertedAt = -1;
    String newName = newItem.getName();
    for (int itemIndex = 0; itemIndex < itemCount; itemIndex++) {
      if (deviceConfigItems.get(itemIndex).getName().compareTo(newName) < 0) continue;
      deviceConfigItems.add(itemIndex, newItem);
      insertedAt = itemIndex;
      break;
    }
                
    if (insertedAt == -1) {
      insertedAt = deviceConfigItems.size();
      deviceConfigItems.add(newItem);
    }
    
    if (!configItems.contains(newItem.getNodeElement())) configItems.add(newItem.getNodeElement());
    
    newItem.parent = this;
    
    markModified();
    fireEvent(newItem, PluginEvent.EventCode.CONFIG_ADDED);
    return true;
  }
  
  public boolean removeConfigItemAt(int itemIndex) {
    if ((itemIndex < 0) || (itemIndex >= configItems.size())) return false;

    DeviceConfigItem theItem = deviceConfigItems.get(itemIndex);
    
    configItems.remove(theItem.getNodeElement());
    deviceConfigItems.remove(itemIndex);
    markModified();
    fireEvent(theItem, PluginEvent.EventCode.CONFIG_REMOVED, itemIndex);
    theItem.releaseResources();

    return true;
  }
  
  /* Command Support */
  public int getCommandCount() { return deviceCommandItems.size(); }
  
  public DeviceCommand getCommandAt(int commandIndex) {
    if ((commandIndex < 0) || (commandIndex >= deviceCommandItems.size())) return null;
    return deviceCommandItems.get(commandIndex);
  }
  
  public int getIndexOfCommand(DeviceCommand theCommand) {
    if (theCommand == null) return -1;
    return deviceCommandItems.indexOf(theCommand);
  }

  public DeviceCommand getCommandNamed(String commandName) {
    if ((commandName == null) || (commandName.length() == 0)) return null;
    
    for (DeviceCommand theItem: deviceCommandItems) {
      if (theItem.getName().equalsIgnoreCase(commandName)) return theItem;
    }
    
    return null;
  }
  
  public DeviceCommand createCommand(String newCommandName) {
    if ((newCommandName != null) && ((newCommandName.length() == 0) || (getCommandNamed(newCommandName) != null))) return null;
    return new DeviceCommand(this, (newCommandName == null ? "" : newCommandName));
  }
  
  public boolean addCommand(DeviceCommand newCommand) {
    if ((newCommand == null)  || (getCommandNamed(newCommand.getName()) != null)) return false;
    
    int commandCount = getCommandCount();
    int insertedAt = -1;
    String newName = newCommand.getName();
    for (int commandIndex = 0; commandIndex < commandCount; commandIndex++) {
      if (deviceCommandItems.get(commandIndex).getName().compareToIgnoreCase(newName) < 0) continue;
      deviceCommandItems.add(commandIndex, newCommand);
      insertedAt = commandIndex;
      break;
    }
                
    if (insertedAt == -1) {
      insertedAt = deviceCommandItems.size();
      deviceCommandItems.add(newCommand);
    }
    
    if (!commandItems.contains(newCommand.getNodeElement())) commandItems.add(newCommand.getNodeElement());
    
    newCommand.parent = this;
    
    markModified();
    fireEvent(newCommand, PluginEvent.EventCode.COMMAND_ADDED);
    return true;
  }
  
  public boolean removeCommandAt(int commandIndex) {
    if ((commandIndex < 0) || (commandIndex >= deviceCommandItems.size())) return false;

    DeviceCommand theCommand = deviceCommandItems.get(commandIndex);
    
    commandItems.remove(theCommand.getNodeElement());
    deviceCommandItems.remove(commandIndex);
    markModified();
    fireEvent(theCommand, PluginEvent.EventCode.COMMAND_REMOVED, commandIndex);
    theCommand.releaseResources();

    return true;
  }

  /* Trigger Support */
  public int getTriggerCount() { return deviceTriggerItems.size(); }
  
  public DeviceTrigger getTriggerAt(int triggerIndex) {
    if ((triggerIndex < 0) || (triggerIndex >= deviceTriggerItems.size())) return null;
    return deviceTriggerItems.get(triggerIndex);
  }
  
  public int getIndexOfTrigger(DeviceTrigger theTrigger) {
    if (theTrigger == null) return -1;
    return deviceTriggerItems.indexOf(theTrigger);
  }

  public DeviceTrigger getTriggerNamed(String triggerName) {
    if ((triggerName == null) || (triggerName.length() == 0)) return null;
    
    for (DeviceTrigger theItem: deviceTriggerItems) {
      if (theItem.getName().equalsIgnoreCase(triggerName)) return theItem;
    }
    
    return null;
  }
  
  public DeviceTrigger createTrigger(String newTriggerName) {
    if ((newTriggerName != null) && ((newTriggerName.length() == 0) || (getTriggerNamed(newTriggerName) != null))) return null;
    return new DeviceTrigger(this, (newTriggerName == null ? "" : newTriggerName));
  }
  
  public boolean addTrigger(DeviceTrigger newTrigger) {
    if ((newTrigger == null)  || (getTriggerNamed(newTrigger.getName()) != null)) return false;
    
    int triggerCount = getTriggerCount();
    int insertedAt = -1;
    String newName = newTrigger.getName();
    for (int triggerIndex = 0; triggerIndex < triggerCount; triggerIndex++) {
      if (deviceTriggerItems.get(triggerIndex).getName().compareToIgnoreCase(newName) < 0) continue;
      deviceTriggerItems.add(triggerIndex, newTrigger);
      insertedAt = triggerIndex;
      break;
    }
                
    if (insertedAt == -1) {
      insertedAt = deviceTriggerItems.size();
      deviceTriggerItems.add(newTrigger);
    }
    
    if (!triggerItems.contains(newTrigger.getNodeElement())) triggerItems.add(newTrigger.getNodeElement());
    
    newTrigger.parent = this;
    
    markModified();
    fireEvent(newTrigger, PluginEvent.EventCode.TRIGGER_ADDED);
    return true;
  }
  
  public boolean removeTriggerAt(int triggerIndex) {
    if ((triggerIndex < 0) || (triggerIndex >= deviceTriggerItems.size())) return false;

    DeviceTrigger theTrigger = deviceTriggerItems.get(triggerIndex);
    
    triggerItems.remove(theTrigger.getNodeElement());
    deviceTriggerItems.remove(triggerIndex);
    markModified();
    fireEvent(theTrigger, PluginEvent.EventCode.TRIGGER_REMOVED, triggerIndex);
    theTrigger.releaseResources();

    return true;
  }

  /* Status Support */
  public int getStatusCount() { return deviceStatusItems.size(); }
  
  public DeviceStatus getStatusAt(int statusIndex) {
    if ((statusIndex < 0) || (statusIndex >= deviceStatusItems.size())) return null;
    return deviceStatusItems.get(statusIndex);
  }
  
  public int getIndexOfStatus(DeviceStatus theStatus) {
    if (theStatus == null) return -1;
    return deviceStatusItems.indexOf(theStatus);
  }

  public DeviceStatus getStatusNamed(String statusName) {
    if ((statusName == null) || (statusName.length() == 0)) return null;
    
    for (DeviceStatus theItem: deviceStatusItems) {
      if (theItem.getName().equalsIgnoreCase(statusName)) return theItem;
    }
    
    return null;
  }
  
  public DeviceStatus createStatus(String newStatusName) {
    if ((newStatusName != null) && ((newStatusName.length() == 0) || (getStatusNamed(newStatusName) != null))) return null;
    return new DeviceStatus(this, (newStatusName == null ? "" : newStatusName));
  }
  
  public boolean addStatus(DeviceStatus newStatus) {
    if ((newStatus == null)  || (getStatusNamed(newStatus.getName()) != null)) return false;
    
    int statusCount = getStatusCount();
    int insertedAt = -1;
    String newName = newStatus.getName();
    for (int statusIndex = 0; statusIndex < statusCount; statusIndex++) {
      if (deviceStatusItems.get(statusIndex).getName().compareToIgnoreCase(newName) < 0) continue;
      deviceStatusItems.add(statusIndex, newStatus);
      insertedAt = statusIndex;
      break;
    }
                
    if (insertedAt == -1) {
      insertedAt = deviceStatusItems.size();
      deviceStatusItems.add(newStatus);
    }
    
    if (!triggerItems.contains(newStatus.getNodeElement())) triggerItems.add(newStatus.getNodeElement());
    
    newStatus.parent = this;
    
    markModified();
    fireEvent(newStatus, PluginEvent.EventCode.STATUS_ADDED);
    return true;
  }
  
  public boolean removeStatusAt(int statusIndex) {
    if ((statusIndex < 0) || (statusIndex >= deviceStatusItems.size())) return false;

    DeviceStatus theStatus = deviceStatusItems.get(statusIndex);
    
    triggerItems.remove(theStatus.getNodeElement());
    deviceStatusItems.remove(statusIndex);
    markModified();
    fireEvent(theStatus, PluginEvent.EventCode.STATUS_REMOVED, statusIndex);
    theStatus.releaseResources();

    return true;
  }

  public void copyInto(PluginDevice theTarget) {
    boolean setCopiedFrom = (this.copiedFrom != theTarget);
    
    if (setCopiedFrom) theTarget.copiedFrom = this;
    if (theTarget.nodeElement == null) theTarget.nodeElement = new Element(nodeElement.getName());
    if (theTarget.configItems == null) theTarget.configItems = theTarget.nodeElement.getChildren("configItem");
    if (theTarget.commandItems == null) theTarget.commandItems = theTarget.nodeElement.getChildren("command");
    if (theTarget.triggerItems == null) theTarget.triggerItems = theTarget.nodeElement.getChildren("trigger");

    theTarget.deviceModified = false;
    theTarget.nodeElement.setAttribute("id", theTarget.deviceIdent = deviceIdent);
    theTarget.setDescription(getDescription());
    theTarget.setInfoURL(getInfoURL());
    theTarget.setVersion(getVersion());

    // Add or update each config item
    DeviceConfigItem origItem = null, targetItem = null;
    for (int theIndex = 0; theIndex < getConfigItemCount(); theIndex++) {
      origItem = getConfigItemAt(theIndex);
      
      if ((targetItem = theTarget.getConfigItemNamed(origItem.getName())) == null) {
        targetItem = origItem.copyOf(theTarget);
        theTarget.addConfigItem(targetItem);
      } else {
        origItem.copyInto(targetItem);
      }
    }
    
    // Purge obsolete items
    for (int theIndex = theTarget.getConfigItemCount() - 1; theIndex >= 0; theIndex--) {
      targetItem = theTarget.getConfigItemAt(theIndex);
      if (getConfigItemNamed(targetItem.getName()) != null) continue;
      theTarget.removeConfigItemAt(theIndex);
    }

    // Add or update each command
    DeviceCommand origCommand = null, targetCommand = null;
    for (int theIndex = 0; theIndex < getCommandCount(); theIndex++) {
      origCommand = getCommandAt(theIndex);
      
      if ((targetCommand = theTarget.getCommandNamed(origCommand.getName())) == null) {
        targetCommand = (DeviceCommand) origCommand.copyOf(theTarget);
        if (!theTarget.addCommand(targetCommand)) System.err.println("Attempt to add command " + targetCommand + " to dest failed");
      } else {
        origCommand.copyInto(targetCommand);
      }
    }
    
    // Purge obsolete commands
    for (int theIndex = theTarget.getCommandCount() - 1; theIndex >= 0; theIndex--) {
      targetCommand = theTarget.getCommandAt(theIndex);
      if (getCommandNamed(targetCommand.getName()) != null) continue;
      theTarget.removeCommandAt(theIndex);
    }
    
    // Add or update each trigger
    DeviceTrigger origTrigger = null, targetTrigger = null;
    for (int theIndex = 0; theIndex < getTriggerCount(); theIndex++) {
      origTrigger = getTriggerAt(theIndex);
      
      if ((targetTrigger = theTarget.getTriggerNamed(origTrigger.getName())) == null) {
        targetTrigger = (DeviceTrigger) origTrigger.copyOf(theTarget);
        theTarget.addTrigger(targetTrigger);
      } else {
        origTrigger.copyInto(targetTrigger);
      }
    }
    
    // Purge obsolete triggers
    for (int theIndex = theTarget.getTriggerCount() - 1; theIndex >= 0; theIndex--) {
      targetTrigger = theTarget.getTriggerAt(theIndex);
      if (getTriggerNamed(targetTrigger.getName()) != null) continue;
      theTarget.removeTriggerAt(theIndex);
    }

    // Add or update each status
    DeviceStatus origStatus = null, targetStatus = null;
    for (int theIndex = 0; theIndex < getStatusCount(); theIndex++) {
      origStatus = getStatusAt(theIndex);
      
      if ((targetStatus = theTarget.getStatusNamed(origStatus.getName())) == null) {
        targetStatus = (DeviceStatus) origStatus.copyOf(theTarget);
        theTarget.addStatus(targetStatus);
      } else {
        origStatus.copyInto(targetStatus);
      }
    }
    
    // Purge obsolete statuses
    for (int theIndex = theTarget.getStatusCount() - 1; theIndex >= 0; theIndex--) {
      targetStatus = theTarget.getStatusAt(theIndex);
      if (getStatusNamed(targetStatus.getName()) != null) continue;
      theTarget.removeStatusAt(theIndex);
    }
    
    // Copy flags
    theTarget.deviceNew = deviceNew;
    theTarget.deviceDiscovered = deviceDiscovered;
  }
    
  public PluginDevice copyOf(VendorPlugin copiesParent) {
    PluginDevice newCopy = new PluginDevice();
    newCopy.parent = copiesParent;
    copyInto(newCopy);
    return newCopy;
  }
  public PluginDevice copyOf() { return copyOf(null); }
  
  
  public void releaseResources() {
    super.releaseResources();

    deviceID = null;
    deviceIdent = null;
    
    configItems = null;
    commandItems = null;
    triggerItems = null;
    menuItems = null;
    
    if (deviceConfigItems != null) {
      for (DeviceConfigItem theItem: deviceConfigItems) {
        theItem.releaseResources();
      }
      deviceConfigItems.clear();
      deviceConfigItems = null;
    }
    
    if (deviceCommandItems != null) {
      for (DeviceCommand theCommand: deviceCommandItems) {
        theCommand.releaseResources();
      }
      deviceCommandItems.clear();
      deviceCommandItems = null;
    }

    if (deviceTriggerItems != null) {
      for (DeviceTrigger theTrigger: deviceTriggerItems) {
        theTrigger.releaseResources();
      }
      deviceTriggerItems.clear();
      deviceTriggerItems = null;
    }

    if (deviceStatusItems != null) {
      for (DeviceStatus theStatus: deviceStatusItems) {
        theStatus.releaseResources();
      }
      deviceStatusItems.clear();
      deviceStatusItems = null;
    }
  
  }
  
  public String toString() { return deviceIdent; }
}
