/*
 * PluginEvent.java
 *
 * Created on September 29, 2005, 12:21 PM
 *
 */

package org.cdp1802.xpl.plugins;


/**
 *
 * @author gerry
 */
public class PluginEvent {
  public enum EventCode { DEVICE_ADDED, DEVICE_UPDATED, DEVICE_RENAMED, DEVICE_REMOVED,
                          CONFIG_ADDED, CONFIG_UPDATED, CONFIG_RENAMED, CONFIG_REMOVED,
                          COMMAND_ADDED, COMMAND_UPDATED, COMMAND_RENAMED, COMMAND_REMOVED,
                          TRIGGER_ADDED, TRIGGER_UPDATED, TRIGGER_RENAMED, TRIGGER_REMOVED,
                          STATUS_ADDED, STATUS_UPDATED, STATUS_RENAMED, STATUS_REMOVED,
                          ELEMENT_ADDED, ELEMENT_UPDATED, ELEMENT_RENAMED, ELEMENT_REMOVED,
                          OPTION_ADDED, OPTION_UPDATED, OPTION_RENAMED, OPTION_REMOVED,
                          PLUGIN_UPDATED
  };
  
  EventCode theEventCode = null;
  int theEventIndex = -1;
  
  VendorPlugin thePlugin = null;
  PluginDevice theDevice = null;
  DeviceConfigItem theConfig = null;
  DeviceAbility theAbility = null;
  DeviceAbilityElement theElement = null;
  DeviceAbilityElementOption theOption = null;
  
  public PluginEvent(BasePluginItem theItem, EventCode theEventCode, int theIndex) {
    if (theItem instanceof VendorPlugin) {
      this.thePlugin = (VendorPlugin) theItem;
      this.theEventCode = theEventCode;
      this.theEventIndex = theIndex;
      return;
    }
    
    if (theItem instanceof PluginDevice) {  
      this.theDevice = (PluginDevice) theItem;
      if (theDevice != null) this.thePlugin = (VendorPlugin) theDevice.getParent();
      this.theEventCode = theEventCode;
      this.theEventIndex = theIndex;
      
      if ((theIndex == -1) && (thePlugin != null)) theEventIndex = thePlugin.getIndexOfDevice(theDevice);
      return;
    }

    if (theItem instanceof DeviceConfigItem) {
      this.theConfig = (DeviceConfigItem) theItem;
      if (theConfig != null) this.theDevice = (PluginDevice) theConfig.getParent();
      if (theDevice != null) this.thePlugin = (VendorPlugin) theDevice.getParent();
      this.theEventCode = theEventCode;
      this.theEventIndex = theIndex;

      if ((theIndex == -1) && (theDevice != null)) theEventIndex = theDevice.getIndexOfConfigItem(theConfig);
      return;
    }

    if (theItem instanceof DeviceAbility) {
      this.theAbility = (DeviceAbility) theItem;
      if (theAbility != null) this.theDevice = (PluginDevice) theAbility.getParent();
      if (theDevice != null) this.thePlugin = (VendorPlugin) theDevice.getParent();
      this.theEventCode = theEventCode;
      this.theEventIndex = theIndex;

      if ((theIndex == -1) && (theDevice != null)) {
        if (theAbility instanceof DeviceCommand) theEventIndex = theDevice.getIndexOfCommand((DeviceCommand) theAbility);
        if (theAbility instanceof DeviceTrigger) theEventIndex = theDevice.getIndexOfTrigger((DeviceTrigger) theAbility);
        if (theAbility instanceof DeviceStatus) theEventIndex = theDevice.getIndexOfStatus((DeviceStatus) theAbility);
      }
      return;
    }
  
    if (theItem instanceof DeviceAbilityElement) {
      this.theElement = (DeviceAbilityElement) theItem;
      if (theElement != null) this.theAbility = (DeviceAbility) theElement.getParent();
      if (theAbility != null) this.theDevice = (PluginDevice) theAbility.getParent();
      if (theDevice != null) this.thePlugin = (VendorPlugin) theDevice.getParent();
      this.theEventCode = theEventCode;
      this.theEventIndex = theIndex;

      if ((theIndex == -1) && (theAbility != null)) theEventIndex = theAbility.getIndexOfElement(theElement);
      return;
    }
  
    if (theItem instanceof DeviceAbilityElementOption) {
      this.theOption = (DeviceAbilityElementOption) theItem;
      if (theOption != null) this.theElement = (DeviceAbilityElement) theOption.getParent();
      if (theElement != null) this.theAbility = (DeviceAbility) theElement.getParent();
      if (theAbility != null) this.theDevice = (PluginDevice) theAbility.getParent();
      if (theDevice != null) this.thePlugin = (VendorPlugin) theDevice.getParent();
      this.theEventCode = theEventCode;
      this.theEventIndex = theIndex;

      if ((theIndex == -1) && (theElement != null)) theEventIndex = theElement.getIndexOfOption(theOption);
      return;
    }
    
    // Anything else is something wierd
    throw new RuntimeException("Attempt to create PluginEvent for unknown plugin item -- " + theItem.getClass().getName());
  }
  
  public EventCode getEventCode() { return theEventCode; }
  public int getEventIndex() { return theEventIndex; }
  
  public VendorPlugin getPlugin() { return thePlugin; }
  public PluginDevice getDevice() { return theDevice; }
  public DeviceConfigItem getConfigItem() { return theConfig; }
  public DeviceAbility getAbility() { return theAbility; }
  public DeviceCommand getCommand() {
    if ((theAbility == null) || (!(theAbility instanceof DeviceCommand))) return null;
    return (DeviceCommand) theAbility;
  }
  public DeviceTrigger getTrigger() {
    if ((theAbility == null) || (!(theAbility instanceof DeviceTrigger))) return null;
    return (DeviceTrigger) theAbility;
  }
  public DeviceStatus getStatus() {
    if ((theAbility == null) || (!(theAbility instanceof DeviceStatus))) return null;
    return (DeviceStatus) theAbility;
  }
  
  
  public DeviceAbilityElement getElement() { return theElement; }
  public DeviceAbilityElementOption getElementOption() { return theOption; }
  
  void releaseResources() {
    theEventCode = null;
    theEventIndex = -1;
    thePlugin = null;
    theDevice = null;
    theConfig = null;
    theAbility = null;
    theElement = null;
    theOption = null;
  }
  
  
}
