/*
 * BaseTimer.java
 *
 * Created on June 25, 2005, 11:13 AM
 */

package org.cdp1802.xpl.script;

import bsh.EvalError;
import java.util.*;

/**
 *
 * @author gerry
 */
public abstract class BaseTimer implements Runnable {
  static private int yearBeforeThisYear = -1;
  static private int daysBeforeThisYear = -1;
  
  String eventHandler = null;   // Handler to invoke when timer fires
  Thread timerThread = null;    // Thread for timer
  XPLScript timerScript = null; // Script associated with this timer
  
  boolean timerRunning = false;        // Shows state of thread
  boolean timerThreadRunning = false;  // Shows state of thread
  
  
  Calendar dateTimeCalendar = Calendar.getInstance();
  
  
  void startIntTimer(String timerDescription) {
    synchronized(timerScript) {
      if (timerRunning) return;
      
      timerRunning = true;
      timerThreadRunning = false;
      timerThread = new Thread(this, timerDescription);
      timerThread.start();
  
      // Wait til it's running
      while (!timerThreadRunning) {
        try {
          timerScript.wait();
        } catch (Exception anyError) { }
      }
    }
  }
  
  void stopIntTimer() {
    synchronized(timerScript) {
      if (!timerRunning) return;
      
      // Flag us no longer running and wait for
      // the thread to confirm it's stopped
      timerRunning = false;
      if (Thread.currentThread() != timerThread) {
        // Wait for the timer to say it's done
        while(timerThreadRunning) {
          try {
            timerScript.notifyAll();
            timerScript.wait();
          } catch (Exception anyError) { }
        }
      }
   }

    // Clear thread out
    timerThread = null;
  }
  
  public int getHourOfDay() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.HOUR_OF_DAY);
  }
  
  public int getHourOfYear() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return ((dateTimeCalendar.get(Calendar.DAY_OF_YEAR) - 1) * 24) + dateTimeCalendar.get(Calendar.HOUR_OF_DAY);
  }
  
  public int getMinuteOfHour() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.MINUTE);
  }
  
  public int getMinuteOfYear() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return getHourOfYear() + dateTimeCalendar.get(Calendar.MINUTE);
  }

  public int getSecondOfMinute() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.SECOND);
  }

  public int getMinuteOfDay() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return (dateTimeCalendar.get(Calendar.HOUR_OF_DAY) * 60) + dateTimeCalendar.get(Calendar.MINUTE);
  }
  
  public int getDayOfMonth() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.DAY_OF_MONTH);
  }
  
  public int getDayOfWeek() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.DAY_OF_WEEK);
  }
  
  public int getDayOfYear() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.DAY_OF_YEAR);
  }

  public int getDayOfEpoch() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    int curYear = dateTimeCalendar.get(Calendar.YEAR);
    
    // Compute # of days before this year
    if (yearBeforeThisYear != curYear) {
      dateTimeCalendar.set(1970, 0, 1, 0, 0);
      daysBeforeThisYear = 0;
      for (int aYear = 1970; aYear < curYear; aYear++) {
        dateTimeCalendar.set(Calendar.YEAR, aYear);
        daysBeforeThisYear += dateTimeCalendar.getActualMaximum(Calendar.DAY_OF_YEAR);
      }
      
      yearBeforeThisYear = curYear;
      dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    }
    
    return daysBeforeThisYear + dateTimeCalendar.get(Calendar.DAY_OF_YEAR);
  }
  
  public int getWeekOfYear() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.WEEK_OF_YEAR);
  }

  public int getMonthOfYear() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.MONTH) + 1;
  }
  
  public int getYear() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    return dateTimeCalendar.get(Calendar.YEAR);
  }
}
