/*
 * HVHelper.java
 *
 * Created on June 3, 2005, 11:20 PM
 */

package org.cdp1802.xpl.script;

import java.util.*;

import org.cdp1802.xpl.*;
import org.cdp1802.hvision.*;
import org.cdp1802.hvision.dinput.*;
import org.cdp1802.hvision.doutput.*;
import org.cdp1802.hvision.flag.*;
import org.cdp1802.hvision.ir.*;
import org.cdp1802.hvision.macro.*;
import org.cdp1802.hvision.phone.*;
import org.cdp1802.hvision.variable.*;
import org.cdp1802.hvision.x10.*;

/**
 *
 * @author gerry
 */
public class HVHelper implements HVDInputListenerI, HVDOutputListenerI, HVFlagListenerI, HVIRListenerI, HVPhoneListenerI, HVVariableListenerI, HVX10ListenerI {
  boolean helperEnabled = true;
  boolean reportOnce = true;
  HashMap<String, String> handlerTranslations = null;
  XPLScript theScript = null;
  
  HVHelper(XPLScript theScript) {
    this.theScript = theScript;

    HVManager theManager = HVManager.getManager();
    theManager.setXPLDevice(XPLScriptEngine.getEngine().getModuleDevice());
    theManager.startManager();
    
    theManager.getDInputManager().addListener(this);
    theManager.getDOutputManager().addListener(this);
    theManager.getFlagManager().addListener(this);
    theManager.getIRManager().addListener(this);
    theManager.getPhoneManager().addListener(this);
    theManager.getVariableManager().addListener(this);
    theManager.getX10Manager().addListener(this);
  }
  
  void releaseResources() {
    HVManager.getManager().removeAllListeners(this);
    
    if (handlerTranslations != null) handlerTranslations.clear();
    handlerTranslations = null;
    
    theScript = null;
  }

  void scriptReloaded() { }
  
  
  public void setEnabled(boolean isEnabled) {
    helperEnabled = isEnabled;
  }
  public boolean isEnabled() { return helperEnabled; }

  public void setReportOnce(boolean onlyOnce) {
    reportOnce = onlyOnce;
  }
  public boolean isReportOnce() { return reportOnce; }
  
  private String translate(String theMethodName) {
    if (handlerTranslations == null) return theMethodName;
    String translation = handlerTranslations.get(theMethodName);
    return (translation == null ? theMethodName : translation);
  }
  
  public void overrideHandler(String methodName, String translation) {
    if ((translation == null) && (handlerTranslations == null)) return;
    if (handlerTranslations == null) handlerTranslations = new HashMap<String, String>();
    if (translation == null)
      handlerTranslations.remove(methodName);
    else
      handlerTranslations.put(methodName, translation);
  }
  public void resetHandler(String methodName) {
    if (handlerTranslations == null) return;
    handlerTranslations.remove(methodName);
  }
  
  private void sendReport(String parameterNames[], Object theParameters[], String[] methodNames) {
    if ((parameterNames != null) != (theParameters != null)) return;
    
    StringBuffer theParameterList = new StringBuffer();
    if (parameterNames != null) {
      if (parameterNames.length != theParameters.length) return;
      
      for (int parmIndex = 0; parmIndex < parameterNames.length; parmIndex++) {
        theScript.setVar(parameterNames[parmIndex], theParameters[parmIndex]);
        
        if (theParameterList.length() != 0) theParameterList.append(", ");
        theParameterList.append(parameterNames[parmIndex]);
      }
    }
    
    for (String aMethod: methodNames) {
      if (theScript.hasMethod(aMethod)) {
        theScript.invokeMethod(aMethod, theParameterList.toString());
        if (reportOnce) break;
      }
    }
    
    if (parameterNames != null) {
      for (int parmIndex = 0; parmIndex < parameterNames.length; parmIndex++) {
        theScript.clearVar(parameterNames[parmIndex]);
      }
    }
  }

  private void sendReport(String parameterName, Object theParameter, String[] methodNames) {
    sendReport(new String[] { parameterName }, new Object[] { theParameter }, methodNames);
  }
  
  public void handleDInputMessage(HVDInputEventI theEvent) {
    if (!helperEnabled) return;
    
    if (theEvent.getReason() == HVDInputEventI.EventReason.STATE_CHANGED) {
      String scriptName = translate("DInput" + theEvent.getDInput().getDInputNum() + "Changed");
      String globalScriptName = translate("DInputChanged");
      sendReport("theInput", theEvent.getDInput(), new String[] { scriptName, globalScriptName });
    }
  }
  
  public void handleDOutputMessage(HVDOutputEventI theEvent) {
    if (!helperEnabled) return;

    if (theEvent.getReason() == HVDOutputEventI.EventReason.STATE_CHANGED) {
      String scriptName = translate("DOutput" + theEvent.getDOutput().getDOutputNum() + "Changed");
      String globalScriptName = translate("DOutputChanged");
      sendReport("theOutput", theEvent.getDOutput(), new String[] { scriptName, globalScriptName });
    }
  }
  
  public void handleFlagMessage(HVFlagEventI theEvent) {
    if (!helperEnabled) return;

    if (theEvent.getReason() == HVFlagEventI.EventReason.STATE_CHANGED) {
      String scriptName = translate("Flag" + theEvent.getFlag().getFlagNum() + "Changed");
      String globalScriptName = translate("FlagChanged");
      sendReport("theFlag", theEvent.getFlag(), new String[] { scriptName, globalScriptName});
    }
  }
  
  public void handleIRMessage(HVIREventI theEvent) {
    if (!helperEnabled) return;

    if (theEvent.getReason() == HVIREventI.EventReason.ATTR_CHANGED) return;

    if (theEvent.getReason() == HVIREventI.EventReason.KNOWN_IR_RECEIVED) {
      String scriptName = translate("KnownIR" + theEvent.getIR().getIRNum() + "Receibved");
      String globalScriptName = translate("KnownIRReceived");
      sendReport("theIR", theEvent.getIR(), new String[] { scriptName, globalScriptName });
      return;
    }
    
    if (theEvent.getReason() == HVIREventI.EventReason.STANDARD_IR_RECEIVED) {
      String scriptName = translate("StandardIR" + theEvent.getStandardDeviceCode() + "_" + theEvent.getStandardKeyCode() + "Recieved");
      String globalScriptName = translate("StandardIRReceived");
      sendReport(new String[] { "theDeviceCode", "theKeyCode" }, 
                 new Object[] { new Integer(theEvent.getStandardDeviceCode()), new Integer(theEvent.getStandardKeyCode()) },
                 new String[] { scriptName, globalScriptName });
      return;
    }
  }
  
  public void handlePhoneMessage(HVPhoneEventI theEvent) {
    if (!helperEnabled) return;
  }
  
  public void handleVariableMessage(HVVariableEventI theEvent) {
    if (!helperEnabled) return;

    if (theEvent.getReason() == HVVariableEventI.EventReason.VALUE_CHANGED) {
      String scriptName = translate("Variable" + theEvent.getVariable().getVariableNum() + "Changed");
      String globalScriptName = translate("VariableChanged");
      sendReport("theVariable", theEvent.getVariable(), new String[] { scriptName, globalScriptName});
    }
  }
  
  public void handleX10Message(HVX10EventI theEvent) {
    if (!helperEnabled) return;

    if (theEvent.getReason() != HVX10EventI.EventReason.ATTR_CHANGED) {
      String scriptName = translate("X10Device" +  theEvent.getX10Device().getDeviceName() + "Changed");
      String houseScriptName = translate("X10House" + theEvent.getX10Device().getHouse() + "Changed");
      String globalScriptName = translate("X10DeviceChanged");

      sendReport("theDevice", theEvent.getX10Device(), new String[] { scriptName, houseScriptName, globalScriptName });
    }
  }
}
