/*
 * MessageFilter.java
 *
 * Created on June 2, 2005, 1:01 PM
 */

package org.cdp1802.xpl.script;

import java.io.*;
import java.util.*;

import bsh.Interpreter;
import bsh.EvalError;

import org.cdp1802.xpl.*;

/**
 *
 * @author gerry
 */
public class MessageFilter {
  public static final String DEFAULT_HANDLER = "handleXPLMessage";

  XPLScript filterScript = null;
  boolean filterEnabled = false;
  xPL_FilterI theFilter = null;
  String filterHandler = null;
  boolean filterMatchesAnyNamedValue = false;
  MutableNamedValuesI filterNamedValues = null;
  
  MessageFilter(XPLScript theScript) {
    filterScript = theScript;
  }

  void releaseResources() {
    if (theFilter != null) theFilter.resetFilters();
    theFilter = null;

    if (filterNamedValues != null) filterNamedValues.releaseResources();
    filterNamedValues = null;
    
    filterHandler = null;
    filterScript = null;
  }

  public void setEnabled(boolean isEnabled) {
    filterEnabled = isEnabled;
  }
  public boolean isEnabled() { return filterEnabled; }
  
  public void setFilter(String newFilterSpec) {
    if ((newFilterSpec == null) || (newFilterSpec.length() == 0)) newFilterSpec = "*";
    theFilter = new xPL_Filter(newFilterSpec);
  }
  public String getFilter() { return (theFilter == null) ? "*" : theFilter.toString(); }
  
  public void setHandler(String newHandler) {
    if ((newHandler != null) && (newHandler.length() == 0)) newHandler = null;
    filterHandler = newHandler;
  }
  public String getHandler() { return (filterHandler != null ? filterHandler : DEFAULT_HANDLER); }
  
  public void setMatchesAnyNamedValue(boolean matchAny) {
    filterMatchesAnyNamedValue = matchAny;
  }
  public boolean isMatchesAnyNamedValue() { return filterMatchesAnyNamedValue; }
  
  public void clearNamedValueMatches() {
    if (filterNamedValues == null) return;
    filterNamedValues.clear();
  }
  
  public void addNamedValueMatch(String theName, String theValue) {
    // Initialize Named values, if needed
    if (filterNamedValues == null) filterNamedValues = xPL_Util.createMutableNamedValues();
    
    filterNamedValues.addNamedValue(theName, theValue);
  }
  
  public void addNamedValueMatch(String theName, int theValue) {
    // Initialize Named values, if needed
    if (filterNamedValues == null) filterNamedValues = xPL_Util.createMutableNamedValues();
    
    filterNamedValues.addNamedValue(theName, String.valueOf(theValue));
  }

  public void addNamedValueMatch(String theName, boolean theValue) {
    // Initialize Named values, if needed
    if (filterNamedValues == null) filterNamedValues = xPL_Util.createMutableNamedValues();
    
    filterNamedValues.addNamedValue(theName, (theValue ? "1" : "0"));
  }
  
  public void setNamedValueMatch(String theName, String theValue) {
    // Initialize Named values, if needed
    if (filterNamedValues == null) filterNamedValues = xPL_Util.createMutableNamedValues();
    
    filterNamedValues.setNamedValue(theName, theValue);
  }
  
  public void setNamedValueMatch(String theName, int theValue) {
    // Initialize Named values, if needed
    if (filterNamedValues == null) filterNamedValues = xPL_Util.createMutableNamedValues();
    
    filterNamedValues.setNamedValue(theName, String.valueOf(theValue));
  }

  public void setNamedValueMatch(String theName, boolean theValue) {
    // Initialize Named values, if needed
    if (filterNamedValues == null) filterNamedValues = xPL_Util.createMutableNamedValues();
    
    filterNamedValues.setNamedValue(theName, (theValue ? "1" : "0"));
  }
  
  public void removeNamedValueMatches(String theName) {
    if (filterNamedValues == null) return;
    filterNamedValues.removeNamedValues(theName);
  }
  
  boolean matchesNamedValues(xPL_MessageI theMessage) {
    boolean matchFound;
    
    // See if there is anything to match
    if ((filterNamedValues == null) || (filterNamedValues.getNamedValuesCount() == 0)) return true;
    
    // Iterate over the names
    for (NamedValueI namedValue: filterNamedValues.getAllNamedValues()) {
      // Handle a wildcard vs value search
      if (namedValue.getValue() == null)
        matchFound = theMessage.hasNamedValue(namedValue.getName());
      else
        matchFound = theMessage.hasNamedValue(namedValue.getName(), namedValue.getValue());
      
      // If this is a match and we are matching ANY, then we have a match
      if (matchFound && filterMatchesAnyNamedValue) return true;
      
      // If this is not a match and we are matching ALL, then we cannot ever have a match
      if (!matchFound && !filterMatchesAnyNamedValue) return false;
    }
    
    // If we got here, either nothing match (ANY) or everything match (ALL)
    return !filterMatchesAnyNamedValue;
  }
  
  boolean doesMessageMatchFilter(xPL_MessageI theMessage) {
    if (!filterEnabled) return false;

    // Does filter match?
    if ((theFilter != null) && !theFilter.doesMessageMatchFilter(theMessage)) return false;
    
    // Do named/values match?
    return matchesNamedValues(theMessage);
  }
}
