/*
 * ScheduledEvent.java
 *
 * Created on June 1, 2005, 2:37 PM
 */

package org.cdp1802.xpl.script;

import bsh.EvalError;

import java.util.*;

/**
 *
 * @author gerry
 */
public class ScheduledEvent extends BaseTimer implements Runnable {
  public static final String DEFAULT_HANDLER = "handleScheduledEvent";

  int eventTimeOfDay = 0;            // Time of day (minutes into the day)
  boolean daysOfWeek[] = new boolean[8]; // Days of week flags
  
  ScheduledEvent(XPLScript theScript) { 
    timerScript = theScript;
  }
  
  void releaseResources() {
    stopEvent();
    eventHandler = null;
    timerThread = null;
    timerScript = null;
    daysOfWeek = null;
    dateTimeCalendar = null;
  }

  void startEvent() { startIntTimer("Scheduled Event"); }
  void stopEvent() { stopIntTimer(); }
  
  public boolean isEnabled() { return timerRunning; }
  public void setEnabled(boolean isEnabled) {
    if (timerScript == null)  return;
    
    if (isEnabled) 
      startEvent();
    else
      stopEvent();
  }

  private int parseTimeOfDay(String theTime) {
    int delimPtr = theTime.indexOf(":");
    if (delimPtr == -1) return -1;
    
    int hours, minutes;
    
    try {
      hours = Integer.parseInt(theTime.substring(0, delimPtr));
      minutes = Integer.parseInt(theTime.substring(delimPtr + 1));
    } catch (Exception anyError) { return -1; }
    
    if ((hours < 0) || (hours > 23) || (minutes < 0) || (minutes > 59)) return -1;
    
    return hours * 60 + minutes;
  }

  private String formatTimeOfDay(int theTime) {
    if ((theTime < 0) || (theTime > 1440)) return null;
    
    int hours = theTime / 60;
    int minutes = theTime % 60;
    
    StringBuffer textTime = new StringBuffer(8);
    
    if (hours < 10) textTime.append('0');
    textTime.append(hours);
    textTime.append(':');
    if (minutes < 10) textTime.append('0');
    textTime.append(minutes);
    
    return textTime.toString();
  }
  
  public void setTimeOfDay(String theTimeOfDay) {
    if (timerRunning || (theTimeOfDay == null) || (theTimeOfDay.length() == 0)) return;

    int newTimeOfDay = parseTimeOfDay(theTimeOfDay);
    if (newTimeOfDay == -1) return;
    eventTimeOfDay = newTimeOfDay;
  }
  public String getTimeOfDay() { return formatTimeOfDay(eventTimeOfDay); }
  

  private void append(String item, StringBuffer theBuffer) {
    if (theBuffer.length() != 0) theBuffer.append(',');
    theBuffer.append(item);
  }
  
  public void setDaysOfWeek(String newDaysOfWeek) {
    if (timerRunning) return;
    
    // Handle clearing all days of week
    if ((newDaysOfWeek == null) || (newDaysOfWeek.length() == 0)) {
      for (int dayIndex = 0; dayIndex < daysOfWeek.length; dayIndex++) {
        daysOfWeek[dayIndex] = false;
      }
      return;
    }
    
    // Parse this
    String tokenList[] = newDaysOfWeek.split(",");
    boolean newFlags[] = new boolean[8];
    for (String theToken: tokenList) {
      if (theToken.equalsIgnoreCase("Sun"))
        newFlags[Calendar.SUNDAY] = true;
      else if (theToken.equalsIgnoreCase("Mon"))
        newFlags[Calendar.MONDAY] = true;
      else if (theToken.equalsIgnoreCase("Tue"))
        newFlags[Calendar.TUESDAY] = true;
      else if (theToken.equalsIgnoreCase("Wed"))
        newFlags[Calendar.WEDNESDAY] = true;
      else if (theToken.equalsIgnoreCase("Thu"))
        newFlags[Calendar.THURSDAY] = true;
      else if (theToken.equalsIgnoreCase("Fri"))
        newFlags[Calendar.FRIDAY] = true;
      else if (theToken.equalsIgnoreCase("Sat"))
        newFlags[Calendar.SATURDAY] = true;
      else if (theToken.equalsIgnoreCase("All")) {
        for (int dayIndex = 0; dayIndex < newFlags.length; dayIndex++) {
          newFlags[dayIndex] = true;
        }
      } else {
        // Unknown token -- die
        return;
      }
    }

    // Install it
    for (int dayIndex = 0; dayIndex < daysOfWeek.length; dayIndex++) {
      daysOfWeek[dayIndex] = newFlags[dayIndex];
    }
  }
  
  public String getDaysOfWeek() {
    if (daysOfWeek[1] && daysOfWeek[2] && daysOfWeek[3] && daysOfWeek[4] && daysOfWeek[5] && daysOfWeek[6] && daysOfWeek[7]) return "All";
    
    StringBuffer dayText = new StringBuffer();
    if (daysOfWeek[Calendar.SUNDAY]) append("Sun", dayText);
    if (daysOfWeek[Calendar.MONDAY]) append("Mon", dayText);
    if (daysOfWeek[Calendar.TUESDAY]) append("Tue", dayText);
    if (daysOfWeek[Calendar.WEDNESDAY]) append("Wed", dayText);
    if (daysOfWeek[Calendar.THURSDAY]) append("Thu", dayText);
    if (daysOfWeek[Calendar.FRIDAY]) append("Fri", dayText);
    if (daysOfWeek[Calendar.SATURDAY]) append("Sat", dayText);
    
    return dayText.toString();
  }
  
  
  public String getHandler() { return (eventHandler != null ? eventHandler : DEFAULT_HANDLER); }
  public void setHandler(String newHandler) {
    if (timerRunning) return;
    if ((newHandler != null) && (newHandler.length() == 0)) newHandler = null;
    eventHandler = newHandler;
  }
  
  long getAbsoluteTimeOfNextEvent() {
    dateTimeCalendar.setTimeInMillis(System.currentTimeMillis());
    dateTimeCalendar.set(Calendar.SECOND, 0);
    
    int eventHours = eventTimeOfDay / 60;
    int eventMinutes = eventTimeOfDay % 60;
    
    int hours = dateTimeCalendar.get(Calendar.HOUR_OF_DAY);
    int minutes = dateTimeCalendar.get(Calendar.MINUTE);
    
    boolean beforeEvent = false;
    boolean afterEvent = false;
    if (hours < eventHours) {
      beforeEvent = true;
    } else if (eventHours == hours) {
      beforeEvent = (minutes < eventMinutes);
      afterEvent = (minutes > eventMinutes);
    } else if (hours > eventHours) {
      afterEvent = true;
    }
    
    // Install the date/time of the event & adjust if needed
    dateTimeCalendar.set(Calendar.HOUR_OF_DAY, eventHours);
    dateTimeCalendar.set(Calendar.MINUTE, eventMinutes);
    if (!beforeEvent) dateTimeCalendar.add(Calendar.DAY_OF_MONTH, 1);
    
    return dateTimeCalendar.getTimeInMillis();
  }
  
  public void run() {
    Calendar dayCalendar = Calendar.getInstance();
    long nextEventAt = 0, sleepTime = 0, rightNow = 0;
    String theHandler = null;
    
    synchronized(timerScript) {
      // Flag us running
      timerThreadRunning = true;
      timerScript.notifyAll();
      
      // Now we actually runn
      mainLoop:
      while (timerRunning) {
        // Get next time the event will occur.
        nextEventAt = getAbsoluteTimeOfNextEvent();
        
        while(timerRunning) {
          try {
            sleepTime = nextEventAt - System.currentTimeMillis();
            if (sleepTime <= 0) sleepTime = 1000;
            timerScript.wait(sleepTime);
          } catch (Exception ignoredException) { }
        
          // If we are no longer running, exit
          if (!timerRunning) break;

          // See if this is the time
          if (System.currentTimeMillis() < nextEventAt) continue;
          
          // See if the day of the week matches
          dayCalendar.setTimeInMillis(System.currentTimeMillis());
          if (!daysOfWeek[dayCalendar.get(Calendar.DAY_OF_WEEK)]) break;
          
          // Install our timer
          timerScript.setVar("curEvent", this);
        
          // Invoke the handler
          theHandler = (eventHandler == null ?  DEFAULT_HANDLER : eventHandler);
          timerScript.invokeMethod(theHandler, "curEvent");

          // Clear things out
          timerScript.clearVar("curEvent");

          // And done
          break;
        }
        
        // If we are no longer running, exit
        if (!timerRunning) break;
      }
      
      // Timer is no longer running
      dayCalendar = null;
      timerThreadRunning = false;
      timerScript.notifyAll();
    }
  }
}
