/*
 * XPLScriptEngine.java
 *
 * Created on May 31, 2005, 11:15 AM
 */

package org.cdp1802.xpl.script;

import java.io.*;
import java.util.*;

import org.cdp1802.xpl.*;
import org.cdp1802.xpl.device.*;
import org.cdp1802.xpl.server.*;

/**
 *
 *
 * @author gerry
 */
public class XPLScriptEngine extends xPL_Module implements xPL_DeviceChangeListenerI, xPL_DeviceMessageListenerI {
  private static XPLScriptEngine scriptEngine = null;
  static XPLScriptEngine getEngine() { return scriptEngine; }

  public XPLScriptEngine() {
    scriptEngine = this;
  }

  void readConfig() {
    HashMap<File, File> existingDirs = new HashMap<File, File>();
    File theDirectory = null;
    XPLScriptManager theManager = XPLScriptManager.getManager();
    
    // List of configured paths
    xPL_DeviceConfigItemI scriptDirs = moduleDevice.getConfigurable("ScriptPath");
    if ((scriptDirs != null) && (scriptDirs.getValueCount() != 0)) {
      for (String dirName: scriptDirs.getValues()) {
        if ((dirName == null) || (dirName.length() == 0)) continue;
        theDirectory = new File(dirName);

        // See if this directory exists
        if (theManager.getDirectory(theDirectory) == null) {
          theManager.addDirectory(theDirectory, true);
        }

        // Flag we know this exists
        existingDirs.put(theDirectory, theDirectory);
      }
    }
    
    // Now we go through the existings directories and if they are no
    // longer listed, delete them
    for (XPLScriptDirectory scriptDirectory: theManager.getAllDirectories()) {
      // If this exists in the existing list, leave it
      if (existingDirs.containsKey(scriptDirectory.getDirectory())) continue;
      
      // Otherwise, this is no longer one of the directories
      theManager.removeDirectory(scriptDirectory);
    }
    
    // Update refresh interval
    xPL_DeviceConfigItemI refreshItem = moduleDevice.getConfigurable("RefreshInterval");
    if (refreshItem != null) {
      int newInterval = refreshItem.getIntValue();
      if (newInterval > 0) {
        if (theManager.getRefreshInterval() != newInterval) {
          theManager.setRefreshInterval(newInterval);
        }
      }
    }

    // Insure all scripts are up to date
    theManager.refreshScripts();
  }

  /***** xPL_Module Support *******/
  
  public void loadModule(xPL_ModuleConfigI moduleConfig, xPL_DeviceI moduleDevice) {
    super.loadModule(moduleConfig, moduleDevice);
  }
  
  public void startModule() {
    // Skip out if already started
    if (isModuleStarted()) return;

    // Install any script directories
    readConfig();
    
    // Add listeners
    moduleDevice.addDeviceChangeListener(this);
    moduleDevice.addMessageListener(this);

    // Start script updator
    XPLScriptManager.getManager().startScriptEngine();
    super.startModule();
  }

  public void stopModule() {
    if (!isModuleStarted()) return;

    // Remove Listeners
    moduleDevice.removeMessageListener(this);
    moduleDevice.removeDeviceChangeListener(this);
    
    // Stop the scripting engine
    XPLScriptManager.getManager().stopScriptEngine();
    
    super.stopModule();
  }
  
  public void unloadModule() {
    // Release script resources
    XPLScriptManager.getManager().releaseResources();
    super.unloadModule();
  }
  
  public void handleXPLDeviceChange(xPL_DeviceChangeEventI theEvent) {
    if (theEvent.getChangeReason() == xPL_DeviceChangeEventI.ChangeReason.CONFIGURATION_CHANGED) {
      XPLScriptManager.getManager().stopScriptEngine();
      readConfig();
      XPLScriptManager.getManager().startScriptEngine();
    }
  }
  
  public void handleXPLDeviceMessage(xPL_DeviceI theDevice, xPL_MessageI theMessage) {
    // Filter out things we do not understand
    if (!theMessage.getSchemaClass().equalsIgnoreCase("script")) return;
    
    // Get the command, if any
    String theCommand = theMessage.getNamedValue("COMMAND");
    if ((theCommand == null) || (theCommand.length() == 0)) return;
    
    // See if it's a refresh
    if (theCommand.equalsIgnoreCase("REFRESH")) {
      boolean forceRefresh = theMessage.getBoolNamedValue("ForceRefresh", false);
      XPLScriptManager.getManager().refreshScripts(forceRefresh);
      return;
    }

    // Don't understand anything else
  }
  
}
