/*
 * ClassPathManager.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */

package org.cdp1802.xpl.server;

import java.lang.reflect.Method;
import java.io.*;
import java.util.*;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;

/**
 *
 * Provides a manager for adding class paths to the system class loaded.  Insures
 * that a path is never added twice.
 *
 * @author gduprey
 */
class ClassPathManager {
  private static HashMap<String, URL> classPathList = new HashMap<String, URL>();
  private static URLClassLoader systemClassLoader =  (URLClassLoader) ClassLoader.getSystemClassLoader();
	private static Class sysclass = URLClassLoader.class;
  private static final Class[] parameters = new Class[]{URL.class};

  static void debug(String theMessage) {
    System.err.println("CLS_PATH_MGR:: " + theMessage);
  }

  public static void addSysURL(URL theURL) {
  	try {
	  	Method method = sysclass.getDeclaredMethod("addURL", parameters);
	  	method.setAccessible(true);
		  method.invoke(systemClassLoader,new Object[]{ theURL });
	  } catch (Throwable t) {
  		t.printStackTrace();
		  throw new RuntimeException("Error, could not add URL to system classloader");
	  }
  }
  
  public static boolean ensurePathInClassPath(String theClassPath) {
    // See if path is already loaded
    if (classPathList.get(theClassPath) != null) return true;
    
    // Create a URL of the class path
    URL theURL = null;
    try { 
      theURL = new File(theClassPath).toURI().toURL();
    } catch (Exception urlException) { 
      System.err.println("Class path was bad, module cannot be loaded -- " + urlException.getMessage());
      return false; 
    }

    // Attempt to add it to the system class path
    addSysURL(theURL);

    // Record we did this so we don't repeat it in the future
    classPathList.put(theClassPath, theURL);
    return true;
  }
  
  public static Class findClass(String className) throws ClassNotFoundException {
    return Class.forName(className, true, systemClassLoader);
  }
  
  public static boolean existsInClassPath(String theClassPath) {
    for (URL theURL: systemClassLoader.getURLs()) {      
      if (theURL.getPath().equalsIgnoreCase(theClassPath)) return true;
    }
    
    return false;
  }
  
  public static URL[] enumerateClassPaths() {
    return systemClassLoader.getURLs();
  }
}
