/*
 * DeviceTrackerModule.java
 *
 * Created on September 25, 2005, 10:34 PM
 *
 * Exists to allow tracking of devices for things that need it
 */

package org.cdp1802.xpl.server;

import org.cdp1802.xpl.*;
import org.cdp1802.xpl.tracker.*;

/**
 *
 * @author gerry
 */
public class DeviceTrackerModule extends xPL_Module implements xPL_ManagerListenerI  {
  boolean discoveryRequestSent = false;
  TrackedDeviceManager trackingManager = null;
  
  public DeviceTrackerModule() { }
 
  public void startModule() {
    // Skip out if already started
    if (isModuleStarted()) return;
    
    // Get hook on manager
    if (trackingManager == null) trackingManager = xPL_Manager.getManager().getTrackedDeviceManager();

    // Install our device as the tracking device
    trackingManager.setTrackingDevice(moduleDevice);
    
    // Listen to manager events and change messages
    xPL_Manager.getManager().addManagerListener(this);
  
    // Start tracker
    discoveryRequestSent = false;
    trackingManager.startTracking();
    if (xPL_Manager.getManager().areHubsConfirmed()) {
      discoveryRequestSent = true;
      trackingManager.sendDiscoveryRequest();
    }
    
    // And we are running
    super.startModule();
  }
  
  public void stopModule() {
    // Stop tracker
    xPL_Manager.getManager().getTrackedDeviceManager().stopTracking();

    // Stop device, if any
    xPL_Manager.getManager().removeManagerListener(this);
    super.stopModule();
  }
  
  public void unloadModule() {
    trackingManager = null;
    super.unloadModule();
  }
  
  public void handleXPLManagerEvent(xPL_ManagerEvent theEvent) {
    if (theEvent.getEventCode() == xPL_ManagerEvent.EventCode.ALL_HUBS_DETECTED) {
      if (!discoveryRequestSent) {
        discoveryRequestSent = true;
        trackingManager.sendDiscoveryRequest();
      }
    }
  }
}
