/*
 * TrackedDeviceEvent.java
 *
 * Created on September 21, 2005, 10:01 PM
 *
 */

package org.cdp1802.xpl.tracker;

/**
 * Event describing a change to the list of tracked devices
 *
 * @author gerry
 */
public class TrackedDeviceEvent {
  public enum EventCode { DEVICE_DISCOVERED, DEVICE_SHUTDOWN, DEVICE_TIMEDOUT, 
                          HEARTBEAT_RECEIVED, CONFIG_DEFINITIONS_RECEIVED, CONFIG_VALUES_RECEIVED };
  
  private EventCode eventCode = null;
  private TrackedDevice theDevice = null;
                          
  TrackedDeviceEvent(EventCode theEventCode, TrackedDevice theDevice) { 
    this.eventCode = theEventCode;
    this.theDevice = theDevice;
  }

  /**
   * Return the code describing this event
   *
   * @return event code describing the event
   */
  public EventCode getEventCode() { return eventCode; }
  
  /**
   * Return the device, if any, that is the focus of this
   * event
   *
   * @return device related to this event
   */
  public TrackedDevice getDevice() { return theDevice; }
  
  void releaseResources() {
    eventCode = null;
    theDevice = null;
  }
}
