/*
 * xplIdentifier.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */

package org.cdp1802.xpl;

/**
 * An xPL Identifier
 *
 * @author  gerry
 */
public class xPL_Identifier implements xPL_IdentifierI {
  boolean identifierIsBroadcast = false;
  boolean identifierIsGroup = false;
  
  String vendorID = "";
  String deviceID = "";
  String instanceID = "";
  String groupName = "";
  String identifierKey = null;
  
  // Package private constructor
  xPL_Identifier() { }
  xPL_Identifier(String vendorID, String deviceID, String instanceID) {
    setVendorID(vendorID);
    setDeviceID(deviceID);
    setInstanceID(instanceID);
    setBroadcastDevice(false);
  }
  xPL_Identifier(boolean isBroadcast) {
    setBroadcastDevice(isBroadcast);
  }
  xPL_Identifier(String groupName) {
    setGroupIdentifier(true);
    setGroupName(groupName);
  }
  

  public String getVendorID() { return vendorID; }
  void setVendorID(String newVendorID) {
    if ((newVendorID == null) || (newVendorID.length() == 0)) return;
    vendorID = newVendorID;
    identifierKey = null;
  }

  public String getDeviceID() { return deviceID; }
  void setDeviceID(String newDeviceID) {
    if ((newDeviceID == null) || (newDeviceID.length() == 0)) return;
    deviceID = newDeviceID;
    identifierKey = null;
  }

  public String getInstanceID() { return instanceID; }
  void setInstanceID(String newInstanceID) {
    if ((newInstanceID == null) || (newInstanceID.length() == 0)) return;
    instanceID = newInstanceID;
    identifierKey = null;
  }
  
  void setIdentifier(String newVendorID, String newDeviceID, String newInstanceID) {
    setVendorID(newVendorID);
    setDeviceID(newDeviceID);
    setInstanceID(newInstanceID);
  }

  public boolean isBroadcastIdentifier() { return identifierIsBroadcast;  }
  void setBroadcastDevice(boolean isBroadcast) {
    if (identifierIsBroadcast = isBroadcast) identifierIsGroup = false;
    identifierKey = null;
  }
  
  public boolean isGroupIdentifier() { return identifierIsGroup; }
  void setGroupIdentifier(boolean isGroupDevice) {
    if (identifierIsGroup = isGroupDevice) {
      setVendorID("xpl");
      setDeviceID("group");
      setInstanceID("");
      identifierIsBroadcast = false;
    } else {
      setGroupName("");
    }
  }
  public String getGroupName() { return groupName; }
  void setGroupName(String newGroupName) {
    groupName = newGroupName;
  }
  
  public String toString() {
    // See if we need to build a new string representation
    if (identifierKey == null) {
      // Build a new single string representation
      if (identifierIsBroadcast)
        identifierKey = "*";
      else if (identifierIsGroup)
        identifierKey = vendorID + '-' + deviceID + '.' + groupName;
      else
        identifierKey = vendorID + '-' + deviceID + '.' + instanceID;
    }
    
    return identifierKey;    
  }
  
  public boolean equals(xPL_IdentifierI otherIdentifier) {
    if (otherIdentifier == this) return true;
    if (otherIdentifier == null) return false;
    if (otherIdentifier.isBroadcastIdentifier() != isBroadcastIdentifier()) return false;
    if (otherIdentifier.isGroupIdentifier() != isGroupIdentifier()) return false;
    
    // Broadcast is simple
    if (isBroadcastIdentifier()) return true;
    
    // Check group names if this is a group
    if (isGroupIdentifier()) return otherIdentifier.getGroupName().equalsIgnoreCase(getGroupName());
    
    // Check each part of normal identifier
    if (otherIdentifier.getVendorID().equalsIgnoreCase(getVendorID())
      && otherIdentifier.getDeviceID().equalsIgnoreCase(getDeviceID())
      && otherIdentifier.getInstanceID().equalsIgnoreCase(getInstanceID())) return true;
    
    // No deal
    return false;
  }
  
  /**
   * Release all resources used in this identifiers.  Once invoked, the identifier 
   * cannot be used anymore.  This should not normally be called -- use
   * the IdentifierManager.removeIdentifier() method to properly cleanup.
   */
  public void releaseResources() {
    vendorID = null;
    deviceID = null;
    instanceID = null;
    groupName = null;
    identifierKey = null;
  }
}