/*
 * xplIdentifierI.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */

package org.cdp1802.xpl;

/**
 * Describe a xPL identifier
 *
 * @author  gerry
 */
public interface xPL_IdentifierI {

  /**
   * Return the Vendor ID if this is not a broadcast identifier
   * @return vendor ID or an empty string if this is a broadcast
   */
  public String getVendorID();
  
  /**
   * Return the Device ID if this is not a broadcast identifier
   * @return device ID or an empty string if this is a broadcast
   */
  public String getDeviceID();
  
  /**
   * Return the instance ID if this is not a broadcast identifier.
   *
   * @return instance ID or an empty string if this is a broadcast or group
   */
  public String getInstanceID();
  
  /**
   * Return true if this is a broadcast identifier or false if 
   * it is specific to a target
   *
   * @return true if this is a broadcast identifier
   */
  public boolean isBroadcastIdentifier();
  
  /**
   * Return true if this is a representation of a group
   * target.  That means the vendor ID will be fixed at
   * XPL, the device ID fixed as GROUP and the instance ID
   * is the targetd group.  Access the group name via
   * getGroupName() (not getInstanceID()).
   *
   * @return true if this is a group targeted identifier
   */
  public boolean isGroupIdentifier();
  
  /**
   * Return the group name when this is a targeted group identfier.
   *
   * @return group name for a group targeted identifier or an empty string if broadcast or non-group
   */
  public String getGroupName();

  /**
   * Return a string format of this identifer in standard xPL
   * format.  For wildcards, a single * is returned.
   * @return printable version of this identifier
   */
  public String toString();
  
  /** 
   * Return true if the passed identifier describes the same identifier as
   * this one (even if they are not the same instance)
   *
   * @return true if the passed identifier describes the same identifier as this instance does
   */
  public boolean equals(xPL_IdentifierI otherIdentifier);
  
  /**
   * Release all resources used in this identifier.  Once invoked, the identifier 
   * cannot be used anymore.  This should not normally be called -- use
   * the IdentiferManager.removeIdentifier() method to properly cleanup.
   */
  public void releaseResources();
}
