/*
 * xPL_MutableMessage.java
 * Copyright 2004, Gerald R Duprey Jr
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License. 
 */

package org.cdp1802.xpl;

/**
 *
 * @author  gerry
 */
public class xPL_MutableMessage extends xPL_Message implements xPL_MutableMessageI {
  xPL_MutableMessage() {
    super();
    namedValues = new MutableNamedValues();
  }
  
  public void setType(MessageType newMessageType ) {
    msgType = newMessageType;
  }
  
  public void setType(String newMessageType) {
    if (newMessageType.equalsIgnoreCase("xpl-stat"))
      msgType = MessageType.STATUS;
    else if (newMessageType.equalsIgnoreCase("xpl-trig"))
      msgType = MessageType.TRIGGER;
    else if (newMessageType.equalsIgnoreCase("xpl-cmnd"))
      msgType = MessageType.COMMAND;
    else
      msgType = MessageType.UNKNOWN;
  }
  
  public void setSchemaClass(String newSchemaClass) {
    if (newSchemaClass == null)
      schemaClass = "";
    else
      schemaClass = newSchemaClass;
  }

  public void setSchemaType(String newSchemaType) {
    if (newSchemaType == null)
      schemaType = "";
    else
      schemaType = newSchemaType;
  }
  
  public void setSchema(String newSchemaClass, String newSchemaType) {
    schemaClass = newSchemaClass;
    schemaType = newSchemaType;
  }

  public void setSchema(String newSchemaInfo) {
    int periodPtr = newSchemaInfo.indexOf('.');
    if (periodPtr == -1) return;
    schemaClass = newSchemaInfo.substring(0, periodPtr);
    schemaType = newSchemaInfo.substring(periodPtr + 1);
  }
  
  public void setSource(xPL_IdentifierI newSourceIdentifier) {
    msgSource = newSourceIdentifier;
  }
  
  public void setSource(String newSourceIdentifier) {
    msgSource = xPL_Manager.getManager().getIdentifierManager().parseNamedIdentifier(newSourceIdentifier);
  }

  public void setTarget(xPL_IdentifierI newTargetIdentifier) {
    msgTarget = newTargetIdentifier;
  }
  
  public void setTarget(String newTargetIdentifier) {
    msgTarget = xPL_Manager.getManager().getIdentifierManager().parseNamedIdentifier(newTargetIdentifier);
  }
  
  public void setMessageBody(NamedValuesI newNamedValues) {
    if (newNamedValues == null) return;
    namedValues = newNamedValues;
  }
  
  public MutableNamedValuesI getMutableMessageBody() {
    // Make sure what we have really is mutable
    if (!(namedValues instanceof MutableNamedValuesI)) return null;
    return (MutableNamedValuesI) namedValues;
  }
  
  public MutableNamedValueI addNamedValue(String theName, boolean theValue) {
    MutableNamedValueI theNamedValue = null;
    if (namedValues instanceof MutableNamedValues) theNamedValue = ((MutableNamedValues) namedValues).addNamedValue(theName, theValue);
    return theNamedValue;
  }

  public MutableNamedValueI addNamedValue(String theName, int theValue) {
    MutableNamedValueI theNamedValue = null;
    if (namedValues instanceof MutableNamedValues) theNamedValue = ((MutableNamedValues) namedValues).addNamedValue(theName, theValue);
    return theNamedValue;
  }

  public MutableNamedValueI addNamedValue(String theName, String theValue) {
    MutableNamedValueI theNamedValue = null;
    if (namedValues instanceof MutableNamedValues) theNamedValue = ((MutableNamedValues) namedValues).addNamedValue(theName, theValue);
    return theNamedValue;
  }

  public MutableNamedValueI setNamedValue(String theName, String theValue) {
    MutableNamedValueI theNamedValue = null;
    if (namedValues instanceof MutableNamedValues) theNamedValue = ((MutableNamedValues) namedValues).setNamedValue(theName, theValue);
    return theNamedValue;
  }
  
  public MutableNamedValueI setNamedValue(String theName, int theValue) {
    MutableNamedValueI theNamedValue = null;
    if (namedValues instanceof MutableNamedValues) theNamedValue = ((MutableNamedValues) namedValues).setNamedValue(theName, theValue);
    return theNamedValue;
  }
  
  public MutableNamedValueI setNamedValue(String theName, boolean theValue) {
    MutableNamedValueI theNamedValue = null;
    if (namedValues instanceof MutableNamedValues) theNamedValue = ((MutableNamedValues) namedValues).setNamedValue(theName, theValue);
    return theNamedValue;
  }

  public void clearMessageBody() {
    if (namedValues instanceof MutableNamedValues) ((MutableNamedValues) namedValues).clear();
  }

  public int removeNamedValues(String theName) {
    int removedCount = 0;
    if (namedValues instanceof MutableNamedValues) removedCount = ((MutableNamedValues) namedValues).removeNamedValues(theName);
    return removedCount;
  }
}
